#!/bin/sh

if [ "`id -u`" -ne 0 ]; then
	exit 0
fi

CERT_FILE="/opt/psa/admin/conf/httpsd.pem"
CA_FILE="/opt/psa/admin/conf/rootchain.pem"
OPENSSL_CNF="/opt/psa/admin/conf/openssl.cnf"
CIPHER_FILE="/opt/psa/admin/conf/cipher.lst"

if
	grep -q "CERTIFICATE" "$CERT_FILE" \
	|| ( echo "US
Virginia
Herndon
Parallels
Parallels Panel
Parallels Panel
info@parallels.com


" | openssl req -x509 -nodes -days 365 -newkey rsa:2048 \
			-config "$OPENSSL_CNF" -set_serial `date +%s` -keyout "${CERT_FILE}_" \
			-out "${CERT_FILE}_" \
		&& cat "${CERT_FILE}_" | sed -e 's/^\(-----END.*\)/\1\
/' > "$CERT_FILE" \
		&& rm -f "${CERT_FILE}_" ) >&2
then
	echo 'ssl.engine = "enable"'
	echo 'ssl.use-sslv2 = "disable"'
	if [ -s "$CIPHER_FILE" ]; then
		echo "ssl.cipher-list = \"`cat $CIPHER_FILE`\""
	fi
	echo "ssl.pemfile = \"$CERT_FILE\""
	if [ -s "$CA_FILE" ]; then
		echo "ssl.ca-file = \"$CA_FILE\""
	fi
fi

case "`uname -s`" in
Linux) hostname=`hostname -f`;;
FreeBSD) hostname=`hostname`;;
*) hostname=;;
esac

if [ -n "$hostname" ]; then
	echo "ssl.plain-redirect = \"https://$hostname:8443/\""
fi
