/* JavaScript Widgets */

Jsw = {

    version: '1.0',
    baseUrl: '',
    showErrorDetails: false,

    _registredComponents: null,
    _initOnReady: false,
    _firedOnReady: false,
    _onReadyEvents: null,

    registerComponent: function(identity, component) {
        if (!Jsw._registredComponents) {
            Jsw._registredComponents = new Hash();
        }

        Jsw._registredComponents.set(identity, component);
    },

    getComponent: function(identity) {
        return Jsw._registredComponents.get(identity);
    },

    clearStatusMessages: function() {
        var contentElement = $('content');

        ['.msg-error', '.msg-info', '.msg-warning'].each(function(msgClass) {
            contentElement.select(msgClass).each(function(messageElement) {
                messageElement.remove();
            });
        });
    },

    addStatusMessage: function(type, message, params) {
        var params = ('undefined' != typeof params) ? params : {};

        var elementParams = { 'class': 'msg-box msg-'  + type };
        if (params['id']) {
            elementParams['id'] = params['id'];
        }

        var statusElement = new Element('div', elementParams);
        statusElement.update(
            '<div><div><div><div><div>' +
            '<div class="msg-content">' + message + '</div>' +
            '</div></div></div></div></div>'
        );

        if (params['closable']) {
            var closeElement = new Element('span', { 'class': 'close' });
            closeElement.observe('click', params['onClose']);
            statusElement.down('.msg-content').insert({bottom: closeElement});
        }

        try {
        	var target;
        	if (params['renderTo']) {
        		target = params['renderTo'];
        	} else {
        		target = 'main';
        	}
        	var mode = 'top';
        	if (params['renderMode']) {
        		mode = params['renderMode'];
        	}
            var insertion = {};
            insertion[mode] = statusElement;

        	$(target).insert(insertion);
        } catch (e) {
            alert(type + ' | ' + message);
        }
    },

    onReady: function(callback) {
        if (self._firedOnReady) {
            callback();
            return;
        }
        if (!self._initOnReady) {
            Event.observe(window, 'load', Jsw._fireOnReady);
            self._initOnReady = true;
            self._onReadyEvents = new Array();
        }

        self._onReadyEvents.push(callback);
    },

    _fireOnReady: function() {
        self._onReadyEvents.each(function(callback) {
            callback();
        });

        self._firedOnReady = true;
    },

    namespace: function(ns) {
        var nsParts = ns.split('.');
        var root = window;

        for (var partIndex = 0; partIndex < nsParts.length; partIndex++) {
            if ('undefined' == typeof root[nsParts[partIndex]]) {
                root[nsParts[partIndex]] = {};
            }

            root = root[nsParts[partIndex]];
        }
    },

    prepareUrl: function(url) {
        if ('undefined' == typeof url || null === url) {
            return null;
        }

        if (
    			url.startsWith('/plesk/') ||
    			(Jsw.baseUrl != '' && url.startsWith(Jsw.baseUrl)) ||
        		!url.startsWith('/')
		) {
            return url;
        }

        return Jsw.baseUrl + url;
    },

    clonePosition: function(element, source, options) {
        // workaround for: https://prototype.lighthouseapp.com/projects/8886/tickets/90-cloneposition-problem-in-ie7

        try {
            element.clonePosition(source, options);
        } catch (e) {
            element.clonePosition(source, options);
        }
    },

    showInternalError: function(message) {
        Jsw.clearStatusMessages();
        message = 'Internal error.' + (Jsw.showErrorDetails ? ("<br/>" + message) : '');
        Jsw.addStatusMessage('error', message);
    }

};

// Monkey patching Prototype for proper handling of session expiration
Ajax.PatchedRequest = Class.create(Ajax.Request, {
    initialize: function($super, url, options) {
        options = Object.extend({
            on400: function() {
                window.location.reload();
            },
            on500: function(transport) {
                Jsw.showInternalError(transport.responseText);
            }
        }, options || {});

        $super(url, options);
    }
});

Ajax.Request.prototype = Ajax.PatchedRequest.prototype;

// cross-site request forgery protection
(function() {
    Ajax.Base.prototype.initialize = Ajax.Base.prototype.initialize.wrap(function() {
        var args = $A(arguments), proceed = args.shift();
        args[0] = args[0] || {};
        var options = args[0];
        if (!(options.method && options.method.toUpperCase() == "GET") && $('forgery_protection_token')) {
            var encodedToken = encodeURIComponent($('forgery_protection_token').content);
            if (Object.isString(options.parameters)) {
                if (options.parameters.indexOf("forgery_protection_token") == -1) {
                    options.parameters += '&forgery_protection_token=' + encodedToken;
                }
            } else if (Object.isHash(options.parameters)) {
                options.parameters = options.parameters.toObject();
                options.parameters.forgery_protection_token = encodedToken;
            } else {
                options.parameters = options.parameters || {};
                options.parameters.forgery_protection_token = encodedToken;
            }
        }
        proceed.apply(null, args);
    });
})();

Jsw.Component = Class.create({

    _tag: 'div',

    initialize: function(config) {
        this._initConfiguration(config);
        this._registerComponent();

        this._initComponentElement();

        if (this._autoRender && ((null != $(this._renderTargetId)) || (null != $(this._applyTargetId)))) {
            this.render();
        }
    },

    _initConfiguration: function(config) {
        this.initialConfig = config;
        this._config = this.initialConfig;

        this._cls = this._getConfigParam('cls', '');
        this._wrapperClass = this._getConfigParam('wrapperClass', '');
        this._autoRender = this._getConfigParam('autoRender', true);
        this._renderTargetId = this._getConfigParam('renderTo', null);
        this._applyTargetId = this._getConfigParam('applyTo', null);

        this._id = (this._applyTargetId)
            ? this._applyTargetId
            : this._getConfigParam('id', null);

        if (null === this._id) {
            this._id = this._getRandomId();
            this._idIsRandom = true;
        }

        this._tag = (this._applyTargetId)
            ? $(this._applyTargetId).tagName
            : this._getConfigParam('tag', this._tag);
    },

    _registerComponent: function() {
        Jsw.registerComponent(this._id, this);
    },

    _initComponentElement: function() {
        if (this._applyTargetId) {
            this._componentElement = $(this._applyTargetId);
            return ;
        }

        this._componentElement = new Element(this._tag, {
            'id': this._id,
            'class': this._cls
        });
    },

    _initDisablerOverlay: function() {
        this._disablerOverlay = new Element('div');

        this._disablerOverlay.setStyle({
            display: 'none',
            zIndex: this._componentElement.style.zIndex + 1000
        });

        this._disablerOverlay.addClassName('content-blocker');
        this._componentElement.appendChild(this._disablerOverlay);
    },

    addEventObserver: function(eventName, handler) {
        if ('undefined' != typeof handler) {
            this._componentElement.observe(eventName, handler);
        }
    },

    removeEventObserver: function(eventName, handler) {
        if ('undefined' != typeof handler) {
            this._componentElement.stopObserving(eventName, handler);
        }
    },

    _hasConfigParam: function(name)
    {
        return 'undefined' != typeof this._config[name];
    },

    _getConfigParam: function(name, defaultValue) {
        return this._hasConfigParam(name)
            ? this._config[name]
            : defaultValue;
    },

    _getRandomId: function() {
        var id = 'gen-id-' + this._getRandomNumber();

        while(null != $(id)) {
            id = 'gen-id-' + this._getRandomNumber();
        }

        return id;
    },

    _getRandomNumber: function() {
        return Math.floor(1000000*Math.random());
    },

    _addEvents: function() {
    },

    _addTooltips: function() {
        var description = this._getConfigParam('description');
        if  (description) {
            this._tooltip = Jsw.Tooltip.init(this._componentElement, {text: description});
        }
    },

    _updateComponentElement: function(content) {
        if (this._componentElement.empty()) {
            this._componentElement.update(content);
            return ;
        }

        var oldElements = this._componentElement.childElements();

        Element.insert(this._componentElement.firstChild, {before: content});

        oldElements.each(function(oldElement) {
            if (
                'undefined' == typeof oldElement.id ||
                '' == oldElement.id
            ) {
                return ;
            }

            var newElement = this._componentElement.select('#' + oldElement.id).first();

            if (newElement) {
                newElement.replace(oldElement);
            }
        }, this);
    },

    getWrapperClass: function() {
        return this._wrapperClass;
    },

    getComponentHtml: function() {
        var wrapper = new Element('span');
        wrapper.update(this._componentElement);

        return wrapper.innerHTML;
    },

    getId: function() {
        return this._id;
    },

    setRenderTarget: function(targetId) {
        this._renderTargetId = targetId;
    },

    getRenderTarget: function() {
        return this._renderTargetId;
    },

    show: function() {
        this._componentElement.show();
    },

    hide: function() {
        this._componentElement.hide();
    },

    render: function() {
        if ($(this._renderTargetId)) {
            var insertion = {};
            insertion[this._getConfigParam('renderMode', 'bottom')] = this._componentElement;
            $(this._renderTargetId).insert(insertion);
        }

        this._addEvents();
        this._addTooltips();
    }

});

Jsw.Hint = Class.create(Jsw.Component, {
    _tag: 'p',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            this._getConfigParam('hint', '')
        );
    }
});

Jsw.Button = Class.create(Jsw.Component, {

    _tag: 'a',

    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
        this._cls = this._getConfigParam('cls', 'b-btn');
        this._disabledClass = this._getConfigParam('disabledClass', 'btn-disabled');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<i><i><i><span>'
            + this._config['title']
            + '</span></i></i></i>'
        );

        this._initVisibility();
    },

    _initVisibility: function() {
        this._disabled = this._getConfigParam('disabled', false);

        if (this._disabled) {
            this._disabled = false;
            this.disable();
        } else {
            this._disabled = true;
            this.enable();
        }
    },

    _addAttribute: function(name, value) {
        if ('undefined' != typeof value) {
            this._componentElement.writeAttribute(name, value);
        }
    },

    setTooltip : function(tooltipText) {
        if (this._tooltip) {
            this._tooltip.setText(tooltipText);
        } else {
            this._tooltip = Jsw.Tooltip.init(this._componentElement, {text: tooltipText});
        }
    },

    enable: function() {
        if (!this._disabled) {
            return ;
        }
        this._disabled = false;

        var classNames = this._config['addCls'] || '';
        if (!Object.isArray(classNames)) {
            classNames = [classNames];
        }
        classNames.each(function(item) {
            this._componentElement.addClassName(item);
        }, this);
        classNames.each(function(item) {
            this._componentElement.removeClassName(item + '-disabled');
        }, this);
        this._componentElement.removeClassName(this._disabledClass);

        this._addAttribute('href', Jsw.prepareUrl(this._getConfigParam('href', '#')));
        this._addAttribute('target', this._getConfigParam('target', null));
        if (this._handler) {
            this.addEventObserver('click', this._handler);
        }
    },

    disable: function() {
        if (this._disabled) {
            return ;
        }
        this._disabled = true;

        this._componentElement.addClassName(this._disabledClass);

        var classNames = this._config['addCls'];
        if (!Object.isArray(classNames)) {
            classNames = [classNames];
        }
        classNames.each(function(item) {
            this._componentElement.addClassName(item + '-disabled');
        }, this);
        classNames.each(function(item) {
            this._componentElement.removeClassName(item);
        }, this);
        this._componentElement.writeAttribute('href', null);
        this._componentElement.writeAttribute('target', null);
        if (this._handler) {
            this.removeEventObserver('click', this._handler);
        }
    }
});

Jsw.SmallButton = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 's-btn');
    }

});

Jsw.BigButton = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._icon = this._getConfigParam('icon', null);
    },

    _initComponentElement: function() {
        this._componentElement = new Element('a').insert(
            '<span class="db-content">' +
                '<span class="db-icon"><img src="' + this._icon + '"></span>' +
                '<span class="db-name">' + this._config['title'] + '</span>' +
                '<span class="db-info">' + this._config['description'] + '</span>' +
            '</span>'
        );

        this._initVisibility();
    }
});

Jsw.submit = function(element) {
    $(element).up("form").submit();
};

Jsw.redirect = function(action) {
    $(document).location.href = Jsw.prepareUrl(action);
};

Jsw.redirectPost = function(action, data, target) {
    Jsw.FormRedirect.redirect(Jsw.prepareUrl(action), 'post', data, target);
};

Jsw.FormRedirect = {
    redirect: function(action, method, data, target) {

    // workaround for http://code.google.com/p/chromium/issues/detail?id=10671
    var randomId = Math.floor(1000000*Math.random());
    action = action.include('?')
        ? (action + '&_randomId=' + randomId)
        : (action + '?_randomId=' + randomId);

        var form = new Element(
            'form',
            {
                method: method,
                action: action
            }
        );
        if (target) {
            form.target = target;
        }
        if (data) {
            var elements = Jsw.FormRedirect.subToElements(data);
            $A(elements).each(function (element) {
                form.insert(element);
            });
        }

        if ($('forgery_protection_token')) {
            form.insert(new Element('input', {
                type: 'hidden',
                name: 'forgery_protection_token',
                value: $('forgery_protection_token').content
            }));
        }

        $(document.body).insert(form);
        form.submit();
    },

    stringToElements: function(value, prefix)
    {
        return [new Element('input', {
            type: 'hidden',
            name: prefix,
            value: value
        })];
    },

    booleanToElements: function(value, prefix)
    {
        return [new Element('input', {
            type: 'hidden',
            name: prefix,
            value: value ? 1 : 0
        })];
    },

    arrayToElements: function(data, prefix)
    {
        var elements = [];
        $A(data).each(function(item) {
            elements = elements.concat(Jsw.FormRedirect.subToElements(item, '', prefix));
        });
        return elements;
    },

    hashToElements: function(data, prefix)
    {
        var elements = [];
        $H(data).each(function(pair) {
            elements = elements.concat(Jsw.FormRedirect.subToElements(pair.value, pair.key, prefix));
        });
        return elements;
    },

    subToElements: function(value, key, prefix)
    {
        var elements = [];
        var name = prefix
            ? prefix + '[' + key + ']'
            : key;
        if (Object.isString(value) || Object.isNumber(value)) {
            elements = elements.concat(Jsw.FormRedirect.stringToElements(value, name));
        } else
        if ('boolean' == typeof value) {
            elements = elements.concat(Jsw.FormRedirect.booleanToElements(value, name));
        } else
        if(Object.isArray(value)) {
            elements = elements.concat(Jsw.FormRedirect.arrayToElements(value, name));
        } else {
            elements = elements.concat(Jsw.FormRedirect.hashToElements(value, name));
        }
        return elements;
    }
}

Jsw.CommandButton = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            new Element('button', {
                type: 'button',
                value: this._config['value'],
                name: this._config['name']
            }).update(this._config['title'])
        );

        this._initVisibility();
    },

    _initVisibility: function() {
        this._disabled = this._getConfigParam('disabled', false);

        if (this._disabled) {
            this._disabled = false;
            this.disable();
        } else {
            this._disabled = true;
            this.enable();
        }
    },

    enable: function() {
        if (!this._disabled) {
            return ;
        }

        this._disabled = false;
        if ('send' == this._config['name']) {
            this._componentElement.addClassName('action');
        }
        this._componentElement.removeClassName('disabled');
        this._componentElement.disabled = false;
        if (this._handler) {
            this.addEventObserver('click', this._handler);
        }
    },

    disable: function() {
        if (this._disabled) {
            return ;
        }

        this._disabled = true;
        this._componentElement.disabled = true;
        if ('send' == this._config['name']) {
            this._componentElement.removeClassName('action');
        }
        this._componentElement.addClassName('disabled');
        if (this._handler) {
            this.removeEventObserver('click', this._handler);
        }
    }
});

Jsw.namespace('Jsw.form');

Jsw.form.DisplayField = Class.create(Jsw.Component, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'form-row');
        this._valueRenderer = this._getConfigParam('valueRenderer', null);
    },

    _initComponentElement: function($super) {
        $super();

        var value = ('function' == typeof this._valueRenderer)
            ? this._valueRenderer()
            : this._getConfigParam('value', '');

        this._componentElement.update(
            '<div class="field-name">' + this._getConfigParam('fieldLabel', '') + '</div>' +
            '<div class="field-value">' +
                '<div class="text-value">' + value + '</div>' +
            '</div>'
        );
    }
});

Jsw.FormAjax = Class.create(Jsw.Component, {
    _sendButtonTitle: '',

    _initConfiguration: function($super, config) {
        $super(config);
        this._sendButtonId = this._getConfigParam('sendButtonId', 'btn-send');
        this._cancelButtonId = this._getConfigParam('cancelButtonId', 'btn-cancel');
    },

    disable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            var sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = true;
            sendButtonWrapper.addClassName('disabled');
            if (!this._sendButtonTitle) {
                this._sendButtonTitle = sendButton.innerHTML;
            }
            sendButton.update('<span class="wait">' + this._config['waitButtonTitle'] + '</span>');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = true;
            cancelButtonWrapper.addClassName('disabled');
        }
    },

    enable: function() {
        var sendButtonWrapper = $(this._sendButtonId);
        if (sendButtonWrapper) {
            sendButton = sendButtonWrapper.down('button');
            sendButton.disabled = false;
            sendButton.update(this._sendButtonTitle);
            sendButtonWrapper.removeClassName('disabled');
        }

        var cancelButtonWrapper = $(this._cancelButtonId);
        if (cancelButtonWrapper) {
            cancelButtonWrapper.down('button').disabled = false;
            cancelButtonWrapper.removeClassName('disabled');
        }
    },

    _initComponentElement: function($super) {
        this._componentElement = $(this._id);
        this._initDisablerOverlay();

        var firstElement = this._componentElement.findFirstElement();

        if ('undefined' != typeof firstElement) {
            try {
                firstElement.focus();
            } catch (e) {}
        }

        this._submitInProgress = false;
    },

    _addEvents: function() {
        this._componentElement._formSubmit = this._componentElement.submit;
        this._componentElement.submit = this._onSubmit.bind(this);
        this._componentElement.observe('submit', this._onSubmitEvent.bind(this));
        this._addChoiceRadioButtonsOnClickEvent();
    },

    _addChoiceRadioButtonsOnClickEvent: function() {
        $$('input[type="radio"]').each(function(element) {
            if (!$(element.id).up('div.choice-block')) {
                return;
            }
            element.observe('click', function() {
                $(this.id).up('div.choice-block').select('span').each(function(element) {
                    element.removeClassName('selected');

                    if ('0' == $(this.id).value) {
                        element.removeClassName('no');
                    }
                }, this);

                $(this.id).up('span').addClassName('selected');

                if ('0' == $(this.id).value) {
                    $(this.id).up('span').addClassName('no');
                }
            });
        });
    },

    _onSubmitEvent: function(event) {
        this._onSubmit();
        Event.stop(event);
        return false;
    },

    _onSubmit: function() {
        if (this._submitInProgress) {
            return false;
        } else {
            this._submitInProgress = true;
        }

        this.disable();

        if (this._componentElement.enctype == "multipart/form-data") {
            this._componentElement._formSubmit();
            return true;
        }

        var actionUrl = this._componentElement.action;

        if (!actionUrl) {
            actionUrl = $(document).location.href;
        }

        // remove hash symbol (and the following) cause this will encoded and lead to problems
        actionUrl = actionUrl.replace(/#.*$/, "");

        var params = this._componentElement.serialize();
        var context = this;
        var request = new Ajax.Request(
            actionUrl,
            {
                method:'post',
                parameters: params,
                onSuccess: this._onSuccess.bind(this),
                onFailure: this._onFailure.bind(this)
            }
        );
    },

    _onFailure: function(req) {
        this._submitInProgress = false;
        this._clearMessages();
        Jsw.showInternalError(req.responseText);
        this.enable();
    },

    _onSuccess: function(req) {
        this._submitInProgress = false;
        Jsw.clearStatusMessages();

        try {
            var response = req.responseText.evalJSON();
            this._processForm(response);
        } catch (e) {
            this._clearMessages();
            this.enable();

            // show error, if request wasn't aborted by user
            if (0 != req.status) {
                Jsw.showInternalError(req.responseText);
            }
        }
    },

    _processForm: function(response) {
        this._processResponseRedirect(response);

        if (!response.redirect) {
            this._clearMessages();
            this._processResponseStatus(response.status);
            this._processResponseStatusMessages(response.statusMessages);
            this._processResponseFormMessages(response.formMessages);

            if (this._hasFieldErrors) {
                this._showFieldErrorArea();
            }
            this.enable();
        }
    },

    _showFieldErrorArea: function() {
        var errorsElements = this._componentElement.select('.field-errors');
        var firstError = null;

        errorsElements.each(function(errorElement) {
            if (errorElement.visible()) {
                firstError = errorElement;
            }
        });

        var hiddenContainerId = null;

        firstError.ancestors().each(function(element) {
            if (!element.visible()) {
                hiddenContainerId = element.id;
            }
        });

        if (!hiddenContainerId) {
            return ;
        }

        // in case of tabbed form we must switch to corresponding tab
        var tabsBar = Jsw.getComponent('form-tab-buttons');

        if (tabsBar) {
            tabsBar.switchTab(hiddenContainerId);
        }
    },

    _processResponseStatus: function (status) {
        if ('success' != status) {
            this._hasErrors = true;
        }
    },

    _clearMessages: function() {
        //clear field messages
        try {
            this._componentElement.select('.field-errors').each(function(errors){
                errors.hide();
                errors.up('.form-row').removeClassName('error');
                errors.select('.error-hint').each(function(error) {
                    error.remove();
                });
            });
        } catch (e) {
        }

        this._hasErrors = false;
        this._hasFieldErrors = false;
    },

    _processResponseStatusMessages: function (messages) {
        $A(messages).each(function(value) {
            this._addFormMessage(value.status, value.content);
        }, this);
    },

    _processResponseFormMessages: function (messages) {
        this._processFieldMessages(messages, []);
    },

    _addFieldMessage: function(errors, message) {
        errors.up('.form-row').addClassName('error');
        errors.insert({bottom: '<span class="error-hint">' + message.escapeHTML() + '</span>' });
        errors.show();
    },

    _processFieldMessage: function(prefix, name, message) {
        var fieldErrors = this._componentElement.down('#' + prefix.join('-')).up('.form-row').down('.field-errors');
        this._addFieldMessage(fieldErrors, message);
        this._hasFieldErrors = true;
    },

    _processFieldMessages: function(messages, prefix) {
        if (Object.isArray(messages)) {
            $A(messages).each(function(message) {
                if (Object.isString(message)) {
                    this._processFieldMessage(prefix, 'error', message);
                } else {
                    prefix.push(name);
                    this._processFieldMessages(message, prefix);
                    prefix.pop();
                }
            }, this);
        } else {
            $H(messages).each(function(pair) {
                if (Object.isString(pair.value)) {
                    this._processFieldMessage(prefix, pair.key, pair.value);
                } else {
                    prefix.push(pair.key);
                    this._processFieldMessages(pair.value, prefix);
                    prefix.pop();
                }
            }, this);
        }
    },

    _addFormMessage: function (type, message) {
        Jsw.addStatusMessage(type, message);
    },

    _processResponseRedirect: function (response) {
        if (response.redirect) {
            if (response.postData) {
                Jsw.redirectPost(response.redirect, response.postData, response.target);
            } else {
                Jsw.redirect(response.redirect);
            }
        }
    }

});

Jsw.FormSimple = Class.create(Jsw.FormAjax, {
    _processResponseFormMessages: function (messages) {
        this._messages = '';
        this._processFieldMessages(messages, []);
        if (this._messages) {
            this._addFormMessage('error', this._messages);
        }
    },

    _processFieldMessage: function(prefix, name, message) {
        if (!this._hasErrors) {
            try {
                this._componentElement.down('#' + prefix.join('-')).focus()
            } catch (e) {}
        }
        this._hasErrors = true;
        this._messages += message + '<br/>';
    },

    _addFormMessage: function (type, message) {
        try {
            this._componentElement.down('.content-area').insert({top: '<div class=\"msg-box msg-' + type +
                '\"><div><div><div><div><div><div class=\"msg-content\">' + message + '</div></div></div></div></div></div></div>'});
        } catch (e) {
            alert(type + ' | ' + message);
        }
    }
});

Jsw.Container = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id;

        this._initItems();
    },

    _initItems: function() {
        this._items = this._getConfigParam('items', null);

        if (null != this._items) {
            this._items.each(function(item, index) {
                if (item && ('undefined' != typeof item.componentType)) {
                    // recreate proper object defined by componentType
                    eval('var resultItem = new ' + item.componentType + '(item);');
                    this._items[index] = resultItem;
                }
            }, this);
        }
    },

    _renderItems: function() {
        if (null != this._items) {
            this._items.each(function(item) {
                if (item) {
                    this._renderItem(item);
                }
            }, this);
        }
    },

    _renderItem: function(item) {
        item.setRenderTarget(this._contentAreaId);
        item.render();
    },

    render: function($super) {
        $super();

        this._renderItems();
    },

    getItems: function() {
        return this._items;
    }

});

Jsw.ListContainer = Class.create(Jsw.Container, {

    _tag: 'ul',

    _renderItem: function(item) {
        var renderTargetId = null;
        var isEmptyItem = Object.isArray(item) && !item.size();

        if (!isEmptyItem) {
            renderTargetId = this._id + '-item-' + item.getId();
        }

        var cls = this._getConfigParam('itemClass', '');

        if (this._getConfigParam('renderLastItemClass', '') && this._items.last() == item) {
            cls = 'last';
        }

        if (isEmptyItem) {
            cls += ' ' + this._getConfigParam('emptyItemClass', '');
        }

        if (cls) {
            cls = ' class="' + cls + '"';
        }

        $(this._contentAreaId).insert('<li ' + (renderTargetId ? 'id="' + renderTargetId + '"' : '') + cls + '></li>');

        if (!isEmptyItem) {
            item.setRenderTarget(renderTargetId);
            item.render();
        }
    }

});

Jsw.namespace('Jsw.layout');

Jsw.layout.TwoColumns = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'two-cols-block clearfix');
        this._columnCls = 'first-col';
    },

    _initComponentElement: function() {
        this._componentElement = new Element('div', {
                'class': this._cls
            }).update(
                '<div id="' + this._contentAreaId + '" class="columns-wrapper">' +
                '</div>'
            );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(this._contentAreaId).insert(
            '<div class="' + this._columnCls + '"><div class="column-box" id="' + renderTargetId +'"></div></div>'
        );
        item.setRenderTarget(renderTargetId);
        item.render();

        this._columnCls = 'second-col';
    }

});

Jsw.Dashboard = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._buttonsAreaId =this._id + '-buttons-area';
    },

    _initComponentElement: function() {
        this._componentElement = new Element('div', {
                'class': this._cls
            }).update(
                '<ul id="' + this._buttonsAreaId + '" class="d-buttons">' +
                    this._getConfigParam('html', '') +
                '</ul>'
            );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-button-' + item.getId();
        $(this._buttonsAreaId).insert('<li id="' + renderTargetId +'"></li>');
        item.setRenderTarget(renderTargetId);
        item.render();
    }

});

Jsw.Panel = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
        this._titleClass = this._getConfigParam('titleClass', '');
        this._hideContentTitle = this._getConfigParam('hideContentTitle', this._title);
        this._titleAreaId = this._id + '-title-area';
        this._contentAreaId = this._id + '-content-area';
        this._boxItemsAreaId = this._id + '-box-area';
        this._initBoxItems();
    },

    _initBoxItems: function() {
        this._boxItems = this._getConfigParam('boxItems', null);

        if (null != this._boxItems) {
            this._boxItems.each(function(item, index) {
                if ('undefined' != typeof item.componentType) {
                    // recreate proper object defined by componentType
                    eval('var resultItem = new ' + item.componentType + '(item);');
                    this._boxItems[index] = resultItem;
                }
            }, this);
        }
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="box-area">' +
            this._getTitleHtml() +
            this._getSubTitleHtml() +
                '<div id="' + this._boxItemsAreaId + '"></div>' +
                '<div class="content">' +
                    '<div class="content-area">' +
                        '<div class="content-wrapper" id="' + this._contentAreaId + '">' +
                            this._getConfigParam('html', '') +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );
    },

    _getTitleHtml: function() {
        return this._title
            ?   '<div class="title ' + (this._titleClass ? this._titleClass : '') + '">' +
                    '<div class="title-area" id="' + this._titleAreaId + '">' +
                        (('-' == this._title) ? '' : '<h3><span>' + this._title + '</span></h3>') +
                    '</div>' +
                '</div>'
            : '';
    },

    _getSubTitleHtml: function() {
    	return '';
    },

    render: function($super) {
        $super();

        this._renderBoxItems();
    },

    _renderBoxItems: function() {
        if (null != this._boxItems) {
            this._boxItems.each(function(item) {
                this._renderBoxItem(item);
            }, this);
        }
    },

    _renderBoxItem: function(item) {
        item.setRenderTarget(this._boxItemsAreaId);
        item.render();
    },

    _isHidden: function()
    {
        var rememberedStatus = Jsw.Cookie.get(this._id + '-hidden');
        if (this._idIsRandom || null === rememberedStatus) {
            return this._getConfigParam('hidden', false);
        }
        return 1 == rememberedStatus;
    },

    _setHidden: function(hidden)
    {
        if (this._getConfigParam('hidden', false) == hidden) {
            Jsw.Cookie.remove(this._id + '-hidden');
        } else {
            Jsw.Cookie.set(this._id + '-hidden', hidden ? 1 : 0);
        }
    },

    _addEvents: function($super) {
        $super();
        if (this._isHidden()) {
            this.hideContent();
        } else {
            this.showContent();
        }
        this._componentElement.select('#' + this._titleAreaId).each(function(titleAreaElement) {
            Event.observe(titleAreaElement, 'click', this.toggleContent.bindAsEventListener(this));
        }, this);
    },

    _updateTitle: function(title) {
        if (title && ('-' != title)) {
            this._componentElement.down('div.title-area > h3 > span').update(title);
        }
    },

    showContent: function() {
        this._componentElement.removeClassName('hide');
        this._updateTitle(this._hideContentTitle);
        if (!this._idIsRandom) {
            this._setHidden(false);
        }
    },

    hideContent: function() {
        this._componentElement.addClassName('hide');
        this._updateTitle(this._title);
        if (!this._idIsRandom) {
            this._setHidden(true);
        }
    },

    toggleContent: function() {
        this._componentElement.hasClassName('hide')
            ? this.showContent()
            : this.hideContent();
    }

});

Jsw.SmallTools = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._operations = this._getConfigParam('operations', null);
        this._operationsClass = this._getConfigParam('operationsClass', 'objects-toolbar clearfix');
        this._operationsTag = this._getConfigParam('operationsTag', 'div');

        if (Object.isArray(this._operations) && (0 == this._operations.size())) {
            this._operations = null;
        }
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement('<div id="' + this._id + '-container"></div>');
    },

    _getActionsBoxHtml: function() {
        if (null == this._operations) {
            return '';
        }

        var html = '<div class="actions-box" style="position: relative;">';

        if (this._operations) {
            html +=
                '<' + this._operationsTag + ' id="' + this._id +
                    '-operations" class="' + this._operationsClass + '">' +
                '</' + this._operationsTag + '>';
        }

        html +=
            '</div>';

        return html;
    },

    _addOperations: function() {
        this._operations.each(function(config) {
            eval('var operation = new ' + config.componentType + '(config);');
            operation.setRenderTarget(this._id + '-operations');
            operation.render();
        }, this);
    },

    render: function($super) {
        $super();

        this._componentElement.down().replace(
            '<div id="' + this._id + '-container">' +
                this._getActionsBoxHtml() +
            '</div>'
        );

        if (this._operations) {
            this._addOperations();
        }

        Jsw.Tooltip.initData(this._componentElement);
    }
});


Jsw.CollapsibleBlock = Class.create(Jsw.Container, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._blockAreaId = this._id + '-block';
        this._titleAreaId = this._id + '-title-area';
        this._contentAreaId = this._id + '-content-area';
    },

    _initComponentElement: function($super) {
        $super();

        var containerContentChilds = this._componentElement.childElements();

        this._updateComponentElement(
            '<div class="' + this._cls +  '-block' + (this._getConfigParam('collapsed', true) ? ' hide' : '') + '" id="' + this._blockAreaId + '">' +
                '<div class="' + this._cls +  '-title" id="' + this._titleAreaId + '">' +
                    '<span>' + this._getConfigParam('title', '') +  '</span>' +
                '</div>' +
                '<div class="' + this._cls +  '-content" id="' + this._contentAreaId + '"></div>' +
            '</div>'
        );
    },

    _addEvents: function($super) {
        $super();
        $(this._titleAreaId).down('span').observe('click', this._onTitleClickEvent.bind(this));
    },

    _onTitleClickEvent: function(event) {
        Event.stop(event);
        $(this._blockAreaId).toggleClassName('hide');
    }
});

Jsw.Collapsible = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._contentAreaId = this._id + '-content-area';
        this._anchorAreaId = this._id + '-anchor-area';
    },

    _initComponentElement: function($super) {
        $super();

        var containerContentChilds = this._componentElement.childElements();

        this._updateComponentElement(
            '<div class="collapsible-box collapsed">' +
                '<div class="collapsible-anchor" id="' + this._anchorAreaId + '"></div>' +
                '<div class="collapsible-area">' +
                    '<div class="tl"><div class="br"><div class="bl"><div class="tr">' +
                        '<div class="collapsible-content" id="' + this._contentAreaId + '">' +
                            this._getConfigParam('html', '') +
                            //...collapsible elements
                        '</div>' +
                    '</div></div></div></div>' +
                '</div>' +
             '</div>'
        );
    },

    _addEvents: function($super) {
        $super();

        //init collapsed status
        if (this._getConfigParam('collapsed', true)) {
            this.collapse();
        } else {
            this.expand();
        }
        //handle click event on anchor
        this._componentElement.select('.collapsible-anchor').each(function(anchorElement) {
            Event.observe(anchorElement, 'click', this._onAnchorClickEvent.bind(this));
        }, this);
    },

    _onAnchorClickEvent: function(event) {
        if (this._collapsed) {
            this.expand();
        } else {
            this.collapse();
        }
        Event.stop(event);
    },

    _getCollapseBinding: function() {
        if (!this._collapseBinding) {
            this._collapseBinding = this.collapse.bind(this);
        }
        return this._collapseBinding;
    },

    collapse: function() {
        if (this._collapsed) {
            return;
        }
        this._collapsed = true;
        Event.stopObserving(document, 'click', this._getCollapseBinding());
        var content = $(this._contentAreaId).up('.collapsible-box');
        content.removeClassName('expanded');
        content.addClassName('collapsed');
    },

    expand: function() {
        if (!this._collapsed) {
            return;
        }
        this._collapsed = false;
        var content = $(this._contentAreaId).up('.collapsible-box');
        content.removeClassName('collapsed');
        content.addClassName('expanded');
        Event.observe(document, 'click', this._getCollapseBinding());
    }

});

Jsw.DropdownArea = Class.create(Jsw.Collapsible, {
    _initComponentElement: function($super) {
	    $super();
	    this._componentElement.down('#' + this._anchorAreaId).update(
	        '<h4><span>' + this._getConfigParam('title', '') + '</span></h4>'
	    );
	}
});

Jsw.DropdownSplit = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
    	$super(config);
    	this._cls = this._getConfigParam('cls', 'split');
	}
});

Jsw.Dropdown = Class.create(Jsw.Collapsible, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._valuesAreaId = this._id + '-values-area';
        this.onChange = this._getConfigParam('onChange', this.onChange);
    },

    _initComponentElement: function($super) {
        $super();
        this._componentElement.down('#' + this._contentAreaId).update(
            '<h4><span>' + this._getConfigParam('title', '') + '</span></h4>' +
            '<ul id="' + this._valuesAreaId + '"></ul>'
        );
    },

    _renderItem: function(item) {
        var dropdownItem = new Jsw.DropdownItem(item);

        var valueItem = new Element('a').update(dropdownItem.title.escapeHTML());

        $(this._valuesAreaId).insert(
            new Element('li').update(valueItem)
        );

        valueItem.observe('click', this._onClick.bindAsEventListener(this, dropdownItem));
    },

    _onClick: function(event, item)
    {
        this.onChange(item);
    },

    onChange: function(item) {
        this.setValue(item.value);
    },

    setValue: function(value) {
        this._value = value;
    },

    getValue: function() {
        return this._value;
    }

});

Jsw.DropdownItem = Class.create({

    initialize: function(item) {
        if (Object.isString(item)) {
            this.value = item;
            this.title = item;
        } else
        if (item.title || item.value) {
            this.value = item.value;
            this.title = item.title;
        }
    }
});

Jsw.Pathbar = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-content-area';
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<ul id="' + this._contentAreaId + '">' +
            '</ul>'
        );
    },

    _renderItem: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(this._contentAreaId).insert('<li id="' + renderTargetId +'"></li>');
        item.setRenderTarget(renderTargetId);
        item.render();
        this._renderItemSuffix(item);
    },

    _renderItemSuffix: function(item) {
        var renderTargetId = this._id + '-item-' + item.getId();
        $(renderTargetId).insert("<b>&gt;</b>");
    }

});

Jsw.Pathbar.Item = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
        this._href = this._getConfigParam('href', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement = new Element('a', {
            'href': this._href
        }).update('<span>' + this._title + '</span>');
    },

    getHref: function() {
        return this._href;
    }
});

Jsw.Pathbar.SimpleItem = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement = new Element('span');
        this._componentElement.update(this._title);
    }
});


Jsw.Bar = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._contentAreaId = this._id + '-content-area';

        // allowed types: static, dynamic
        this._type = this._getConfigParam('type', 'static');
    },

    _initComponentElement: function($super) {
        $super();

        var content = '<ul id="' + this._contentAreaId + '"></ul>';
        if ('navbar' != this._id) {
            content = '<div class="tabs-area">' + content + '</div>';
        }

        this._updateComponentElement(content);
    },

    _renderItems: function() {
        if (null != this._items) {
            var size = this._items.size();
            this._items.each(function(item, i) {
                this._renderItem(item, i, size);
            }, this);
        }
    },

    _renderItem: function(item, i, size) {
        if (0 == i && 'navbar' != this._id) {
            var renderTargetId = 'first-tab';
        } else if (size - 1 == i && 'navbar' != this._id) {
            var renderTargetId = 'last-tab';
        } else {
            var renderTargetId = this._id + '-item-' + item.getId();
        }
        var containerCls = item.getContainerClass();

        $(this._contentAreaId).insert('<li class="' + containerCls + '" id="' + renderTargetId +"\"></li>");
        item.setRenderTarget(renderTargetId);
        item.render();

        if ('dynamic' == this._type) {
            item.addEventObserver('click', item.switchTab.bindAsEventListener(item));
        }

        if ($('main')) {
            if (!$('main').hasClassName('tabed') && 'navbar' != this._id) {
                $('main').addClassName('tabed');
            }
        }
    }
});

Jsw.bar = {};

Jsw.bar.Tabs = Class.create(Jsw.Bar, {

    switchTab: function(tabId) {
        var activeTab = null;

        this.getItems().each(function(item) {
            if (tabId == item.getTabId()) {
                activeTab = item;
            }

            $(item.getTabId()).hide();
            $(item.getRenderTarget()).removeClassName('active');
        });

        $(tabId).show();
        $(activeTab.getRenderTarget()).addClassName('active');
    },

    _renderItem: function($super, item, i, size) {
        $super(item, i, size);

        var content = item.getTabContent();

        if (content) {
            var tabContentElement = new Element('div', {
                'id': item.getTabId()
            });

            if (!item.isActive()) {
                tabContentElement.setStyle({ 'display': 'none' });
            }

            this._componentElement.up().insert({ bottom: tabContentElement });

            if (Object.isString(content)) {
                tabContentElement.update(content);
            } else {
                content.setRenderTarget(item.getTabId());
                content.render();
            }
        }
    }
});

Jsw.bar.Button = Class.create(Jsw.Button, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._isActive = this._getConfigParam('active', false);
        this._cls = this._getConfigParam('cls', '');
        this._containerCls = this._getConfigParam('containerCls', this._isActive ? 'active' : '');
        this._tabId = this._getConfigParam('tabId', null);
        this.navigationTab = this._getConfigParam('navigationTab', false);
        this._content = this._getConfigParam('content', null);
    },

    _initComponentElement: function() {
        this._componentElement = new Element('a', {
            'class': this._cls
        }).update('<span>' + this._config['title'] + '</span>');

        this._initVisibility();
    },

    getContainerClass: function() {
        return this._containerCls;
    },

    getTabId: function() {
        return this._tabId;
    },

    switchTab: function(event) {
        var bar = Jsw.getComponent(Event.element(event).up('ul').up('div').up('div').id);
        bar.switchTab(this._tabId);
        event.stop();
    },

    getTabContent: function() {
        return this._content;
    },

    isActive: function() {
        return this._isActive;
    }

});

Jsw.bar.Separator = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'separator');
    },

    _initComponentElement: function($super) {
        $super();
        this._componentElement.update('<span></span>');
    }
});

Jsw.bar.HorizontalSeparator = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._wrapperClass = this._getConfigParam('wrapperClass', 'separator');
    }
});

Jsw.list = {

    COLUMN_SELECTION: {
        cls: 'select',
        headerCls: 'select',
        headerRenderer: function(column) {
            return '<input type="checkbox" class="checkbox" name="listGlobalCheckbox"/>';
        },
        renderer: function(item, isDisabled) {
            if (!isDisabled) {
                return '<input type="checkbox" class="checkbox" name="listCheckbox[]" value="' + item.id + '"/>';
            }

            var html =
                '<div>' +
                    '<span class="checkbox-disabled-wrapper"></span>' +
                    '<input style="z-index: 1;" disabled="disabled" type="checkbox" class="checkbox"/>' +
                '</div>';

            return html;
        }
    },

    ITEMS_UNLIMITED: 100000

};

Jsw.list.AdditionalActions = Class.create(Jsw.Component, {

    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);

        this._operations = this._getConfigParam('operations', null);
        this._cls = this._getConfigParam('cls', 'actions-menu');
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.insert({
            bottom:
                '<div class="popup-box collapsed">' +
                    '<table class="popup-wrapper" cellspacing="0"><tbody><tr>' +
                        '<td class="popup-container">' +
                            '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                                '<div class="popup-heading">' +
                                    '<div class="heading-area">' +
                                        '<span class="close"></span>' +
                                        '<h4>' +
                                            '<span>' +
                                                this._getConfigParam('title', '') +
                                            '</span>' +
                                        '</h4>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="popup-content">'+
                                    '<div class="popup-content-area">' +
                                        '<ul id="' + this._id + '-operations">' +
                                        '</ul>' +
                                    '</div>' +
                                '</div>' +
                            '</div></div></div></div></div>' +
                        '</td>' +
                    '</tr></tbody></table>' +
                '</div>'
        });
    },

    _addEvents: function($super) {
        $super();

        var context = this;
        $(document.body).observe('click', function() {
            context.collapse();
        });
    },

    enable: function() {
        this._titleButton.enable();
    },

    disable: function() {
        this._titleButton.disable();
    },

    toggle: function() {
        var popup = this._componentElement.down('.popup-box');

        if (popup) {
            popup.toggleClassName('collapsed');
        }
    },

    collapse: function() {
        var popup = this._componentElement.down('.popup-box');

        if (popup) {
            popup.addClassName('collapsed');
        }
    },

    render: function($super) {
        $super();

        this._titleButton = new Jsw.SmallButton({
            title: this._getConfigParam('title', ''),
            cls: 's-btn',
            addCls: 'sb-more-actions',
            renderTo: this._id,
            handler: (function(event) {
                Event.stop(event);
                this.toggle();
            }).bind(this)
        });

        if (!this._operations) {
            return;
        }

        var operationsArea = $(this._id + '-operations');
        this._operations.each(function(config) {
            var liElement = new Element('li');
            operationsArea.insert(liElement);

            eval('var operation = new ' + config.componentType + '(config);');

            var wrapperClass = operation.getWrapperClass();
            if (wrapperClass) {
                liElement.addClassName(wrapperClass);
            }

            operation.setRenderTarget(liElement);
            operation.render();
        }, this);

        if (this._getConfigParam('disabled', false)) {
            this.disable();
        } else {
            this.enable();
        }
    }

});

Jsw.List = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._listCls = this._getConfigParam('listCls', 'list');
        this._tableId = this._id + '-table';
        this._dataUrl = this._getConfigParam('dataUrl', '');
        this._operations = this._getConfigParam('operations', null);
        this._operationsClass = this._getConfigParam('operationsClass', 'objects-toolbar clearfix');
        this._operationsTag = this._getConfigParam('operationsTag', 'div');
        this._columns = this._getConfigParam('columns', null);
        this._limitValues = this._getConfigParam('limitValues', [10, 25, 100, Jsw.list.ITEMS_UNLIMITED]);
        this._splitListData(this._getConfigParam('data', { data: null, pathbar: null, state: null, pager: null, locale: null }));
        this._rowSelectable = this._isColumnSelectionPresent();
        this._searchFilters = $H(this._getConfigParam('searchFilters', {}));
        this._searchable = this._getConfigParam('searchable', this._searchFilters.size() > 0);
        this._searchOveral = this._getConfigParam('searchOveral', null);
        this._searchExtended = this._getConfigParam('searchExtended', this._searchFilters.size() > 1);
        this._pageable = this._getConfigParam('pageable', true);
        this._disabledSelectHint = this._getConfigParam('disabledSelectHint', '');
        this.isDisabledItem = this._getConfigParam('isDisabledItem', function() {
            return false;
        });
        this._onRedraw = this._getConfigParam('onRedraw', null);

        if (Object.isArray(this._operations) && (0 == this._operations.size())) {
            this._operations = null;
        }
    },

    _getSearchOveralField: function()
	{
    	if (null === this._searchOveral) {
    		this._searchFilters.each(function (pair){
        		if(pair.value.overal) {
        			this._searchOveral = pair.key;
        		}
    		}, this);
    		if (null === this._searchOveral && 1 == this._searchFilters.size()) {
    			this._searchOveral = this._searchFilters.keys().first();
    		}
    		if (null === this._searchOveral) {
    			this._searchOveral = false;
    		}
    	}

    	return this._searchOveral;
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement('<div id="' + this._id + '-container"></div>');
        this._initDisablerOverlay();
    },

    _splitListData: function(listData) {
        if (!listData) {
            return false;
        }

        if (!listData.data || !listData.state || !listData.pager || !listData.locale) {
            return false;
        }
        this._data = listData.data;
        this._state = listData.state;
        this._pager = listData.pager;
        this._locale = listData.locale;

        if (listData.pathbar) {
            this._pathbar = listData.pathbar;
        }
        return true;
    },

    _isColumnSelectionPresent: function() {
        for (var index = 0; index < this._columns.length; index++) {
            if (Jsw.list.COLUMN_SELECTION == this._columns[index]) {
                return true;
            }
        }

        return false;
    },

    onHeaderClick: function(dataIndex) {
        this._state.sortField = dataIndex;
        this._state.sortDirection = ('down' == this._state.sortDirection)
            ? 'up'
            : 'down';

        this.reload('/sort-field/' + dataIndex + '/sort-dir/' + this._state.sortDirection);
    },

    onLimitClick: function(value) {
        this.reload('/items-per-page/' + value);
    },

    onPagerClick: function(page) {
        this.reload('/page/' + page);
    },

    onEnumSearchFilterChange: function(element) {
        this._onSearchClick(Element.up(element, 'form'));
    },

    _getListHeadersHtml: function() {
        var html = '<tr>';
        var colspan = 0;

        this._columns.each(function(column) {
            if (colspan) {
                colspan --;
                return;
            }

            if (column.headerColspan) {
                colspan = column.headerColspan;
            }

            var data = '';

            var headerCls = '';

            if ('undefined' != typeof column.headerCls) {
                headerCls = column.headerCls;
            }

            if ('undefined' != typeof column.headerRenderer) {
                data = column.headerRenderer(column);
            } else {
                data = column.header;

                var sortCssClass = '';

                if (this._state.sortField == column.dataIndex) {
                    sortCssClass = ('down' == this._state.sortDirection)
                        ? ''
                        : 'sort-up';

                    headerCls = (headerCls)
                        ? (headerCls + ' ' + 'sort')
                        : 'sort';
                }

                if (column.sortable) {
                    data = '<a class="' + sortCssClass + '" href="#" onclick="Jsw.getComponent(\'' + this._id
                        + '\').onHeaderClick(\'' + column.dataIndex + '\')">' + data + '</a>';
                }
            }

            html += '<th class="' + headerCls + '"' + (colspan ? 'colspan="' + colspan + '"' : '') + '>' + data + '</th>';
        }, this);

        html += '</tr>';

        return html;
    },

    _getListDataHtml: function() {
        if (!this._data) {
            return ;
        }

        var html = '';
        var currentRowClass = 'odd';

        this._data.each(function(item) {
            html += '<tr class="' + currentRowClass + '">';

            this._columns.each(function(column) {
                var data = '';

                if ('undefined' != typeof column.renderer) {
                    var isDisabled = this.isDisabledItem(item, this._data);
                    data = column.renderer(item, isDisabled);
                } else if ('undefined' != typeof column.dataIndex) {
                    data = item[column.dataIndex].escapeHTML();
                }

                var itemCls = '';

                if ('undefined' != typeof column.cls) {
                    itemCls = column.cls;
                }

                html += '<td class="' + itemCls + '">' + data + '</td>';
            }, this);

            html += '</tr>';

            currentRowClass = ('odd' == currentRowClass) ? 'even' : 'odd';
        }, this);

        return html;
    },

    _getListHtml: function() {
        if (0 == this._pager.totalItemCount) {
            return '<span class="empty-list">' + this._locale.noEntriesFound + '</span>';
        }

        var html =
            this._getPagerHtml() +
            '<div class="' + this._listCls + '">' +
                '<table width="100%" cellspacing="0" id="' + this._tableId + '">' +
                    this._getListHeadersHtml() +
                    this._getListDataHtml() +
                '</table>' +
            '</div>' +
            this._getPagerHtml();

        return html;
    },

    _getPathbarHtml: function() {
        if (!this._pathbar) {
            return '';
        }

        return '<div id="' + this._id + '-pathbar"></div>';
    },

    _getPagerHtml: function() {
        if (!this._pageable) {
            return '';
        }
        var totalRecords = this._locale.recordsTotal.replace('%%total%%', this._pager.totalItemCount);

        var html =
            '<div class="paging">' +
                '<div class="paging-area">' +
                    '<span class="paging-info">' + totalRecords + '</span>' +
                    '<span class="paging-view">' + this._locale.numberOfItemPerPage + ': ' +
                        this._getLimitValuesHtml() +
                    '</span>';

        if (this._pager.pageCount > 1) {
            html += '<span class="paging-nav">' + this._locale.pages + ': ' + this._getPagesItemsHtml() + '</span>';
        }

        html += '</div></div>';

        return html;
    },

    _getLimitValuesHtml: function() {
        var html = '';

        this._limitValues.each(function(limitValue) {
            var limitValueTitle = (Jsw.list.ITEMS_UNLIMITED == limitValue)
                ? this._locale.allItems
                : limitValue;

            if (this._pager.itemCountPerPage != limitValue) {
                html += '<a href="#" onclick="Jsw.getComponent(\'' + this._id +
                    '\').onLimitClick(' + limitValue + ');">' + limitValueTitle + '</a> ';
            } else {
                html += '<span>' + limitValueTitle + '</span> ';
            }
        }, this);

        return html;
    },

    _getPagesItemsHtml: function() {
        var html = '';

        if (this._pager.first != this._pager.current) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.first) + '">'
                + this._locale.firstPage + '</a> ';
        } else {
            html += '<span>' + this._locale.firstPage + '</span> ';
        }

        if (this._pager.previous) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.previous) + '">&lt;&lt;</a> ';
        } else {
            html += '<span>&lt;&lt;</span> ';
        }

        $H(this._pager.pagesInRange).values().each(function(page) {
            if (page != this._pager.current) {
                html += '<a href="#" onclick="' + this._getPageItemLink(page) + '">' + page + '</a> ';
            } else {
                html += page + ' ';
            }
        }, this);

        if (this._pager.next) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.next) + '">&gt;&gt;</a> ';
        } else {
            html += '<span>&gt;&gt;</span> ';
        }

        if (this._pager.last != this._pager.current) {
            html += '<a href="#" onclick="' + this._getPageItemLink(this._pager.last) + '">'
                + this._locale.lastPage + '</a> ';
        } else {
            html += '<span>' + this._locale.lastPage + '</span> ';
        }

        return html;
    },

    _getPageItemLink: function(page) {
        return "Jsw.getComponent('" + this._id + "').onPagerClick(" + page + ");";
    },

    _getActionsBoxHtml: function() {
        if (!this._searchable && null == this._operations) {
            return '';
        }

        var searchStateClass = '';

        if (this._state.forceShowSearch) {
            Jsw.Cookie.set(this._id + '-search-show', 'true');
        }

        if (
            this._searchable &&
            this._searchExtended &&
            'true' != Jsw.Cookie.get(this._id + '-search-show')
        ) {
            searchStateClass = ' hide';
        }

        var html = '<div class="actions-box' + searchStateClass + '" style="position: relative;">';

        if (this._operations || this._searchable) {
            html +=
                '<' + this._operationsTag + ' id="' + this._id +
                    '-operations" class="' + this._operationsClass + '">' +
                '</' + this._operationsTag + '>';
        }

        if (this._searchable && this._searchExtended) {
            html +=
                '<div class="search-box">' +
                    '<form>' +
                    	'<ul>' +
                        	this._getSearchFiltersHtml() +
                    	'</ul>' +
                        '<a class="s-btn sb-search" href="#">' +
	                        '<i><span>' + this._locale.buttonSearch + '</span></i>' +
	                    '</a>' +
	                    '<a class="s-btn sb-show-all" href="#">' +
	                        '<i><span>' + this._locale.buttonResetSearch + '</span></i>' +
	                    '</a>' +
	                    '<input type="image" style="border: 0pt none ; height: 0pt; width: 0pt; position: absolute; left: -9999px;" src="' + Jsw.skinUrl + '/img/blank.gif">' +
                	'</form>' +
                '</div>';
        }

        html +=
            '</div>';

        return html;
    },

    _getSearchFiltersHtml: function() {
    	if (!this._searchable) {
    		return '';
    	}

    	var searchFilterHtml = '';

    	this._searchFilters.each(function(pair) {
    		var filterName = pair.key;
    		var filterConfig = pair.value;

    		var filterTitle = filterConfig.title;
    		var filterTitleHtml = '';
    		if (filterTitle) {
    			filterTitleHtml = '<span>' + filterTitle + '</span>';
            }

    		var filterBodyHtml = this._getSearchFilterBodyHtml(filterName, filterConfig);

    		searchFilterHtml +=
    			'<li>' +
    				'<div>' +
    					filterTitleHtml +
    					filterBodyHtml +
    				'</div>' +
    			'</li>';
    	}, this);

    	return searchFilterHtml;
    },

    _getSearchFilterBodyHtml: function(filterName, filterConfig) {
		var filterBodyHtml = '';

		if (filterConfig.render) {
			filterBodyHtml = filterConfig.render(filterConfig);
		} else if(filterConfig.options) {
			filterBodyHtml = this._getSearchFilterHtmlEnum(filterName, filterConfig);
		} else {
			filterBodyHtml = this._getSearchFilterHtmlText(filterName, filterConfig);
		}

		return filterBodyHtml;
    },

    _getSearchFilterHtmlText: function(filterName, filterConfig) {
    	var filterHtml = '<input' +
    			' id="' + this._id + '-search-text-' + filterName + '"' +
    			' name="searchFilter[' + filterName + '][searchText]"' +
    			' type="text"' +
    			' value="' + $H(this._state.searchFilters).get(filterName).searchText.escapeHTML().replace(/"/g, '&quot;') + '"' +
			'/>';
    	return filterHtml;
    },

    _getSearchFilterHtmlEnum: function(filterName, filterConfig) {
    	var currentValue = $H(this._state.searchFilters).get(filterName).searchText;
    	var optionsHtml = '';

    	filterOptions = $A(filterConfig.options);

    	filterOptions.each(function(item) {
    		if ('value' == item.type) {
    			optionsHtml += this._getSearchFilterHtmlEnumValue(currentValue, item.value, item.label);
    		} else {
    			optionsHtml += this._getSearchFilterHtmlEnumGroup(currentValue, item.options, item.label);
    		}
    	}, this);

    	var filterHtml = '<select' +
			' id="' + this._id + '-type-filter"' +
			' name="searchFilter[' + filterName + '][searchText]"' +
			' onchange="Jsw.getComponent(\'' + this._id + '\').onEnumSearchFilterChange(this)"' +
		'>' + optionsHtml + '</select>';

    	return filterHtml;
    },

    _getSearchFilterHtmlEnumValue: function(currentValue, value, label)
    {
        var selected = currentValue == value ? 'selected="selected"' : '';
    	var ret = '<option value="' + value.escapeHTML().replace(/"/g, '&quot;') + '"' + selected + '>' + label.escapeHTML() + '</option>';
    	return ret;
    },

    _getSearchFilterHtmlEnumGroup: function(currentValue, options, label)
    {
    	var ret = '';
		ret += '<optgroup label="' + label.escapeHTML().replace(/"/g, '&quot;') + '">';
		$A(options).each(function(item) {
    		if ('value' == item.type) {
    			ret += this._getSearchFilterHtmlEnumValue(currentValue, item.value, item.label);
    		} else {
    			ret += this._getSearchFilterHtmlEnumGroup(currentValue, item.options, item.label);
    		}
		}, this);
		ret += '</optgroup>';
    	return ret;
    },

    _addSearchHandlers: function() {
        if (!this._searchable) {
            return;
        }

        var context = this;

        if (this._searchExtended) {
	        this._componentElement.select('.search-control .s-btn').each(function(element) {
	            element.observe('click', function() {
	                Jsw.Cookie.set(context._id + '-search-show', this.up('.actions-box').hasClassName('hide'));
	                this.up('.actions-box').toggleClassName('hide');
	                context._hideItemsNotSelectedWarning();
	                return false;
	            });
	        });
        }

        if (this._searchExtended) {
	        var searchLink = this._componentElement.select('.sb-search').first();
	        var form = searchLink.up('form');
	        searchLink.observe('click', function(event) {
	        	context._onSearchClick(form);
	        	Event.stop(event);
	        });
	        form.observe('submit', function(event) {
	        	context._onSearchClick(form);
	        	Event.stop(event);
	        });


	        var resetSearchLink = this._componentElement.select('.sb-show-all').first();
	        resetSearchLink.observe('click', function(event) {
	        	context._onResetSearchClick();
	        	Event.stop(event);
	        });
        }

        if (this._getSearchOveralField()) {
        	this._componentElement.select('.quick-search-box form span em').each(function(element) {
        		element.observe('click', function(event) {
        			element.up('form').submit();
        		});
        	});
        	this._componentElement.select('.quick-search-box form').each(function(element) {
        		element.observe('submit', function(event) {
        			context._onSearchClick(element);
        			Event.stop(event);
        		});
        	});
        }
    },

    _onSearchClick: function(form) {
        var searchQuery = '?';
        if (this._dataUrl.include('?')) {
            searchQuery = '&';
        }
        searchQuery += form.serialize();
        this.reload(searchQuery);
    },

    _onResetSearchClick: function() {
    	this.reload('/reset-search/true/');
    },

    _addSelectionHandlers: function() {
        var context = this;

        var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();

        globalCheckbox.observe('click', function() {
            context._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
                if (globalCheckbox.checked) {
                    checkbox.up('tr').addClassName('selected');
                } else {
                    checkbox.up('tr').removeClassName('selected');
                }
                checkbox.checked = globalCheckbox.checked;
            });
        });

        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            checkbox.observe('click', function() {
                if (checkbox.checked) {
                    checkbox.up('tr').addClassName('selected');
                } else {
                    checkbox.up('tr').removeClassName('selected');
                    globalCheckbox.checked = false;
                }
            });
        });
    },

    _addOperations: function() {
        this._operations.each(function(config) {
            eval('var operation = new ' + config.componentType + '(config);');
            operation.setRenderTarget(this._id + '-operations');
            operation.render();
        }, this);
    },

    _addPathbar: function() {
        if (!this._pathbar) {
            return;
        }

        eval('this._pathbarComponent = new ' + this._pathbar.componentType + '(this._pathbar);');
        this._pathbarComponent.setRenderTarget(this._id + '-pathbar');
        this._pathbarComponent.render();
    },

    disable: function() {
        this._disablerOverlay.show();
        Jsw.clonePosition(this._disablerOverlay, this._componentElement);
    },

    enable: function() {
        this._disablerOverlay.hide();
    },

    getSelectedItemsIds: function() {
        var itemsIds = new Array();

        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            if (checkbox.checked) {
                itemsIds.push(checkbox.value);
            }
        });

        return itemsIds;
    },

    checkNonEmptySelection: function() {
    	if (0 == this.getSelectedItemsIds().length) {
    		this._showItemsNotSelectedWarning();
    		return false;
    	}
    	this._hideItemsNotSelectedWarning();
    	return true;
    },

    execGroupOperation: function(params) {
        var context = this;

        var ids = $H();
        var count = 0;

        var idsParamName = params.submitVarName
            ? params.submitVarName
            : 'ids';

        this.getSelectedItemsIds().each(function(id) {
            ids.set(idsParamName + '[' + count + ']', id);
            count++;
        });

        if (params.checkSelection) {
            if (!params.checkSelection()) {
                return;
            }
        } else if (0 == count) {
            this._showItemsNotSelectedWarning();
            return ;
        } else {
            this._hideItemsNotSelectedWarning();
        }

        var sendRequestHandler = params.submitHandler
            ? params.submitHandler
            : function(url, ids) {
                new Ajax.Request(Jsw.prepareUrl(url), {
                    method: 'post',
                    parameters: ids,
                    onSuccess: function (transport) {
                        var status = transport.responseText.evalJSON();
                        if (status.redirect) {
                            Jsw.redirect(status.redirect);
                        } else {
                            context.reload();
                        }
                        Jsw.clearStatusMessages();
                        $A(status.statusMessages).each(function(message) {
                            Jsw.addStatusMessage(message.status, message.content);
                        });

                        if (params.onSuccess) {
                            params.onSuccess();
                        } else {
                            context.enable();
                        }
                    },

                    onCreate: function() {
                        context.disable();
                    },

                    onFailure: function() {
                        context.enable();
                    }
                });
            }

        var sendRequest = function() {
            sendRequestHandler(params.url, ids);
        }

        if (params.skipConfirmation) {
            sendRequest();
        } else {
            Jsw.messageBox.show({
                type: Jsw.messageBox.TYPE_YESNO,
                buttonTitles: {
                    yes: this._locale.messageBoxButtonYes,
                    no: this._locale.messageBoxButtonNo
                },
                text: params.getConfirmOnGroupOperation
                    ? params.getConfirmOnGroupOperation()
                    : params.locale.confirmOnGroupOperation,
                subtype: params.subtype,
                needAttention: params.needAttention ? params.needAttention : false,
                needAttentionText: params.needAttentionText ? params.needAttentionText : '',
                needAttentionBlockSubmit: params.needAttentionBlockSubmit ? params.needAttentionBlockSubmit : false,
                onYesClick: sendRequest
            });
        }
    },

    _showItemsNotSelectedWarning: function() {
        var element = this._componentElement.down('.actions-msg-container');
        if (element) {
            element.show();
        } else {
            this._componentElement.down('.actions-box').insert({ top:
                '<div class="actions-msg-container"><span class="list-actions-msg"><span>' +
                this._locale.itemsNotSelected + '</span></span></div>'
            });
        }
    },

    _hideItemsNotSelectedWarning: function() {
        var element = this._componentElement.down('.actions-msg-container');
        if (element) {
            element.hide();
        }
    },

    render: function($super) {
        $super();

        if (!this._data) {
            this.reload();
            return ;
        }

        this.redraw();
    },

    reload: function(params) {
        params = params || '';
        var context = this;

        if (!this._dataUrl) {
        	return;
        }

        new Ajax.Request(Jsw.prepareUrl(this._dataUrl + params), {
            method: 'get',
            onSuccess: function(transport) {
                if ('' == transport.responseText) {
                    //:INFO: sometimes happens in FF if you are go to other page during request
                    return;
                }

                try {
                    var data = transport.responseText.evalJSON();
                } catch (e) {
                    Jsw.showInternalError(transport.responseText);
                    return;
                }

                if (context._splitListData(data)) {
                    context.redraw();
                } else {
                    context.processReloadError(data);
                }
            },

            onFailure: this._onReloadFailure.bind(this),

            onException: this._onReloadException.bind(this),

            onCreate: this._onReloadCreate.bind(this),

            onComplete: this._onReloadComplete.bind(this)
        });
    },

    _onReloadFailure: function(transport) {
        Jsw.showInternalError(transport.responseText);
    },

    _onReloadException: function(transport, exception) {
        Jsw.showInternalError(exception + "\n" + transport.responseText);
    },

    _onReloadCreate: function(transport) {
        Jsw.clearStatusMessages();
        this.disable();
    },

    _onReloadComplete: function() {
        this.enable();
    },

    _addDisabledCheckboxesHints: function() {
        var checkboxes = this._componentElement.select('input[type="checkbox"][disabled]');

        checkboxes.each(function(checkbox) {
            new Jsw.Tooltip.Instance(checkbox.previous(), {text: this._disabledSelectHint });
        }, this);
    },

    redraw: function() {
        this._componentElement.down().replace(
            '<div id="' + this._id + '-container">' +
                this._getActionsBoxHtml() +
                this._getPathbarHtml() +
                this._getListHtml() +
            '</div>'
        );

        if (this._operations) {
            this._addOperations();
        }

        if (this._searchable) {
        	this._addSearchTools();
        }

        if (this._pager.totalItemCount > 0) {
            if (this._rowSelectable) {
                this._addSelectionHandlers();
            }
        }

        this._addPathbar();
        this._addSearchHandlers();

        if (this._disabledSelectHint) {
            this._addDisabledCheckboxesHints();
        }

        Jsw.Tooltip.initData(this._componentElement);
        if (this._onRedraw) {
        	this._onRedraw();
        }
    },

    _addSearchTools: function() {
        searchBoxHtml =
            '<div class="quick-search-box">';

    	if (this._getSearchOveralField()) {
    		searchBoxHtml +=
    			'<form>' +
		            '<span class="search-field">' +
						this._getSearchFilterBodyHtml(this._getSearchOveralField(), this._searchFilters.get(this._getSearchOveralField())) +
						'<em><span></span></em>' +
					'</span>' +
				'</form>';
    	}

    	if (this._searchExtended) {
            searchBoxHtml +=
                    '<span class="search-control">' +
                        '<a class="s-btn sb-search-show" href="#">' +
                            '<i><i><i><span>' + this._locale.showSearch + '</span></i></i></i>' +
                        '</a>' +
                        '<a class="s-btn sb-search-hide" href="#">' +
                            '<i><i><i><span>' + this._locale.hideSearch + '</span></i></i></i>' +
                        '</a>' +
                    '</span>' +
                '</div>';
    	}

    	searchBoxHtml += '</div>';

        $(this._id + '-operations').insert(searchBoxHtml);
    },

    processReloadError: function(data) {
        Jsw.showInternalError('Unable to load list data.');
    }
});

Jsw.DynamicSubForm = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        var getDynamicContent = this._getConfigParam('getDynamicContent', null);
        if (getDynamicContent) {
            this._getDynamicContent = getDynamicContent.bind(this);
        }
        var prepareDynamicContent = this._getConfigParam('prepareDynamicContent', null);
        if (prepareDynamicContent) {
            this._prepareDynamicContent = prepareDynamicContent.bind(this);
        }
        var bindDynamicContent = this._getConfigParam('bindDynamicContent', null);
        if (bindDynamicContent) {
            this._bindDynamicContent = bindDynamicContent.bind(this);
        }
        var insertDynamicContent = this._getConfigParam('insertDynamicContent', null);
        if (insertDynamicContent) {
            this._insertDynamicContent = insertDynamicContent.bind(this);
        }
        var onUpdateDynamic = this._getConfigParam('onUpdateDynamic', null);
        if (onUpdateDynamic) {
            this._onUpdateDynamic = onUpdateDynamic.bind(this);
        }
        var onInitDynamic = this._getConfigParam('onInitDynamic', null);
        if (onInitDynamic) {
            this._onInitDynamic = onInitDynamic.bind(this);
        }
    },

    _addEvents: function($super) {
        $super();
        var dynamicElements = this._componentElement.select('.dynamicSubFormContent');
        if (0 == dynamicElements.size()) {
            var element = this._prepareDynamicContent(this._getDynamicContent());
            this._insertDynamicContent(element);
            this._bindDynamicContent(element, true);
        } else {
            dynamicElements.each(function(element, index) {
                this._bindDynamicContent(element, 0 == index);
            }, this);
        }
        this._onInitDynamic();
        this._onUpdateDynamic();
    },

    _getDynamicContent: function() {
        var dynamicElement = this._componentElement.down('.dynamicSubFormTemplate').cloneNode(true);
        dynamicElement.removeClassName('dynamicSubFormTemplate').addClassName('dynamicSubFormContent').show();
        return dynamicElement;
    },

    _prepareDynamicContent: function(content) {
        return content;
    },

    _insertDynamicContent: function(content) {
        var dynamicElements = this._componentElement.select('.dynamicSubFormContent');
        if (0 == dynamicElements.size()) {
            this._componentElement.insert({top : content});
        } else {
            dynamicElements.last().insert({after : content});
        }
    },

    _bindDynamicContent: function(content, isFirstElement) {
    },

    _onInitDynamic: function() {
    },

    _onUpdateDynamic: function() {
    },

    deleteContent: function(content) {
        content.remove();
        this._onUpdateDynamic();
    },

    addContent: function() {
        var content = this._prepareDynamicContent(this._getDynamicContent());
        this._insertDynamicContent(content);
        this._bindDynamicContent(content, false);
        this._onUpdateDynamic();
    }

});

Jsw.Box = Class.create(Jsw.Component, {
    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            this._getConfigParam('html', '')
        );
    }
});

Jsw.Gauge = Class.create(Jsw.Component, {
    _tag: 'span',

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'gauge-wrapper f9 not_empty');
    },

    _initComponentElement: function($super) {
        $super();

        this.setProgress(this._getConfigParam('value', ''));
    },

    setProgress: function(value)
    {
        this._componentElement.update(
            '<span class="gauge-container">' +
                '<span style="width: ' + value + '%" class="gauge-filled"></span>' +
            '</span>'
        );
    }
});

Jsw.Cookie = {
    get: function(name) {
        var cookies = document.cookie.split("; ");
        for (var i = 0; i < cookies.length; i++) {
            var crumbs = cookies[i].split("=");
            if (name == crumbs[0]) {
               return unescape(crumbs[1]);
            }
        }
        return null;
    },

    set: function(name, value, expires, path) {
        var cookie = name + "=" + escape(value) + "; ";

        if (('undefined' != typeof expires) && expires) {
            cookie += 'expires=' + expires + '; '
        }

        if (('undefined' != typeof path) && path) {
            cookie += "path=" + path + '; ';
        }

        document.cookie = cookie;
    },

    setPermanent: function(name, value, path) {
        var date = new Date();
        date.setTime(date.getTime()+(365*24*60*60*1000));
        var expires = date.toGMTString();
        this.set(name, value, expires, path);
    },

    remove: function(name) {
        document.cookie = name + "=; expires=Fri, 31 Dec 1999 23:59:59 GMT;";
    }
};

Jsw.Tooltip = {
    showTimeoutStatus: true,    //show timeout
    showTimeout: 0.55,            //show timeout
    minDelta: 5,                //minimum viewport delta
    xOffsetLeft: -5,            //horisonatl offset from cursor
    xOffsetRight: 5,            //horisonatl offset from cursor
    yOffsetTop: 1,                //vertical offset from cursor
    yOffsetBottom: 13,            //vertical offset from cursor

    current: null,

    initData: function(contentElement) {
        var elements;

        if ('undefined' != typeof contentElement) {
            elements = contentElement.select('.tooltipData');
        } else {
            elements = $$('.tooltipData');
        }

        elements.each(function(tooltipData) {
            Jsw.Tooltip.init(tooltipData.parentNode, {element: tooltipData});
        });
    },

    init: function(element, config) {
        return new Jsw.Tooltip.Instance(element, config);
    },

    set: function(tooltip) {
        if (Jsw.Tooltip.current && Jsw.Tooltip.current !== tooltip) {
            Jsw.Tooltip.hide();
        }
        if (Jsw.Tooltip.current === tooltip) {
            return;
        }
        Jsw.Tooltip.current = tooltip;
        Element.observe(tooltip._element, 'mousemove', Jsw.Tooltip._onMouseMove);
        Element.observe(tooltip._element, 'mouseout', Jsw.Tooltip._onMouseOut);

        Jsw.Tooltip.show();
    },

    show: function() {
        Jsw.Tooltip.showCancel();
        Jsw.Tooltip.hideCancel();
        if (Jsw.Tooltip.showTimeoutStatus) {
            Jsw.Tooltip.showTask = new PeriodicalExecuter(Jsw.Tooltip.showFinish, Jsw.Tooltip.showTimeout);
        } else {
            Jsw.Tooltip.showFinish();
        }
    },

    showCancel: function() {
        if (Jsw.Tooltip.showTask) {
            Jsw.Tooltip.showTask.stop();
            Jsw.Tooltip.showTask = null;
        }
    },

    showFinish: function() {
        Jsw.Tooltip.showCancel();
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        var tooltip = Jsw.Tooltip.current;
        if (tooltip._config.text) {
            tooltipElement.down('span').update(tooltip._config.text);
        } else
        if(tooltip._config.element) {
            tooltipElement.down('span').update(tooltip._config.element.innerHTML);
        }
        tooltipElement.show();
        Jsw.Tooltip.update();
    },

    hide: function() {
        Jsw.Tooltip.showCancel();
        if (!Jsw.Tooltip.current)
            return;
        var tooltip = Jsw.Tooltip.current;
        Jsw.Tooltip.current = null;
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        tooltipElement.hide();
        Element.stopObserving(tooltip._element, 'mousemove', Jsw.Tooltip._onMouseMove);
        Element.stopObserving(tooltip._element, 'mouseout', Jsw.Tooltip._onMouseOut);
        Jsw.Tooltip.hideTask = new PeriodicalExecuter(Jsw.Tooltip.hideFinish, Jsw.Tooltip.showTimeout);
    },

    hideCancel: function() {
        if (Jsw.Tooltip.hideTask) {
            Jsw.Tooltip.hideTask.stop();
            Jsw.Tooltip.hideTask = null;
        }
    },

    hideFinish: function() {
        Jsw.Tooltip.showTimeoutStatus = true;
    },

    update: function() {
        if (!Jsw.Tooltip.current)
            return;
        var tooltipElement = Jsw.Tooltip._fetchTooltipElement();
        var position = Jsw.Tooltip._fetchTooltipPosition(tooltipElement);
        Element.setStyle(tooltipElement, {
            top: position.y + 'px',
            left: position.x + 'px'
        });
    },

    _onMouseOut: function(event) {
        Jsw.Tooltip.hide();
    },

    _onMouseMove: function(event) {
        Jsw.Tooltip.tooltipX = Event.pointerX(event);
        Jsw.Tooltip.tooltipY = Event.pointerY(event);
        Jsw.Tooltip.update();
    },

    _fetchTooltipElement: function() {
        if (!Jsw.Tooltip._tooltipElement) {
            Jsw.Tooltip._tooltipElement = $$('.tooltip').first();
        }

        return Jsw.Tooltip._tooltipElement;
    },

    _fetchTooltipPosition: function(tooltipElement) {
        var x = Jsw.Tooltip.tooltipX;
        var y = Jsw.Tooltip.tooltipY;

        var tDim = Element.getDimensions(tooltipElement);
        var vDim = document.viewport.getDimensions();
        var vScr = document.viewport.getScrollOffsets();

        var freeRight = vDim.width + vScr.left - Jsw.Tooltip.tooltipX;
        var freeLeft = Jsw.Tooltip.tooltipX - vScr.left;
        var freeTop = Jsw.Tooltip.tooltipY - vScr.top;
        var freeBottom = vDim.height + vScr.top - Jsw.Tooltip.tooltipY;

        //apply better x
        if( freeRight > tDim.width + Jsw.Tooltip.minDelta + Jsw.Tooltip.xOffsetRight) {
            x = Jsw.Tooltip.tooltipX + Jsw.Tooltip.xOffsetRight;
        } else
        if( freeLeft > tDim.width + Jsw.Tooltip.minDelta + Jsw.Tooltip.xOffsetLeft) {
            x = Jsw.Tooltip.tooltipX - tDim.width - Jsw.Tooltip.xOffsetLeft;
        } else {
            x = Jsw.Tooltip.tooltipX - (tDim.width + Jsw.Tooltip.xOffsetLeft) / 2;
        }

        //apply better y
        if (freeBottom > tDim.height + Jsw.Tooltip.minDelta + Jsw.Tooltip.yOffsetBottom) {
            y = Jsw.Tooltip.tooltipY + Jsw.Tooltip.yOffsetBottom;
        } else
        if( freeTop > tDim.height + Jsw.Tooltip.minDelta + Jsw.Tooltip.yOffsetTop) {
            y = Jsw.Tooltip.tooltipY - tDim.height - Jsw.Tooltip.yOffsetTop;
        } else {
            y = Jsw.Tooltip.tooltipY - (tDim.height + Jsw.Tooltip.yOffsetTop) / 2;
        }
        if (x < 0) {
        	x = Jsw.Tooltip.minDelta;
    	}

        return { x: x, y: y };
    }

}

Event.observe(window, 'load', function() {
    Jsw.Tooltip.initData();
});

Jsw.Tooltip.Instance = Class.create({

    initialize: function(element, config) {
        this._element = element;
        this._config = config;
        //observe events
        Event.observe(this._element, 'mouseover', this.set.bind(this));
    },

    set: function() {
        Jsw.Tooltip.set(this);
    },

    setText: function(text) {
        this._config.text = text;
    }
});

Jsw.AjaxTooltip = {
    windowId: 'ajaxTooltipWindow',
    delay: 0.55,
    _isInitialized: false,
    _currentTooltip: null,

    _init: function(cls) {
        if (this._isInitialized) {
            return ;
        }

        this._cls = cls;
        this._initWindowElement();
        this._initWindowEvents();
        this._isInitialized = true;
    },

    _initWindowElement: function($super) {
        $(document.body).insert({
            top: '<div id="' + this.windowId + '" class="' + this._cls + '" style="display: none;"></div>'
        });

        $(this.windowId).update(
            '<table cellspacing="0" class="popup-wrapper"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<span class="close" id="' + this.windowId + '-close' + '"></span>' +
                            '<h4><span id="' + this.windowId + '-title' + '"></span></h4>' +
                        '</div>' +
                    '</div>' +
                    '<div class="popup-content">' +
                        '<div id="' + this.windowId + '-content' + '" class="popup-content-area"></div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr></tbody></table>'
        );
    },

    _initWindowEvents: function() {
        Event.observe($(this.windowId), 'mouseover', this._onWindowMouseOver.bindAsEventListener(this));
        Event.observe($(this.windowId), 'mouseout', this._onWindowMouseOut.bindAsEventListener(this));
        Event.observe($(this.windowId + '-close'), 'click', this._onCloseClick.bindAsEventListener(this));
    },

    _onWindowMouseOver: function(event, title) {
        this._showWindow();
    },

    _onWindowMouseOut: function() {
        $(this.windowId).hide();
    },

    _onCloseClick: function() {
        $(this.windowId).hide();
    },

    set: function(tooltip) {
        this._init(tooltip.getCls());
        var title = tooltip.getTitle();
        var titleElement = $(this.windowId + '-title');
        if (false === title) {
        	$(this.windowId + '-title').update('');
        	titleElement.up('.popup-heading').hide();
        } else {
        	$(this.windowId + '-title').update(title);
        	titleElement.up('.popup-heading').show();
        }
        $(this.windowId + '-content').update(tooltip.getContent());
        Jsw.clonePosition($(this.windowId), tooltip.getTargetElement(), { setWidth: false, setHeight: false });
        this._showWindow();
        this._currentTooltip = tooltip;
    },

    updateContent: function(tooltip) {
        if (this._currentTooltip == tooltip) {
            $(this.windowId + '-content').update(tooltip.getContent());
        }
        this._updatePosition();
    },

    _showWindow: function() {
        $(this.windowId).show();
        this._updatePosition();
    },

    _updatePosition: function()
    {
    	var element = $(this.windowId);

        var vWidth = document.viewport.getWidth();
        var vLeft = document.viewport.getScrollOffsets().left;
        var eWidth = Element.getWidth(element);
        var eLeft = Element.cumulativeOffset(element).left;
        if (eLeft + eWidth > vWidth + vLeft) {
        	Jsw.clonePosition(element, element, {offsetLeft: (vWidth + vLeft) - (eLeft + eWidth),  setWidth: false, setHeight: false });
        }
    }
}

Jsw.AjaxTooltip.Instance = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._targetElement = $(this._getConfigParam('target', ''));
        this._title = this._getConfigParam('title', '');
        this._waitMsg = this._getConfigParam('waitMsg', '');
        this._url = this._getConfigParam('url', '');
        this._cls = this._getConfigParam('cls', 'popup-box');

        this._content = '<div class="ajax-loading">' + this._waitMsg + '</div>';
        this._contentLoaded = false;
        this._delayedExecuter = null;
    },

    _initComponentElement: function($super) {
        this._initTargetEvents();
    },

    _initTargetEvents: function() {
        Event.observe(this._targetElement, 'mouseover', this.set.bind(this));
        Event.observe(this._targetElement, 'mouseout', this.unset.bind(this));
    },

    _loadContent: function() {
        var context = this;
        new Ajax.Request(Jsw.prepareUrl(this._url), {
            method: 'get',
            onSuccess: function(transport) {
                if ('' == transport.responseText) {
                    return;
                }

                context._content = transport.responseText;
                Jsw.AjaxTooltip.updateContent(context);
            }
        });

        this._contentLoaded = true;
    },

    setText: function(text)
    {
    	this._contentLoaded = true;
    	this._content = text;
    },

    set: function() {
        var context = this;
        this._delayedExecuter = new PeriodicalExecuter(function(pe) {
            Jsw.AjaxTooltip.set(context);
            pe.stop();
            this._delayedExecuter = null;
        }, Jsw.AjaxTooltip.delay);
    },

    unset: function() {
        if (this._delayedExecuter) {
            this._delayedExecuter.stop();
            this._delayedExecuter = null;
        }
    },

    getTitle: function() {
        return this._title;
    },

    getContent: function() {
        if (!this._contentLoaded) {
            this._loadContent();
        }

        return this._content;
    },

    getTargetElement: function() {
        return this._targetElement;
    },

    getCls: function() {
        return this._cls;
    }

});

Jsw.messageBox = {

    TYPE_OK: 'TYPE_OK',
    TYPE_YESNO: 'TYPE_YESNO',

    _messageBox: null,

    show: function(config) {
        if (!self._messageBox) {
            self._messageBox = new Jsw.MessageBox(config);
        } else {
            self._messageBox.initialize(config);
        }

        self._messageBox.show();
    }
}

Jsw.ModalDialogBox = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._screenDisablerId = this._getConfigParam('screenDisablerId', 'screenDisabler');

        if (!$(this._screenDisablerId)) {
            $(document.body).insert({
                top: '<div id="' + this._screenDisablerId + '" class="main-disabled-block" style="display: none;"></div>'
            });
        }

        this._screenDisabler = $(this._screenDisablerId);

        this._renderTargetId = this._getConfigParam('renderTo', 'modalDialogBox');

        if ($(this._renderTargetId)) {
            $(this._renderTargetId).remove();
        }

        this._screenDisabler.insert({ after: '<div id="' + this._renderTargetId + '"></div>' });
    },

    _makeCentered: function() {
        var windowElement = this._componentElement.down('div');
        var viewportTopOffset = document.viewport.getScrollOffsets()[1];
        var windowTopOffset = document.viewport.getHeight() / 2 - windowElement.getHeight() / 2;

        if (windowTopOffset > 0) {
            windowElement.setStyle({ top: (viewportTopOffset + windowTopOffset) + 'px' });
        }
    },

    show: function($super) {
        this._screenDisabler.show();
        $super();
        this._makeCentered();
    },

    hide: function($super) {
        $super();

        this._screenDisabler.hide();
    }

});

Jsw.MessageBox = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._type = this._getConfigParam('type', Jsw.messageBox.TYPE_YESNO);
        this._text = this._getConfigParam('text', '');
        this._subtype = this._getConfigParam('subtype', null);
        this._description = this._getConfigParam('description', '');
        this._descriptionWrapperTag = this._getConfigParam('descriptionWrapperTag', 'p');
        this._buttonTitles = this._getConfigParam('buttonTitles', null);

        this._onYesClick = this._getConfigParam('onYesClick', null);
        this._onNoClick = this._getConfigParam('onNoClick', null);
        this._onOkClick = this._getConfigParam('onOkClick', null);
        this._needAttention = this._getConfigParam('needAttention', false);
        this._needAttentionText = this._getConfigParam('needAttentionText', '');
        this._needAttentionBlockSubmit = this._getConfigParam('needAttentionBlockSubmit', false);

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
           if (this._onYesClick) {
               this._onYesClickHandler = this._onYesClick.bindAsEventListener(this);
           } else {
               this._onYesClickHandler = this._defaultOnButtonClick.bindAsEventListener(this);
           }
        }
    },

    _initComponentElement: function($super) {
        $super();

        var descriptionCode = '';
        if (Object.isArray(this._description)) {
            this._description.each(function(item) {
                descriptionCode += '<' + this._descriptionWrapperTag + '>' + item + '</' + this._descriptionWrapperTag +'>';
            })
        } else {
            descriptionCode += '<' + this._descriptionWrapperTag + '>' + this._description + '</' + this._descriptionWrapperTag + '>';
        }

        var needAttentionCode = '';
        if (this._needAttention) {
           needAttentionCode = '<hr>';
           if (this._needAttentionBlockSubmit) {
               needAttentionCode += '<p><span class="b-indent"><span class="b-indent-icon"><input type="checkbox" class="checkbox" id="needAttentionConfirmationCheckbox"></span><label for="needAttentionConfirmationCheckbox">' + this._needAttentionText + '</label></span></p>';
           } else {
               needAttentionCode += '<p class="hint-warning">' + this._needAttentionText + '</p>';
           }
           needAttentionCode += '<hr>';
        }

        var contentHtml =
            '<div class="confirmation-box">' +
                '<table cellspacing="0"><tr><td>' +
                    '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                        '<div class="confirmation-msg' + (this._subtype ? ' mw-' + this._subtype : '') + '">' +
                            '<h3>' + this._text + '</h3>' +
                            descriptionCode + needAttentionCode +
                            '<div class="btns-container">';

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            var buttonDefault = this._getConfigParam('buttonDefault', 'no');
            contentHtml += '<span class="btn' + ('yes' == buttonDefault ? ' action' : '') + '"><button type="submit" value="">' + this._buttonTitles.yes + '</button></span>';
            contentHtml += '<span class="btn' + ('no' == buttonDefault ? ' action' : '') + '"><button type="submit" value="">' + this._buttonTitles.no + '</button></span>';
        }

        if (Jsw.messageBox.TYPE_OK == this._type) {
            contentHtml += '<span class="btn action"><button type="submit" value="">' + this._buttonTitles.ok + '</button></span>';
        }

        contentHtml +=      '</div>' +
                        '</div>' +
                    '</div></div></div></div></div>' +
                '</td><tr></table>' +
            '</div>';

        this._updateComponentElement(contentHtml);

        this._componentElement.hide();
    },

    _disableYesButton: function() {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            this._componentElement.select('.btn').first().addClassName('disabled');
            Event.stopObserving(this._componentElement.select('button').first(), 'click', this._onYesClickHandler);
        }
    },

    _enableYesButton: function(button) {
        if (Jsw.messageBox.TYPE_YESNO == this._type) {
           this._componentElement.select('.btn').first().removeClassName('disabled');
           Event.observe(this._componentElement.select('button').first(), 'click', this._onYesClickHandler);
        }
    },

    _addEvents: function($super) {
        $super();

        // add default behavior - hide dialog
        this._componentElement.select('button').each(function(button) {
            Event.observe(button, 'click', this._defaultOnButtonClick.bindAsEventListener(this));
        }, this);

        if (Jsw.messageBox.TYPE_YESNO == this._type) {
            if (this._onYesClick) {
                var buttonYes = this._componentElement.select('button').first();
                Event.observe(buttonYes, 'click', this._onYesClickHandler);
            }

            if (this._onNoClick) {
                var buttonNo = this._componentElement.select('button').last();
                Event.observe(buttonNo, 'click', this._onNoClick.bindAsEventListener(this));
            }

           if (this._needAttention && this._needAttentionBlockSubmit) {
                var confirmationCheckbox = this._componentElement.select('input').last();
                var top = this;
                Event.observe(confirmationCheckbox, 'click',
                              function() { if ($F(confirmationCheckbox)) {top._enableYesButton()} else {top._disableYesButton()}} );
           }
        }

        if (Jsw.messageBox.TYPE_OK == this._type) {
            if (this._onOkClick) {
                var buttonOk = this._componentElement.select('button').first();
                Event.observe(buttonOk, 'click', this._onOkClick.bindAsEventListener(this));
            }
        }
    },

    _addTooltips: function() {
    },

    _defaultOnButtonClick: function() {
        this.hide();
    },

    show: function($super) {
        $super();
        this._componentElement.select('.action').first().select('button').first().focus();
        if (this._needAttention && this._needAttentionBlockSubmit) {
           this._disableYesButton();
        }
    }

});

Jsw.Popup = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._popupContentAreaId = this._id + '-popup-content-area';
        this._titleAreaId = this._id + '-title-area';
        this._actionButtonsId = this._id + '-action-buttons';
        this._leftActionButtonsAreaId = this._id + '-left-action-buttons-area';
        this._rightActionButtonsAreaId = this._id + '-right-action-buttons-area';

        this._titleCls = this._getConfigParam('titleCls', '');
        this._buttons = this._getConfigParam('buttons', []);
        this._onShow = this._getConfigParam('onShow', function(){});
    },

    _initComponentElement: function($super) {
        $super();
        this._updateComponentElement(
            '<div class="popup-panel"><table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<h2 class="' + this._titleCls + '"><span id="' + this._titleAreaId + '">' +
                                this._getConfigParam('title', '') +
                            '</span></h2>' +
                        '</div>' +
                    '</div>'+
                    '<div class="popup-content">' +
                        '<div class="popup-content-area" id="' + this._popupContentAreaId + '">' +
                            this._getContentArea() +
                        '</div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr><tr/></tbody></table></div>'
        );
    },

    _getContentArea: function() {
        return '' +
            '<form>' +
                this._getConfigParam('content', '') +
                this._getButtonsArea() +
            '</form>';
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
                '<div class="box-area">' +
                    '<div class="form-row">' +
                        '<div class="field-name" id="' + this._leftActionButtonsAreaId + '"> </div>' +
                        '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _addEvents: function($super) {
        $super();

        var context = this;

        this._buttons.each(function(buttonDesc) {
            var buttonContainer = new Element('span', { 'class': 'btn action' });
            buttonContainer.update('<button value="" type="button">' + buttonDesc['title'] + '</button>');
            buttonContainer.down('button').observe(
                'click',
                buttonDesc['handler'].bindAsEventListener(this, context)
            );
            $(context._rightActionButtonsAreaId).insert(buttonContainer);
        });

        this.show();
    },

    show: function($super) {
        $super();
        this._onShow();
    }

});

Jsw.DoubleListSelector = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = 'doubleListSelect';
        this._name = this._getConfigParam('name', '');
        this._initList();
        this._selectedMarked = 0;
        this._unselectedMarked = 0;
    },

    _initList: function() {
        this._list = this._getConfigParam('list', $A());
        this._list.each(function(element) {
            element.marked = false;
        });
    },

    _initComponentElement: function($super) {
        $super();
        var listHtml =
            '<div class="dls-first-box">' +
                '<h4><span>' + this._getConfigParam('unselectedTitle', '') + '</span></h4>' +
                '<div class="dls-box-area"><ul id="' + this._id + '-unselected"></ul></div>' +
            '</div>' +
            '<div class="dls-second-box">' +
                '<h4><span>' + this._getConfigParam('selectedTitle', '') +'</span></h4>' +
                '<div class="dls-box-area"><ul id="' + this._id + '-selected"></ul></div>' +
            '</div>' +
            '<div id="' + this._id + '-controls" class="dls-controls">' +
                '<span id="' + this._id + '-controls-select" class="btn disabled">' +
                    '<button id="'+ this._id +'-submitSelect" onclick="return false;"><span class="dls-add">&gt;&gt;</span></button>' +
                '</span>' +
                '<span id="' + this._id + '-controls-unselect" class="btn disabled">' +
                    '<button id="'+ this._id +'-submitUnselect" onclick="return false;"><span class="dls-remove">&lt;&lt;</span></button>' +
                '</span>' +
            '</div>';

        this._updateComponentElement(listHtml);

        this._list.each(function(element, index) {
            this._initListItem(element, index, true);
            this._initListItem(element, index, false);
            this._initInputElement(element, index);
        }, this);
    },

    _initListItem: function(element, index, isSelected) {
        var container = null;
        var isHidden = false;
        if (isSelected) {
            container = $(this._id + '-selected');
            isHidden = !element.selected;
        } else {
            container = $(this._id + '-unselected');
            isHidden = element.selected;
        }

        var listItem = new Element('li', {
            'id': this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item'
        })
        .insert('<b>' + element.title + '</b>')
        .insert('<span>' + element.description + '</span>');

        container.insert(listItem);
        var context = this;
        listItem.observe('click', function() {
            context._toggleItem(index, isSelected);
        });
        if (isHidden) {
            listItem.hide();
        }
    },

    _initInputElement: function(element) {
        this._componentElement.insert(new Element('input', {
            'id': this._id + '-' + element.id,
            'name': this._name + '[' + element.id + ']',
            'type': 'hidden',
            'value': (element.selected ? 'true' : 'false')
        }));
    },

    _addEvents: function() {
        $(this._id + '-controls-select').observe('click', this._onSubmitSelect.bind(this));
        $(this._id + '-controls-unselect').observe('click', this._onSubmitUnselect.bind(this));
    },

    _toggleItem: function(index, isSelected) {
        if (this._list[index].marked) {
            this._unmarkItem(index, isSelected);
        } else {
            this._markItem(index, isSelected);
        }
    },

    _markItem: function(index, isSelected) {
        this._list[index].marked = true;
        $(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').addClassName('marked');
        if (isSelected) {
            this._selectedMarked++;
            $(this._id + '-controls-unselect').removeClassName('disabled');
        } else {
            this._unselectedMarked++;
            $(this._id + '-controls-select').removeClassName('disabled');
        }
    },

    _unmarkItem: function(index, isSelected) {
        this._list[index].marked = false;
        $(this._id + '-' + index + '-' + (isSelected ? 'selected' : 'unselected') + '-item').removeClassName('marked');
        if (isSelected) {
            this._selectedMarked--;
            if (0 == this._selectedMarked) {
                $(this._id + '-controls-unselect').addClassName('disabled');
            }
        } else {
            this._unselectedMarked--;
            if (0 == this._unselectedMarked) {
                $(this._id + '-controls-select').addClassName('disabled');
            }
        }
    },

    _onSubmitSelect: function() {
        this._list.each(function(element, index) {
            if (element.marked && !element.selected) {
                this._unmarkItem(index, false);
                element.selected = true;
                $(this._id + '-' + element.id).value = 'true';
                $(this._id + '-' + index + '-unselected-item').hide();
                $(this._id + '-' + index + '-selected-item').show();
            }
        }, this);
    },

    _onSubmitUnselect: function() {
        this._list.each(function(element, index) {
            if (element.marked && element.selected) {
                this._unmarkItem(index, true);
                element.selected = false;
                $(this._id + '-' + element.id).value = 'false';
                $(this._id + '-' + index + '-selected-item').hide();
                $(this._id + '-' + index + '-unselected-item').show();
            }
        }, this);
    }
});

Jsw.ImageSelector = Class.create(Jsw.Component, {
    _initConfiguration: function($super, config) {
        $super(config);
        var list = this._getConfigParam('list', {});
        this._list = Object.isArray(list) ? $H() : $H(list);
        var captions = this._getConfigParam('captions', {});
        this._captions = Object.isArray(captions) ? $H() : $H(captions);
        this._name = this._getConfigParam('name', '');
        this._value = this._getConfigParam('value', null);

        this._currentId = this._id + '-current';
        this._listId = this._id + '-list';
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="' + this._cls + '-area">' +
                '<div id="' + this._currentId + '" class="' + this._cls + '-current">' +
                    '<img/>' +
                    '<p>' + this._captions.get('selectedImage').escapeHTML() + '</p>' +
                '</div>' +
                '<div id="' + this._listId + '" class="' + this._cls + '-select clearfix"></div>' +
            '</div>'
        );

        this._valueElement = new Element('input', {
            'id': this._id + '-value',
            'name': this._name,
            'type': 'hidden',
            'value': null
        });

        this._componentElement.insert(this._valueElement);
    },

    _addEvents: function() {
        this._list.each(function(pair) {
            this._addImage(pair.key, pair.value);
        }, this);

        this.set(this._value);
    },

    _addImage: function(value, image) {
        this._componentElement.select('#' + this._listId).each(function(target) {
            var element = new Element('img', {
                'src': image.url,
                'alt': image.title
            });

            target.insert(element);

            element.observe('click', this._onImageClick.bindAsEventListener(this, value));
        }, this);
    },

    _onImageClick: function(event, value)
    {
        this.set(value);
    },

    set: function(value)
    {
        this._valueElement.value = value;
        var image = this._list.get(value);
        this._componentElement.select('#' + this._currentId + ' img').each(function(target) {
            if (image) {
                target.src = image.url;
                target.alt = image.title;
                target.show();
            } else {
                target.hide();
            }
        }, this);

    }

});

Jsw.ToolList = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
		$super(config);
        this._tools = this._getConfigParam('tools', null);
        this._cls = this._getConfigParam('cls', 'tools-list-box');
        this._contentAreaId = this._id + '-tools-list';

        if (Object.isArray(this._tools) && (0 == this._operations.size())) {
            this._operations = null;
        }
    },

    _initComponentElement: function($super) {
    	$super();
        this._updateComponentElement(
            '<div id="' + this._id + '-box-area" class="box-area">' +
	            '<div id="' + this._id + '-content" class="content">' +
		            	'<div id="' + this._id + '-content-area" class="content-area">' +
		            		'<ul id="' + this._id + '-tools-list" class="tools-list"></ul>' +
		            	'</div>' +
	            '</div>' +
            '</div>'
        );
    },

    render: function($super) {
    	$super();
        Jsw.Tooltip.initData(this._componentElement);
    }
});

Jsw.toollist = {};

Jsw.toollist.Button = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._handler = this._getConfigParam('handler', false);
        if (Object.isString(this._handler)) {
            eval('this._handler = ' + this._handler);
        }
        this._href = this._getConfigParam('href', '#');
        this._target = this._getConfigParam('target', null);
        this._image = this._getConfigParam('image', '');
        this._disabledImage = this._getConfigParam('disabledImage', this._image);
        this._title = this._getConfigParam('title', '');
        this._description = this._getConfigParam('description', '');
        this._disabled = this._getConfigParam('disabled', false);
    },

    _initComponentElement: function($super) {
        this._componentElement = new Element('li', {
        	'class': 'tools-item'
        });

    	cls = 'tool-block';
    	if (this._disabled) {
    		cls += ' tool-block-disabled';
    		href = '#';
    		image = this.getImagePath(this._disabledImage);
    	} else {
    		href = this._href;
    		image = this.getImagePath(this._image);
    	}
        this._componentElement.update(new Element('a', {
	        	'class': cls,
	            'href': href,
	            'target': this._target
	        }).insert(
	            '<span class="tool-icon">'+
	            	'<img src="' + image + '"/>'+
	            '</span>'+
	            '<span class="tool-name">' + this._title + '</span>'+
	            '<span class="tool-info">' + this._description + '</span>'
	        )
        );

        if (this._handler && !this._disabled) {
        	this._componentElement.observe('click', this._handler);
        }
    },

    _addTooltips: function() {
        // don't show tooltips, 'cause we've inline descriptions
    },

    getImagePath: function(path) {
    	 return Jsw.skinUrl + "/" + path;
    }
});

Jsw.CustomButton =  Class.create(Jsw.toollist.Button, {
	getImagePath: function(path) {
		return path;
	}
});

Object.extend(Form.Element.Methods, {
    disable: function(element) {
        element = $(element);
        element.disabled = 'disabled';
        element.addClassName('disabled');
        return element;
    },

    enable: function(element) {
        element = $(element);
        element.disabled = false;
        element.removeClassName('disabled');
        return element;
    }
});

Element.addMethods();

Jsw.namespace('Jsw.passwordmeter');

Jsw.PasswordMeter = Class.create(Jsw.Component, {

	_initConfiguration: function($super, config) {
    	$super(config);
    	this._passwordElement = $(this._getConfigParam('observe', null));
    	this._passwordElement.observe('keyup', this._onChange.bind(this));
    	this._locale = this._getConfigParam('locale', {});
    	this._tag = 'span';
    	this._cls = 'password-strength';
	},

	_initComponentElement: function($super) {
    	$super();

		this._hintContainer = new Element('span', {cls: 'hint-inline hint-info'});
		this._hint = new Element('span');
		this._hint.insert('(?)');
		this._hintContainer.insert(this._hint);

		this._progress = new Element('i');
		this._componentElement.insert(this._progress);

		this._verdict = new Element('b');
		this._componentElement.insert(this._verdict);
	},

    render: function($super) {
        $super();

        if (!this._tooltip) {
	        this._componentElement.insert({'after':this._hintContainer});
	        this._tooltip = new Jsw.AjaxTooltip.Instance({
	                title: this._locale['title'],
	                waitMsg: '',
	                url: '',
	                target: this._hint
	            });
        }
        this._onChange();
    },

    setProgress: function(value)
    {
    	this._progress.writeAttribute('style', 'width: ' + value + '%');
    },

    _onChange: function() {
		var result = this._applyRules(this._passwordElement.value);
		this._updateProgress(result.score);
		this._updateHint(result.unusedRules);
		this._updateVerdict(result.score);
		this._updateColor(result.score);
	},

	_applyRules: function(passwd)
	{
		var unusedRules = new Array();
		var sum = 0;
		$H(this.rules).each(function(pair) {
			var mark = pair.value(passwd);
			if (mark < 0) {
				unusedRules.push({rule: pair.key, value: -mark});
			} else {
				sum += mark;
			}
	    });
		return {score: sum, unusedRules: unusedRules};

	},

	_verdictClasses: {
		verdictNone: 'verdictNone',
		verdictVeryWeak: 'password-strength-very-weak',
		verdictWeak: 'password-strength-weak',
		verdictMediocre: 'password-strength-medium',
		verdictStrong: 'password-strength-strong',
		verdictStronger: 'password-strength-very-strong'
	},

	_updateColor: function(score)
	{
		var verdict = this._getVerdict(score);
		var newClass = this._verdictClasses[verdict];
		if (this._oldClass == newClass ) {
			return;
		}
		if (this._oldClass) {
			this._componentElement.removeClassName(this._oldClass);
		}
		this._componentElement.addClassName(newClass);
		this._oldClass = newClass;
	},

	_updateHint: function(unusedRules)
	{
		var description = '';
		if (unusedRules.length) {
			description =  this._locale['description'] + '<br/><ul>'
			for(var i=0; i < unusedRules.length && i < 3; ++i) {
				description += "<li>" +  this._locale[unusedRules[i].rule] + "</li>";
			}
			description += "</ul>";
		} else {
			description += this._locale['yourPasswordIsStrong'];
		}

		this._tooltip.setText(description);
	},

	_updateVerdict: function(score)
	{
		this._verdict.update(this._locale[this._getVerdict(score)])
	},

	_updateProgress: function(score)
	{
		if (score > 45) {
			score = 45;
		}
		var value = ((score - 3) / 42) * 100;
		this.setProgress(value);
	},

	_getVerdict: function(score) {
		if(score < 1) {
			return "verdictNone"
		}
		else if(score < 16)
		{
		   return "verdictVeryWeak"
		}
		else if (score > 15 && score < 25)
		{
			return "verdictWeak"
		}
		else if (score > 24 && score < 35)
		{
			return "verdictMediocre"
		}
		else if (score > 34 && score < 45)
		{
			return "verdictStrong"
		}
		return "verdictStronger"
	},

	rules: {
		passwordTooShort: function(passwd) {
			if (passwd.length<5) {
				return -1;
			}
			return 0;
		},
		passwordLength: function(passwd) {
			// PASSWORD LENGTH
			if (passwd.length<5)                         // length 4 or less
			{
				return 3;
			}
			else if (passwd.length>4 && passwd.length<8) // length between 5 and 7
			{
				return 6;
			}
			else if (passwd.length>7 && passwd.length<16)// length between 8 and 15
			{
				return 12;
			}
			else           // length 16 or more
			{
				return 18;
			}
			return -1;
		},
		lettersLowerCase: function(passwd) {
			if (passwd.match(/[a-z]/))                              // [verified] at least one lower case letter
			{
				return 1;
			}
			return -1;
		},
		lettersUpperCase: function(passwd) {
			if (passwd.match(/[A-Z]/))                              // [verified] at least one upper case letter
			{
				return 5;
			}
			return -1;
		},
		numbers1: function(passwd) {
			if (passwd.match(/\d+/))                                 // [verified] at least one number
			{
				return 5;
			}
			return -1;
		},
		numbers3: function (passwd) {
			if (passwd.match(/(.*[0-9].*[0-9].*[0-9])/))             // [verified] at least three numbers
			{
				return 5;
			}
			return -1;
		},
		specialChar1: function(passwd) {
			if (passwd.match(/.[!,@,#,$,%,^,&,*,?,_,~]/))            // [verified] at least one special character
			{
				return 1;
			}
			return -1;
		},
		specialChar2: function(passwd) {
			// [verified] at least two special characters
			if (passwd.match(/(.*[!,@,#,$,%,^,&,*,?,_,~].*[!,@,#,$,%,^,&,*,?,_,~])/))
			{
				return 5;
			}
			return -1;
		},
		comboUpperAndLower: function(passwd) {
			if (passwd.match(/([a-z].*[A-Z])|([A-Z].*[a-z])/))        // [verified] both upper and lower case
			{
				return 2;
			}
			return -1;
		},
		comboLettersAndNumbers: function(passwd) {
			if (passwd.match(/([a-zA-Z])/) && passwd.match(/([0-9])/)) // [verified] both letters and numbers
			{
				return 2;
			}
			return -1;
		},
		comboLettersNumbersSpecial: function(passwd) {
			// [verified] letters, numbers, and special characters
			if (passwd.match(/([a-zA-Z0-9].*[!,@,#,$,%,^,&,*,?,_,~])|([!,@,#,$,%,^,&,*,?,_,~].*[a-zA-Z0-9])/))
			{
				return 2;
			}
			return -1;
		}
	}
});

Jsw.Promo = Class.create(Jsw.Container, {
	_tag: 'div',
	_rows: 3,

    _initConfiguration: function($super, config) {
		$super(config);
        this._title = this._getConfigParam('title', '');
        this._locale = this._getConfigParam('locale', {});
        this._cls = this._getConfigParam('text', 'p-box home-promo-block');
        this._contentAreaId = this._id + '-content-area';
        this._itemCount = 0;
        this._current = 0;
    },

    _initComponentElement: function($super) {
    	$super();
        this._componentElement.update(
        		'<div class="p-box home-promo-block">' +
        		'<div class="p-box-tl"><div class="p-box-tr"><div class="p-box-tc"><!----></div></div></div>' +
        		'<div class="p-box-ml"><div class="p-box-mr"><div class="p-box-mc">' +
        		'<div class="p-box-header">' +
        		  '<div class="p-box-header-wrap">' +
        		    '<div class="p-box-header-wrap-inner">' +
        		      '<div class="p-box-header-nav">' +
        		        '<a class="link-01" id="' + this._id + '-prev" href="#"><span>' + this._locale.prev + '</span></a> ' +
        		        '<a class="link-01" id="' + this._id + '-next" href="#"><span>' + this._locale.next + '</span></a>' +
        		      '</div>' +
        		      '<div class="p-box-header-text">' +
        		        this._title +
        		      '</div>' +
        		    '</div>' +
        		  '</div>' +
        		'</div>' +
        		'<div class="p-box-content">' +
        		  '<ul class="tools-list" id="' + this._contentAreaId + '">' +
        		  '</ul>' +
        		'</div>' +
        		'</div></div></div>' +
        		'<div class="p-box-bl"><div class="p-box-br"><div class="p-box-bc"><!----></div></div></div>' +
        		'</div>'
        );
    },

    _renderItems: function() {
    	this._showCurrentItems();
    },

    _onHide: function(item) {
    	for (var i=0; i < this._items.length; ++i) {
    		if (this._items[i] == item) {
    			this._items.splice(i, 1);
    			break;
    		}
    	}

    	this._showCurrentItems();
    },

    _addEvents: function() {
    	$(this._id + '-next').observe('click', this._onNext.bind(this));
    	$(this._id + '-prev').observe('click', this._onPrev.bind(this));
    },

    _onNext: function() {
		this._current += this._rows;
		this._showCurrentItems();
    },

    _onPrev: function() {
		this._current += this._rows;
		this._showCurrentItems();
    },

    _clampCurrentItem: function () {
    	if (this._items.length > this._rows) {
    		$(this._id + '-prev').show();
    		$(this._id + '-next').show();
    	} else {
    		$(this._id + '-next').hide();
    		$(this._id + '-prev').hide();
    	}

    	if (this._current >= this._items.length) {
    		this._current = 0;
    	} else if (this._current < 0) {
    		this._current = this._items.length - this._items.length % this._rows;
    	}
    },

    _showCurrentItems: function() {
    	if (this._items.length == 0) {
    		this.hide();
    		return;
    	}

    	this._clampCurrentItem();

    	for (var i=0; i < this._items.length; ++i) {
    		var item = this._items[i];
            item.setRenderTarget(this._contentAreaId);
            item.setHideHandler(this._onHide.bind(this));
    		item.hide();
    		if (i >= this._current && i < this._current + this._rows) {
    			item.render();
    			item.show();
    		}
    	}
    }
});


Jsw.Promo.Item = Class.create(Jsw.Component, {

	_tag: 'li',

    _initConfiguration: function($super, config) {
        $super(config);

        this._text = this._getConfigParam('text', '');
        this._title = this._getConfigParam('title', '');
        this._buttonUrl = this._getConfigParam('buttonUrl', '');
        this._buttonText = this._getConfigParam('buttonText', '');
        this._buttonTarget = this._getConfigParam('buttonTarget', '');
        this._hideUrl = this._getConfigParam('hideUrl', '');
        this._hideText = this._getConfigParam('hideText', '');
        this._iconUrl = this._getConfigParam('iconUrl', '');
        this._cls = "tools-item";
    },

    render: function($super) {
    	this._componentElement.update(
			'<a class="tool-block" href="' + this._buttonUrl + '" target="' + this._buttonTarget + '">' +
				'<span class="tool-icon"><img src="' + this._iconUrl + '"></span>' +
				'<span class="tool-name">' + this._title.escapeHTML() + '</span>' +
				'<span class="tool-info">' + this._text.escapeHTML() + '</span>' +
			'</a>' +
			'<div class="tool-actions">' +
				'<a href="' + this._buttonUrl + '" class="m-btn" target="' + this._buttonTarget + '"><i><i><i><span>' + this._buttonText.escapeHTML() + '</span></i></i></i></a>' +
				'<a href="#" class="link-02" id="' + this._id + '-hide"><span>' + this._hideText.escapeHTML() + '</span></a>' +
			'</div>'
        );
    	$super();
    },

    setHideHandler: function (handler) {
    	this._hideHandler = handler;
    },

    _onClose: function(event)
    {
    	event.stop();
        this._hideHandler(this);
        this._componentElement.remove();
        if (this._hideUrl) {
        	new Ajax.Request(this._hideUrl, { method: 'post' });
        }
        return false;
    },

    _addEvents: function() {
    	var el = $(this._id + '-hide');
    	if (el) {
    		el.observe('click', this._onClose.bind(this));
    	}
    }
});

