Smb.FormApsCatalog = Class.create(Jsw.FormAjax, {

    _isSearchResultsShown: false,
    _catalogType: 'apscatalog',

    _initConfiguration: function($super, config) {
        $super(config);
        this._locale = this._getConfigParam('locale', null);
        Jsw.baseUrl = this._getConfigParam('baseUrl', '');
        this._catalogType = $('catalogType').value;

        this._resizeCallback = this._getConfigParam('resizeCallback', null);

        if (Object.isString(this._resizeCallback) && this._resizeCallback) {
            eval('this._resizeCallback = ' + this._resizeCallback);
        }
    },

    _initComponentElement: function($super) {
        $super();

        element = Element('div', {
            'class': 'objects-content'
        });
        element.appendChild(
            Element('div', {
                'id': 'overview-catalog',
                'class': 'objects-content-area'
            })
        );
        element.appendChild(
            Element('div', {
                'id': 'category-info',
                'class': 'objects-content-area'
            })
        );
        element.appendChild(
            Element('div', {
                'id': 'details-info',
                'class': 'objects-content-area'
            })
        );

        this._componentElement.appendChild(this._getSearchBox());
        this._componentElement.appendChild(element);
    },

    _executeResizeCallback: function($super) {
        if (this._resizeCallback) {
            this._resizeCallback();
        }
    },

    render: function($super) {
        $super();
        this._addSearchHandlers();
        this._initBlocksResize();

        if ('apscatalog' == this._catalogType) {
            this.showCategories();
        } else {
            this.showSearchResults();
        }

        var category = this._getConfigParam('category');
        if (category) {
            this.showCategory(category);
        }

        var details = this._getConfigParam('details');
        if (details) {
            this.showDetails(details);
        }

        this._executeResizeCallback();
    },

    _initializeOverviewScreen: function() {
        this._showPanel('overview-catalog');
        Element.update($('current-action-navigation'), this._locale.selectCategory);
    },

    _showPanel: function(panel) {
        if (panel == 'overview-catalog') {
            $('overview-catalog').show();
        } else {
            $('overview-catalog').hide();
        }

        if (panel == 'category-info') {
            $('category-info').show();
        } else {
            $('category-info').hide();
        }

        if (panel == 'details-info') {
            $('details-info').show();
            $('current-action-navigation').hide();
            $('current-action-buttons').show();
            $('search-control').hide();
        } else {
            $('details-info').hide();
            $('current-action-navigation').show();
            $('current-action-buttons').hide();
            $('search-control').show();
        }

        if (('marketplace' == this._catalogType) && ('category-info' == panel)) {
            $('catalog-actions-box').setStyle({ display: 'none' });
        } else {
            $('catalog-actions-box').setStyle({ display: '' });
        }

        this._executeResizeCallback();
    },

    returnToHome: function() {
        this._initializeOverviewScreen();
        return false;
    },

    returnToCategory: function() {
        this._showPanel('category-info');
        return false;
    },

    showCategories: function() {
        this._initializeOverviewScreen();

        Element.update($('overview-catalog'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        new Ajax.Updater("overview-catalog", Jsw.baseUrl + "/app/categories-data/catalogId/" + $('catalogId').value, {
            method: 'get'
        });

        return false;
    },

    showDetails: function(downloadItem) {
        Element.update($('details-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        this._showPanel('details-info');

        var applicationDetails = $H(downloadItem);
        new Ajax.Updater({success: "details-info"}, Jsw.baseUrl + "/app/details-data/catalogId/" + $('catalogId').value, {
            method: 'post',
            parameters: { applicationDetails: applicationDetails.toJSON() },
            evalScripts: true,
            onFailure: this.onErrorShowDetails.bind(this, downloadItem)
        });
        return false;
    },

    onErrorShowDetails: function(downloadItem) {
        $('details-info').update(this._locale.connectionFailed.replace('%%retryLink%%', '<a href="#">' + this._locale.retry + '</a>'));
        $('details-info').select('a').each(function(element) {
            element.observe('click', this.showDetailsRetry.bind(this, downloadItem));
        }, this);
    },

    showDetailsRetry: function(downloadItem) {
        this.showDetails(downloadItem);
    },

    showCategory: function(catExternal) {
        this.resetSearchResults(false);

        this._showPanel('category-info');

        Element.update($('category-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        var context = this;

        new Ajax.Request(Jsw.baseUrl + "/app/applications-list-data/catalogId/" + $('catalogId').value, {
            method: 'get',
            parameters: { category: catExternal, cert: 'any' },
            onSuccess: function(transport) {
                try {
                    var result = transport.responseText.evalJSON();

                    if (result['list']) {
                        Element.update($('category-info'), result['list']);
                    }
                    if (result['navigation']) {
                        Element.update($('current-action-navigation'), result['navigation']);
                    }

                    context._executeResizeCallback();
                } catch(e) {
//                    alert('showCategory: ' + e);
                }
            }
        });

        return false;
    },

    showCategoryRetry: function(catExternal) {
        this.showCategory(catExternal);
    },

    install: function(item) {
        $('name').value = item['name'];
        $('version').value= item['version'];
        $('release').value = item['release'];
        $('vendor').value = item['vendor'];
        $('packager').value = item['packager'];
        this._componentElement.action = Jsw.baseUrl + "/app/download/";
        this._componentElement.submit();
        return false;
    },

    uninstall: function(packageId) {
       $('packageId').value = packageId;
       this._componentElement.action = Jsw.baseUrl + "/app/remove/";
       this._componentElement.submit();
       return false;
    },

    _getSearchBox: function() {
        var searchStateClass = '';

        if ('true' != Jsw.Cookie.get(this._id + '-search-show')) {
            searchStateClass = ' hide';
        }

        var div = Element('div', {
            'id': 'catalog-actions-box',
            'class': 'actions-box' + searchStateClass
        }).update(
                '<div class="search-box">' +
                    '<ul>' +
                        '<li><div><input id="search-name" type="text" value="' + this._locale.searchName + '"/></div></li>' +
                        '<input id="search-vendor" type="hidden" value=""/>' +
                        '<li><div><input id="search-packager" type="text" value="' + this._locale.searchPackager + '"/></div></li>' +
                    '</ul>' +

                    '<a class="s-btn sb-search" href="#">' +
                        '<i><i><i><span>' + this._locale.buttonSearch + '</span></i></i></i>' +
                    '</a>' +
                    '<a class="s-btn sb-show-all" href="#">' +
                        '<i><i><i><span>' + this._locale.buttonResetSearch + '</span></i></i></i>' +
                    '</a>' +
                '</div>' +
                '<div class="action-btns-block">' +
                    '<div class="action-btns-area">' +
                        '<span id="search-control" class="search-control">' +
                            '<a class="s-btn sb-search-show" href="#">' +
                                '<i><span>' + this._locale.showSearch + '</span></i>' +
                            '</a>' +
                            '<a class="s-btn sb-search-hide" href="#">' +
                                '<i><span>' + this._locale.hideSearch + '</span></i>' +
                            '</a>' +
                        '</span>' +
                        '<span id="current-action" class="list-actions">' +
                            '<span id="current-action-navigation"></span>' +
                            '<span id="current-action-buttons">' +
                                '<span class="link-back-to-category">' +
                                    '<a href="#tolist">' + this._locale.returnToCategories + '</a>' +
                                '</span>' +
                            '</span>' +
                        '</span>' +
                    '</div>' +
                '</div>'

        );

        return div;
    },

    _addSearchHandlers: function() {
        var context = this;

        this._componentElement.select('.search-control .s-btn').each(function(element) {
            element.observe('click', function() {
                Jsw.Cookie.set(context._id + '-search-show', this.up('.actions-box').hasClassName('hide'));
                this.up('.actions-box').toggleClassName('hide');
                context._executeResizeCallback();
                return false;
            });
        });

        var searchLink = this._componentElement.select('.sb-search').first();
        searchLink.observe('click', function() {
            context.showSearchResults();
        });

        var resetSearchLink = this._componentElement.select('.sb-show-all').first();
        resetSearchLink.observe('click', function() {
            context.resetSearchResults();
        });

        var searchFields = $w('search-name search-vendor search-packager');

        searchFields.each(function(searchFieldName) {
            var defaultValue = $(searchFieldName).value;

            $(searchFieldName).addClassName('input-default-value');

            $(searchFieldName).observe('focus', function() {
                if ($(searchFieldName).hasClassName('input-default-value')) {
                    $(searchFieldName).removeClassName('input-default-value');
                    $(searchFieldName).value = '';
                }
            });

            $(searchFieldName).observe('blur', function() {
                if ('' == $(searchFieldName).value) {
                    context._initSearchField(searchFieldName, defaultValue);
                }
            });

            $(searchFieldName).observe('keydown', function(event) {
                // search by Enter keydown
                if (13 == event.keyCode) {
                    context.showSearchResults();
                    Event.stop(event);
                    return false;
                }
            });
        });

        this._componentElement.select('.link-back-to-category a').each(function(element) {
            element.observe('click', this.returnToCategory.bind(this));
        }, this);
    },

    _initSearchField: function(searchFieldName, defaultValue) {
        $(searchFieldName).addClassName('input-default-value');
        $(searchFieldName).value = defaultValue;
    },

    showSearchResults: function() {
        this._showPanel('category-info');
        Element.update($('current-action-navigation'), this._locale.searchResults);

        Element.update($('category-info'), '<div class="ajax-loading">' + this._locale.loading + '</div>');

        var searchName = $('search-name').hasClassName('input-default-value') ? '' : $('search-name').value;
        var searchVendor = $('search-vendor').hasClassName('input-default-value') ? '' : $('search-vendor').value;
        var searchPackager = $('search-packager').hasClassName('input-default-value') ? '' : $('search-packager').value;

        new Ajax.Updater('category-info', Jsw.baseUrl + "/app/search-data/catalogId/" + $('catalogId').value, {
            method: 'get',
            parameters: {
                name: searchName,
                vendor: searchVendor,
                packager: searchPackager,
                cert: 'any'
            },

            onComplete: function() {
                Smb.Views.Apps.fixContentWidth();
                Smb.Views.Apps.fixMarketplaceContentHeight();
            }
        });

        this._isSearchResultsShown = true;

        return false;
    },

    resetSearchResults: function(full) {
        this._initSearchField('search-name', this._locale.searchName);
        this._initSearchField('search-vendor', this._locale.searchVendor);
        this._initSearchField('search-packager', this._locale.searchPackager);

        if (!this._isSearchResultsShown) {
            return ;
        }

        this._isSearchResultsShown = false;

        this.returnToHome();
    },

    // resizable height for div elements
    _initBlocksResize: function() {
        try {
            // this needs to be executed twice for proper work on Firefox
            this._resizeBlocks();
            this._resizeBlocks();
            var context = this;
            Event.observe(window, 'resize', function(){
              context._resizeBlocks();
              context._resizeBlocks();
            }, false);
        } catch (e) {
//            alert(e.description);
        }
    },

    _resizeBlocks: function() {
        try {
            var bodyHeight = document.body.clientHeight - 12;
            if (Prototype.Browser.IE) {
                bodyHeight = document.documentElement.clientHeight - 12;
            }
            var blockTop = Element.cumulativeOffset($('application-info'))[1];
            var blockBottom = Element.getHeight("footer-wrapper");
            var newBlockHeight = bodyHeight - blockTop - blockBottom;

            if (newBlockHeight < 200) { newBlockHeight = 200; } // minimum height

            $('application-info').style.height = newBlockHeight + 'px';
            $('category-select').style.height = newBlockHeight + 'px';

        } catch (e) {
//            alert(e.description);
        }
    }
});

Smb.ApsCatalogDownloadBar = Class.create(Jsw.Component, {
    STATUS_STARTED: 0,
    STATUS_NOT_STARTED: 1,
    STATUS_DONE: 2,
    STATUS_ERROR: 3,

    _initConfiguration: function($super, config) {
        $super(config);
        this._locale = this._getConfigParam('locale', null);
        Jsw.baseUrl = this._getConfigParam('baseUrl', '');
        var context = this;
        window.setTimeout(function(){context.updateProgress();}, 1000);
    },

    _initComponentElement: function() {
        this._progressTextElement = new Element('div', {
            'class': 'progress-counter'
        });
        //TODO: define progress bar element
        this._progressBarElement = new Element('div', {
            'class': 'progress-bar'
        }).update('<div><span></span></div>');
        this._componentElement = new Element('div', {
            'class': this._cls
        });
        this._componentElement.appendChild(this._progressTextElement);
        this._componentElement.appendChild(this._progressBarElement);
        this._update(0);
    },

    _update: function(percents) {
        this._progressTextElement.update(this._locale.progress.sub('%%percents%%', percents));
        this._progressBarElement.select('div').each(function(element) {
            $(element).setStyle({
                width: percents + '%'
            })
        });
    },

    updateProgress: function() {
        var context = this;
        var requestUrl = Jsw.baseUrl + "/app/download-progress-data/catalogId/"
            + this._getConfigParam('catalogId') + "/taskId/" + this._getConfigParam('taskId');

        new Ajax.Request(requestUrl, {
            method: 'get',
            onSuccess: function(transport) {
                try {
                    var result = transport.responseText.evalJSON();
                    if (result['status'] == context.STATUS_STARTED ||
                        result['status'] == context.STATUS_NOT_STARTED ||
                        result['status'] == context.STATUS_DONE) {

                        context._update(result['progress']);

                        if (result['status'] == context.STATUS_DONE) {

                            var redirectUrl = context._getConfigParam('redirectUrl').replace("%packageId%", result['packageId']);

                            if (context._getConfigParam('redirectMethod') == 'post') {
                                Jsw.redirectPost(redirectUrl);
                            } else {
                                Jsw.redirect(redirectUrl);
                            }
                        } else {
                            window.setTimeout(function(){context.updateProgress();}, 1000);
                        }
                    } else {
                        alert('Internal error occured during download.');
                        Jsw.redirect("/app");
                    }
                } catch (e) {
//                    alert('updateProgress exception: ' + e);
                }
            }
       });
    }
});