Jsw.namespace('Smb.FileSharing');

Smb.FileSharing = {

    getFileUrl: function(storageUrl, folderPath, fileName) {
        var uri = '';

        $A(folderPath.split('/')).each(
            function(item) {
                if('' == item) {
                    return;
                }
                uri += encodeURIComponent(item) + '/';
            }
        );

        if (fileName) {
            uri += encodeURIComponent(fileName);
        }

        return storageUrl + uri;
    },

    trimLastSlash: function(str) {
        while (str[str.length-1] == '/') {
            str = str.substring(0, str.length-1);
        }
        return str;
    },

    collapseExpandedWidgets: function(skip) {
        var widget = Jsw.getComponent('files-list-additional-buttons');
        if (!(skip && ('files-list-additional-buttons' == skip)) && widget) {
            widget.collapse();
        }

        widget = Jsw.getComponent('files-list-pathbar-component-popup-box');
        if (!(skip && ('files-list-pathbar-component-popup-box' == skip)) && widget) {
            widget.hide();
        }

        widget = Jsw.getComponent('files-list-access-info-popup');
        if (!(skip && ('files-list-access-info-popup' == skip)) && widget) {
            widget.hide();
        }
    }
}

Smb.FileSharing.Popup = Class.create(Jsw.ModalDialogBox, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._popupContentAreaId = this._id + '-popup-content-area';
        this._titleAreaId = this._id + '-title-area';
        this._hintAreaId = this._id + '-hint-area';
        this._hint1AreaId = this._id + '-hint1-area';
        this._boxAreaId = this._id + '-box-area';
        this._contentAreaId = this._id + '-content-area';

        this._actionButtonsId = this._id + '-action-buttons';
        this._leftActionButtonsAreaId = this._id + '-left-action-buttons-area';
        this._rightActionButtonsAreaId = this._id + '-right-action-buttons-area';

        this._titleCls = this._getConfigParam('titleCls', '');
        this._isRemoved = false;
    },

    _initComponentElement: function($super) {
        $super();
        this._updateComponentElement(
            '<table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<h2 class="' + this._titleCls + '"><span id="' + this._titleAreaId + '">' +
                                this._getConfigParam('title', '') +
                            '</span></h2>' +
                        '</div>' +
                    '</div>'+
                    '<div class="popup-content">' +
                        '<div class="popup-content-area" id="' + this._popupContentAreaId + '">' +
                            this._getContentArea() +
                        '</div>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</td></tr><tr/></tbody></table>'
        );
        this._screenDisabler.show();
    },

    _getContentArea: function() {
        return '' +
            '<p id="' + this._hint1AreaId + '"></p>' +
            '<span id="' + this._hintAreaId + '"></span>' +
            this._getBoxArea() +
            this._getButtonsArea();
    },

    _getBoxArea: function() {
        return '' +
            '<div class="form-box" id="' + this._boxAreaId + '">' +
                '<div class="box-area">' +
                    '<div class="content">' +
                        '<div class="content-area" id="' + this._contentAreaId + '">' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _getButtonsArea: function() {
        return '' +
            '<div class="btns-box" id="' + this._actionButtonsId + '">' +
                '<div class="box-area">' +
                    '<div class="form-row">' +
                        '<div class="field-name" id="' + this._leftActionButtonsAreaId + '"> </div>' +
                        '<div class="field-value" id="' + this._rightActionButtonsAreaId + '"> </div>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    setBoxType: function(type) {
        $(this._boxAreaId).className = type;
        if ('list-box' == type) {
            $(this._actionButtonsId).addClassName('no-separator');
            $(this._actionButtonsId).addClassName('simple-box');
        } else {
            $(this._actionButtonsId).removeClassName('no-separator');
            $(this._actionButtonsId).removeClassName('simple-box');
        }
    },

    setTitleType: function(cls) {
        $(this._titleAreaId).parentNode.className = cls;
    },

    setTitle: function(text) {
        $(this._titleAreaId).update(text);
    },

    setHint: function(text) {
        if ('' == text) {
            $(this._hintAreaId).hide();
        } else {
            $(this._hintAreaId).update(text);
            $(this._hintAreaId).show();
        }
    },

    setHint1: function(text) {
        if ('' == text) {
            $(this._hint1AreaId).hide();
        } else {
            $(this._hint1AreaId).update(text);
            $(this._hint1AreaId).show();
        }
    },

    remove: function() {
        this._isRemoved = true;
        $(this._renderTargetId).remove();
    },

    addRightButton: function(title, handler, isDefault, isAction) {
        var button = this._createButton(title, handler, isDefault, isAction);
        $(this._rightActionButtonsAreaId).insert(button);
        return button;
    },

    addLeftButton: function(title, handler, isDefault, isAction) {
        var button = this._createButton(title, handler, isDefault, isAction);
        $(this._leftActionButtonsAreaId).insert(button);
        return button;
    },

    removeRightButtons: function() {
        $(this._rightActionButtonsAreaId).update(' ');
    },

    removeLeftButtons: function() {
        $(this._leftActionButtonsAreaId).update(' ');
    },

    _createButton: function(title, handler, isDefault, isAction) {
        var buttonContainer = new Element('span');

        buttonContainer.addClassName('btn');
        if (isAction) {
            buttonContainer.addClassName('action');
        }

        buttonContainer.update(
            '<button value="" type="' + (isDefault ? 'submit' : 'button') + '">' +
                title +
            '</button>'
        );
        buttonContainer.down('button').observe(
            'click',
            this._onClick.bindAsEventListener(this, handler.bind(this))
        );

        return buttonContainer;
    },

    _updateButton: function(element, config) {
        $H(config).each(function(pair) {
            switch (pair.key) {
            case 'title':
                element.down('button').update(pair.value);
                break;
            case 'disabled':
                element.down('button').disabled = pair.value ? true : false;
                pair.value ? Element.addClassName(element, 'disabled')
                           : Element.removeClassName(element, 'disabled');
                break;
            }
        });
    },

    _onClick: function(event, handler) {
        Event.stop(event);
        handler();
    },

    _addEvents: function($super) {
        $super();

        if ($(this._hintAreaId)) {
            $(this._hintAreaId).hide();
        }
        if ($(this._hint1AreaId)) {
            $(this._hint1AreaId).hide();
        }
    },

    _clearMessages: function() {
        var contentElement = $(this._popupContentAreaId);

        ['.msg-error', '.msg-info', '.msg-warning'].each(function(msgClass) {
            contentElement.select(msgClass).each(function(messageElement) {
                messageElement.remove();
            });
        });
    },

    _addErrorMessage: function(message) {
        $(this._popupContentAreaId).insert(
            {
                top:
                    '<div class="msg-box msg-error">' +
                        '<div>' +
                            '<div>' +
                                '<div>' +
                                    '<div>' +
                                        '<div>' +
                                            '<div class="msg-content">' +
                                                message +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>'
            }
        );
    },

    _userError: function(error) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', error);
        this._close();
    },

    _internalError: function(error) {
        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', 'Internal error: ' + error);
        this._close();
    }

});

Smb.FileSharing.ListPopup = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._itemsAreaId = this._id + '-items';
    },

    _addEvents: function($super) {
        $super();
        Event.observe(
            window,
            'resize',
            this.resizeList.bind(this),
            false
        );
    },

    resizeList: function() {
        if (this._isRemoved) {
            return;
        }

        var minHeigh = 150;
        var list = $(this._contentAreaId).down('table');
        var listContainer = $(this._contentAreaId).down('.list');

        var bodyHeight = (Prototype.Browser.IE)
            ? $(document.documentElement).getHeight()
            : $(document.body).getHeight();

        var buttonsHeight = $(this._actionButtonsId).getHeight();
        var top = Position.cumulativeOffset(list)[1];
        var height = list.getHeight();
        var newHeight = bodyHeight - top - buttonsHeight - 100; // 100px below dialog

        if (newHeight > height) {
            listContainer.style.height = '';
            listContainer.removeClassName('with-scroll');
            return;
        }

        if (newHeight < minHeigh) {
            newHeight = minHeigh;
        }

        listContainer.style.height = newHeight + 'px';
        listContainer.style.overflowY = 'auto';
        listContainer.style.overflowX = 'hidden';
        listContainer.addClassName('with-scroll');
    },

    _renderContentArea: function() {
        $(this._contentAreaId).insert (
            '<div class="list">' +
                '<table width="100%" cellspacing="0">' +
                    '<tbody id="' + this._itemsAreaId + '">' +
                    '</tbody>' +
                '</table>' +
            '</div>'
        );
    },

    render: function($super) {
        $super();
        this._renderContentArea();
    }
});

Smb.FileSharing.Explorer = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._typesAreaId =  this._id + '-types-area';
        this._foldersAreaId =  this._id + '-folders-area';

        this._cls = this._getConfigParam('cls', 'explorer-block');
        this._type = this._getConfigParam('type', null);
        this._storageTabs = $H(this._getConfigParam('storageTabs', {}));
        this._storageTypes = $H(this._getConfigParam('storageTypes', {}));
        this._folder = this._getConfigParam('folder');
        this._locale = this._getConfigParam('locale', {});
        this._onSelectFolder = this._getConfigParam('onSelectFolder', null);
        this._onDeselectFolder = this._getConfigParam('onDeselectFolder', null);

        this._typeControl = null;
        this._typeTrees = $H();
        this.reset();
    },

    reset: function() {
        if (this._typeControl) {
            this._typeControl.reset();
        }

        this._typeTrees.each(
            function(pair) {
                pair.value.hide();
            }
        );

        if ($(this._foldersAreaId)) {
            $(this._foldersAreaId).update('<div class="tree-box"></div>');
        }
        this._typeTrees = $H();

        this._onDeselectFolderWrapper();
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<div id="' + this._typesAreaId + '" class="folders-list"></div>' +
            '<div id="' + this._foldersAreaId + '"><div class="tree-box"></div></div>'
        );
    },

    render: function($super) {
        $super();

        this._initTypeControl();
    },

    _initTypeControl: function() {
        this._typeControl = new Smb.FileSharing.TypesList({
            renderTo: this._typesAreaId,
            items: this._storageTabs,
            value: this._type,
            onClick: this._select.bind(this)
        });

        if (null != this._folder) {
            this._select(this.getType(), this._folder);
        }
    },

    _select: function(type, folder) {

        this._typeTrees.each(
            function(pair) {
                pair.value.hide();
            }
        );

        if (1 > this._typeTrees.keys().length) {
            $(this._foldersAreaId).update('');
        }

        var tree = this._typeTrees.get(type);
        if (!tree) {
            this._onDeselectFolderWrapper();
            tree = this._makeTree(type, folder);
            this._typeTrees.set(type, tree);
        } else {
            tree.show();
            if (Object.isFunction(this._onSelectFolder)) {
                this._onSelectFolder();
            }
        }
    },

    _onDeselectFolderWrapper: function() {
        if (Object.isFunction(this._onDeselectFolder)) {
            this._onDeselectFolder();
        }
    },

    _makeTree: function(type, folder) {
        return new Smb.FileSharing.TreeBox({
            renderTo: this._foldersAreaId,
            fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
            itemsSource: Jsw.prepareUrl('/file/tree-data/type/' + encodeURIComponent(type)),
            locale: this._locale,
            onLoad: this._onSelectFolder,
            onSelectFolder: this._onSelectFolder,
            value: folder
        });
    },

    getType: function() {
        return this._typeControl.getValue();
    },

    setType: function(type) {
        this._typeControl.setValue(type);
    },

    getTypeTitle: function() {
        var listControl = Jsw.getComponent('files-list');
        return listControl.getStorageTitle(this.getType());
    },

    getEscapedFolderTitle: function() {
        var tree = this._typeTrees.get(this.getType());
        if (!tree) {
            return null;
        }

        var value = tree.getValue();
        if ('' == value) {
            return this.getTypeTitle();
        }

        return $A(value.split('/')).last().escapeHTML();
    },

    getEscapedPathTitle: function() {
        var tree = this._typeTrees.get(this.getType());
        if (!tree) {
            return null;
        }

        var value = tree.getValue();
        if ('' == value) {
            return this.getTypeTitle();
        }

        return this.getTypeTitle() + '/' + value.escapeHTML();
    },

    getFileUrl: function(file) {
        return Smb.FileSharing.getFileUrl(
            this._storageTypes.get(this.getType()).url,
            this.getFolder(),
            file
        );
    },

    makeRelativePath: function(file) {
        var currentFolder = this.getFolder();
        return ('' == currentFolder)
            ? file
            : Smb.FileSharing.trimLastSlash(currentFolder) + '/' + file;
    },

    getFolder: function() {
        var tree = this._typeTrees.get(this.getType());
        if (!tree) {
            return null;
        }

        return tree.getValue();
    }
});

Smb.FileSharing.PredefinedExplorer = Class.create(Smb.FileSharing.Explorer, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'form-row');
        this._title = this._getConfigParam('title', '');
    },

    reset: function($super) {
        $super();

        if ($(this._foldersAreaId)) {
            this._componentElement.down('.fs-select-block').addClassName('hide');
            this._select(this.getType(), this._folder);
        }
    },

    getEscapedPathTitle: function($super) {
        var result = $super();

        if (null == result) {
            result = this.getTypeTitle();
        }
        return result;
    },

    _initComponentElement: function($super) {
        $super();

        this._componentElement.update(
            '<div id="' + this._typesAreaId + '" class="field-name"></div>' +
            '<div class="field-value">' +
                '<div class="fs-select-block hide">' +
                    '<div class="path-to-folder">' +
                        '<span>' +
                            this.getEscapedPathTitle() +
                        '</span>' +
                    '</div>' +
                    '<div class="select-folder">' +
                        '<span>' +
                            this._locale.selectFolder +
                        '</span>' +
                    '</div>' +
                    '<div id="' + this._foldersAreaId + '">' +
                        '<div class="tree-box"></div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );

        this._componentElement.down('.fs-select-block .select-folder span')
            .observe('click', this._onSelectFolderClick.bindAsEventListener(this));
    },

    _onSelectFolderClick: function(event) {
        Event.stop(event);
        this._componentElement.down('.fs-select-block').toggleClassName('hide');
    },

    render: function($super) {
        $super();

        $(this._typesAreaId).update(this._title);
    },

    _initTypeControl: function($super) {
        this._select(this.getType(), this._folder);
    },

    _onDeselectFolderWrapper: function($super) {
        //:INFO: deselect never executes here, folder is always known
    },

    _makeTree: function($super, type, folder) {
        var tree = new Smb.FileSharing.TreeBox({
            renderTo: this._foldersAreaId,
            fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
            itemsSource: Jsw.prepareUrl('/file/tree-data/type/' + encodeURIComponent(type)),
            locale: this._locale,
            onLoad: this._onSelectFolderWrapper.bind(this),
            onSelectFolder: this._onSelectFolderWrapper.bind(this),
            value: folder
        });
        return tree;
    },

    _onSelectFolderWrapper: function() {
        this._componentElement.down('.fs-select-block .path-to-folder span').update(
            this.getEscapedPathTitle()
        );

        this._onSelectFolder();
    },

    getType: function($super) {
        return this._type;
    },

    setType: function($super, type) {
        this._type = type;
        this._select(type);
        this._onSelectFolderWrapper();
    }
});

Smb.FileSharing.Upload = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._selectFilesDialogId = this._id + '-select-files-dialog';
        this._selectFilesWarningBlockId = this._selectFilesDialogId + '-warning-block';
        this._explorerBlockId = this._id + '-explorer-block';
        this._selectFilesDialogItemsAreaId = this._selectFilesDialogId + '-items';
        this._specifyEmailDialogId = this._id + '-specify-email-dialog';

        this._locale = this._getConfigParam('locale', {});
        this._notificationDialogLocale = this._getConfigParam('notificationDialogLocale', {});
        this._confirmationDialoglocale = this._getConfigParam('confirmationDialogLocale', {});
        this._errorDialogLocale = this._getConfigParam('errorDialogLocale', {});

        this._explorer = null;
        this._confirmDialog = null;
    },

    _getUploadIFrameElement: function() {
        if (!this._uploadFormElement) {
            this._uploadFormElement = new Element('iframe', {
                id: this._id + '-upload-iframe',
                name: '_uploadIFrame',
                style: "display: none;",
                src: this._getConfigParam('uploadHandler', '') + '?init=true'
            });
            $(this._contentAreaId).insert(this._uploadFormElement);
            this._uploadFormElement.observe('load', this._onUploadFrameInitialized.bindAsEventListener(this));
        }

        return this._uploadFormElement;
    },

    _onUploadFrameInitialized: function()
    {
        if (this._uploadFrameInitialized) {
            return;
        }
        this._uploadFrameInitialized = true;
        this._uploadFormElement.observe('load', this._onNextQueueUploadFinished.bindAsEventListener(this));
        if (this._scheduleStartUpload) {
            this._scheduleStartUpload = null;
            this.startNextQueueUpload();
        }
    },

    _addEvents: function($super) {
        $super();
        this._cleanupDialog();
        this.showSelectFilesDialog();
    },

    showSelectFilesDialog: function() {
        this.setBoxType('form-box');
        this.setTitle(this._locale.titleSelectFiles);
        this.setHint(this._locale.hintSelectFiles);

        this.addSelectFilesDialogButtons();

        if (!$(this._selectFilesDialogId)) {
            this.renderSelectFilesDialog();
        } else {
            $(this._selectFilesDialogId).show();
        }
    },

    addSelectFilesDialogButtons: function(disabled) {
        if (this._sendEmailNotification) {
            var title = this._locale.buttonNext;
            var updateHandler = function() {
                if (!this._checkUploadQueue()) {
                    return;
                }
                this.hide();
                this.hideSelectFilesDialog();
                this.showSpecifyEmailDialog();
            }
        } else {
            var title = null;
            var updateHandler = function() {
                if (!this._checkUploadQueue()) {
                    return;
                }
                this.hideSelectFilesDialog();
                this.showCheckUploadDialog();
            }
        }
        this._selectUpdateButtonElement = this.addUpdateButton(updateHandler, true, title);
        this._updateButton(this._selectUpdateButtonElement, { disabled: disabled });
        this.addCancelButton();
    },

    _checkUploadQueue: function() {
        this._clearMessages();
        this._prepareUploadQueue();
        if (0 == this._uploadQueue.length) {
            this._addErrorMessage(this._locale.noFilesSelectedForUpload);
            return false;
        }
        return true;
    },

    renderSelectFilesDialog: function() {
        var sendEmailNotificationOptionId = this._id + '-send-email-notification-option';

        $(this._contentAreaId).insert('<div id="' + this._selectFilesDialogId + '">' +
                '<div id="' + this._selectFilesDialogItemsAreaId + '"></div>' +
                '<div class="form-row">' +
                    '<div class="single-row">' +
                        '<a href="#" class="s-btn sb-item-add"><i><i><i><span>' + this._locale.addFileItem + '</span></i></i></i></a>' +
                    '</div>' +
                '</div>' +
                '<hr class="hr"/>' +
                '<div class="form-row" id="' + this._explorerBlockId + '">' +
                    '<p>' + this._locale.uploadTo + '</p>' +
                '</div>' +
                '<div class="form-row" style="display: none;" id="' + this._selectFilesWarningBlockId + '">' +
                    '<div class="single-row">' +
                        '<div class="hint-attention">' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="form-row">' +
                    '<div class="single-row">' +
                        '<div class="indent-box"><input type="checkbox" class="checkbox" value="1" id="' + sendEmailNotificationOptionId + '" name="general[account][isActive]"/>' +
                            '<div class="indent-box-content"><label for="' + sendEmailNotificationOptionId + '">' + this._locale.sendEmailNotificationOption + '</label></div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );
        this._explorer = new Smb.FileSharing.Explorer({
            id: this._id + 'Explorer',
            renderTo: this._explorerBlockId,
            type: this._getConfigParam('type'),
            storageTabs: Smb.FileSharing.ListInstance.getStorageTabs(),
            storageTypes: Smb.FileSharing.ListInstance.getStorageTypes(),
            fileTypesIconsPathTemplate: this._getConfigParam('fileTypesIconsPathTemplate', null),
            folder: this._getConfigParam('folder'),
            locale: this._locale,
            onSelectFolder: (function() {
                if ('private' != this._explorer.getType() || this._explorer.getFolder() || !Smb.FileSharing.ListInstance.isBuiltIn()) {
                    this.enable();
                    $(this._selectFilesWarningBlockId).hide();
                } else {
                    this.disable();
                    $(this._selectFilesWarningBlockId).down('.hint-attention').update(this._locale.hintInvalidDestination);
                    $(this._selectFilesWarningBlockId).show();
                }
            }).bind(this),
            onDeselectFolder: this.disable.bind(this)
        });

        this.addSelectFilesItem();

        $(this._selectFilesDialogId).select('.sb-item-add').each(function(element) {
            element.observe('click', this._onAddSelectFilesItemClick.bindAsEventListener(this));
        }, this);
        $(sendEmailNotificationOptionId).observe('click', this._onSendEmailNotificationOptionClick.bindAsEventListener(this));
    },

    enable: function() {
        if (this._selectUpdateButtonElement) {
            this._updateButton(this._selectUpdateButtonElement, {disabled: false});
        }
    },

    disable: function() {
        if (this._selectUpdateButtonElement) {
            this._updateButton(this._selectUpdateButtonElement, {disabled: true});
        }
    },

    _onAddSelectFilesItemClick: function(event) {
        this.addSelectFilesItem();
        Event.stop(event);
    },

    _onSendEmailNotificationOptionClick: function(event) {
        var element = Event.element(event);
        this._sendEmailNotification = element.checked;
        var disabled =
            this._selectUpdateButtonElement &&
            this._selectUpdateButtonElement.down('button') &&
            this._selectUpdateButtonElement.down('button').disabled;

        this.removeRightButtons();

        this.addSelectFilesDialogButtons(disabled);
    },

    addSelectFilesItem: function() {
        var itemsArea = $(this._selectFilesDialogItemsAreaId);
        var itemsCount = itemsArea.select('form').length;

        var itemId = this._selectFilesDialogId + '-item-' + itemsCount;
        var item = '<form id="' + itemId + '" action="' + this._getConfigParam('uploadHandler', '') + '" method="post" enctype="multipart/form-data" target="_uploadIFrame">' +
                '<input name="type" type="hidden"/>' +
                '<input name="folder" type="hidden"/>' +
                '<input name="fileName" type="hidden"/>' +
                '<div class="form-row">' +
                    '<input name="file" type="file"/>' +
                    (itemsCount ? ' <a href="#" class="s-btn sb-item-remove"><i><i><i><span> ' + this._locale.removeFileItem + '</span></i></i></i></a>' : '') +
                '</div>' +
            '</form>';

        itemsArea.insert({ bottom : item });

        if (itemsCount) {
            $(itemId).select('.sb-item-remove').each(function(element) {
                element.observe('click', this._onRemoveSelectFilesItemClick.bindAsEventListener(this));
            }, this);
        }
    },

    _onRemoveSelectFilesItemClick: function(event) {
        var item = Event.element(event).up('form');
        if (item) {
            item.remove();
        }
        Event.stop(event);
    },

    hideSelectFilesDialog: function() {
        this._cleanupDialog();
        if ($(this._selectFilesDialogId)) {
            $(this._selectFilesDialogId).hide();
        }
    },

    showSpecifyEmailDialog: function() {
        if (this._notificationDialog) {
            this._notificationDialog.show();
        } else {
            this._notificationDialog = new Smb.FileSharing.Notification({
                id: this._id + '-notification',
                renderTo: this.getRenderTarget() + '-notification',
                cls: 'popup-panel',
                locale: this._notificationDialogLocale,
                titleCls: this._titleCls,
                title: this._notificationDialogLocale.specifyEmailParameters,
                onBack: this._onNotificationBackClick.bind(this),
                onOk: this._onNotificationOkClick.bind(this),
                onCancel: this._onNotificationCancelClick.bind(this),
                getFilesHandler: this.getFilesForNotificationDialog.bind(this)
            });
        }

        this._notificationDialog.setTargetType(this._explorer.getType());
        this._notificationDialog.setTargetFolder(this._explorer.getFolder());
        this._notificationDialog.reset();
    },

    getFilesForNotificationDialog: function() {
        var files = $A();
        this._uploadQueue.each(function(queueItem) {
            files.push(queueItem.originalFileName);
        });
        return files;
    },

    _onNotificationBackClick: function() {
        this._notificationDialog.hide();
        this.show();
        this.showSelectFilesDialog();
    },

    _onNotificationOkClick: function() {
        this._notificationDialog.hide();
        this.show();
        this.showCheckUploadDialog();
    },

    _onNotificationCancelClick: function() {
        this._notificationDialog.hide();
        this._onCancelClick();
    },

    hideSpecifyEmailDialog: function() {
        this._cleanupDialog();
        if ($(this._specifyEmailDialogId)) {
            $(this._specifyEmailDialogId).hide();
        }
    },

    _prepareUploadQueue: function() {
        this._uploadQueue = $A();
        var items = $(this._selectFilesDialogItemsAreaId).select('form');
        items.each(function(formElement) {
            var fileName = formElement.down('input[type="file"]').value;
            if (!fileName) {
                return;
            }
            var queueItem = {
                inputId: formElement.id,
                progressId: formElement.id + '-progress',
                formElement: formElement,
                originalFileName: fileName,
                savedFileName: fileName,
                status: 'PENDING',
                type: 'unknown'
            };
            this._uploadQueue.push(queueItem);
        }, this);
    },

    showCheckUploadDialog: function() {
        this.setBoxType('list-box');
        this.setTitle(this._locale.titleCheckUpload);
        this.setHint('<span class="ajax-loading">' + this._locale.hintCheckUpload + '</span>');

        this._skipSelected = false;
        var checkQueue = $H({
            type: this._explorer.getType(),
            folder: this._explorer.getFolder()
        });

        this._uploadQueue.each(function(queueItem) {
            checkQueue.set('queue[' + queueItem.inputId + '][id]', queueItem.inputId);
            checkQueue.set('queue[' + queueItem.inputId + '][originalFileName]', queueItem.originalFileName);
        }, this);

        new Ajax.Request(Jsw.baseUrl + '/file/check-upload/', {
            method: 'post',
            parameters: checkQueue,
            onSuccess: this._onSuccessCheckUploads.bind(this),
            onException: (function(transport, exception) {
                Smb.Views.FileSharing.clearStatusMessages();
                Jsw.addStatusMessage(
                    'error',
                    'Internal error: ' + exception + "\n" +
                    transport.responseText
                );
                this.hide();
            }).bind(this)
        });
    },

    _onSuccessCheckUploads: function(transport) {
        var errorsFound = false;

        var result = transport.responseText.evalJSON();
        switch (result.status) {
            case 'SUCCESS':
                break;
            case 'ERROR':
                this._reportError(result.message)
                return;
            default:
                this._reportError('Bad upload check response status: ' + result.status);
                return;
        }
        $H(result.data).each(function(pair) {
            var id = pair.key;
            var checkItem = pair.value;
            var queueItem = null;
            this._uploadQueue.each(function(item) {
                if (item.inputId == id) {
                    queueItem = item;
                    throw $break;
                }
            }, this);

            if (!queueItem) {
                return;
            }

            if (checkItem.exists) {
                queueItem.exists = checkItem.exists;
                errorsFound = true;
            }
            if (checkItem.savedFileName) {
                queueItem.savedFileName = checkItem.savedFileName;
            }
            if (checkItem.type) {
                queueItem.type = checkItem.type;
            }
        }, this);

        this.hideCheckUploadDialog();
        if (errorsFound) {
            this.showConfirmReplaceDialog();
        } else {
            this.showUploadProgressDialog();
            this.startNextQueueUpload();
        }
    },

    hideCheckUploadDialog: function() {
        this._cleanupDialog();
    },

    showConfirmReplaceDialog: function() {
        var context = this;

        this.hide();
        var items = $A();
        this._uploadQueue.each(function(queueItem) {
            if (!queueItem.exists) {
                return;
            }
            items.push(queueItem.exists);
        });

        var listControl = Jsw.getComponent('files-list');
        this._confirmDialog = new Smb.FileSharing.UploadConfirmationDialog({
            id: this._id + '-confirmation',
            renderTo: this._id + '-confirmation-dialog-box',
            cls: 'popup-panel',
            locale: this._confirmationDialoglocale,
            destination: context._explorer.getEscapedPathTitle(),
            items: items,
            overwriteHandler: function() {
                context.hideConfirmReplaceDialog();
                context.showUploadProgressDialog();
                context.startNextQueueUpload();
            },
            skipHandler: function() {
                context._skipSelected = true;
                context._uploadQueue.each(function(item) {
                    if (item.exists) {
                        context._uploadQueue = context._uploadQueue.without(item);
                    }
                }, context);
                context.hideConfirmReplaceDialog();
                if (context._uploadQueue.length > 0) {
                    context.showUploadProgressDialog();
                    context.startNextQueueUpload();
                } else {
                    context.reportUploadFinished();
                }
            },
            cancelHandler: function() {
                context.close();
            }
        });
    },

    hideConfirmReplaceDialog: function() {
        this._cleanupDialog();
        this._confirmDialog.remove();
        this._confirmDialog = null;
    },

    showUploadProgressDialog: function() {
        this.hide();
        this._uploadProgressDialog = new Smb.FileSharing.uploadProgressDialog({
            id: this._id + '-progress',
            renderTo: this._id + '-progress-dialog-box',
            items: this._uploadQueue,
            locale: this._locale,
            onCancelClick: this._onCancelUploadClick.bind(this)
        });
    },

    _onCancelUploadClick: function(event) {
        this._uploadCanceled = true;
    },

    startNextQueueUpload: function() {
        var statusFrame = this._getUploadIFrameElement();

        if (!this._uploadFrameInitialized) {
            this._scheduleStartUpload = true;
            return;
        }

        if (this._uploadCurrent) {
            return;
        }

        if (this._uploadCanceled) {
            this._onAllQueueUploadFinished();
            return;
        }

        var started = false;

        this._uploadQueue.each(function(queueItem) {
            if ('PENDING' != queueItem.status) {
                return;
            }
            started = true;
            this.updateUploadProgressItemStatus(queueItem, 'UPLOADING');
            this._uploadCurrent = queueItem;
            queueItem.formElement.select('input[name="type"]').each(function(element) {
                element.value = this._explorer.getType();
            }, this);
            queueItem.formElement.select('input[name="folder"]').each(function(element) {
                element.value = this._explorer.getFolder();
            }, this);
            queueItem.formElement.select('input[name="fileName"]').each(function(element) {
                element.value = queueItem.savedFileName;
            }, this);
            queueItem.formElement.submit();
            throw $break;
        }, this);

        if (!started) {
            this._onAllQueueUploadFinished();
            return;
        }

        if (1 == this._uploadQueue.length) {
            return;
        }

        var status = this.getQueueStatus();

        this._uploadProgressDialog.updateListDescription(
            this._locale.descriptionUploadProgress
                .replace('%%total%%', status.total)
                .replace('%%pending%%', status.pending)
                .replace('%%uploading%%', status.uploading)
                .replace('%%completed%%', status.completed)
                .replace('%%failed%%', status.failed)
                .replace('%%folder%%', '<b>' + this._explorer.getEscapedPathTitle() + '</b>')
        );
    },

    _onNextQueueUploadFinished: function() {
        if (!this._uploadCurrent) {
            return;
        }

        var queueItem = this._uploadCurrent;
        this._uploadCurrent = null;

        var status = this._getNextQueueUploadResult();
        switch(status.status) {
            case 'SUCCESS':
                this.updateUploadProgressItemStatus(queueItem, 'SUCCESS');
                this.startNextQueueUpload();
                break;
            case 'ERROR':
            default:
                this.updateUploadProgressItemStatus(queueItem, 'ERROR');
                this._onQueueUploadFailed(status);
                break;
        }
    },

    _getNextQueueUploadResult: function() {
        var statusFrame = this._getUploadIFrameElement();
        var statusDocument = statusFrame.contentDocument;
        if (statusDocument == undefined || statusDocument == null)
            statusDocument = statusFrame.contentWindow.document;
        var status = null;
        try {
            status = statusDocument.body.innerHTML.evalJSON();
        } catch(e) {
            status = {
                status: 'ERROR',
                message: 'No upload response',
                data: null
            }
        }
        return status;
    },

    _onAllQueueUploadFinished: function() {
        Smb.FileSharing.ListInstance.reload();
        this.reportUploadFinished.bind(this).delay(0.5);
    },

    _onQueueUploadFailed: function(status) {
        var errorItems = $A();
        this._uploadQueue.each(function(item) {
            errorItems.push({
                'name': item.savedFileName,
                'type': item.type,
                'status': item.status
            });
        });

        this.close();

        new Smb.FileSharing.ErrorDialog({
            errorMessage: status.message,
            message: 1 < errorItems.length ? this._locale.failed : '',
            locale: this._errorDialogLocale,
            items: errorItems
        });
    },

    reportUploadFinished: function() {
        var messageType = null;
        var message = null;

        Smb.Views.FileSharing.clearStatusMessages();

        this.close();

        if (0 == this._uploadQueue.length) {
            return;
        }

        var status = this.getQueueStatus();

        var sendLinks = false;

        if (this._uploadCanceled) {
            message = this._locale.uploadCanceled;
            messageType = 'warning';
        } else if (1 == this._uploadQueue.length) {
            var queueItem = this._uploadQueue.first();
            if ('SUCCESS' == queueItem.status) {
                message = this._locale.uploadFinishedSingleFile
                    .replace('%%file%%',
                        '<a href="' +
                            this._explorer.getFileUrl(queueItem.savedFileName) +
                        '" target="_blank">' +
                            queueItem.savedFileName.escapeHTML() +
                        '</a>');
                messageType = 'info';
                sendLinks = true;
            } else {
                message = this._locale.uploadFailedSingleFile
                    .replace('%%file%%', '<b>' + queueItem.savedFileName.escapeHTML() + '</b>');
                messageType = 'error';
            }
        } else if (status.completed > 0) {
            message = this._locale.uploadFinished;
            messageType = 'info';
            sendLinks = true;
        } else {
            message = this._locale.uploadFinishedNoFiles;
            messageType = 'warning';
        }

        Jsw.addStatusMessage(messageType, message
            .replace('%%total%%', status.total)
            .replace('%%pending%%', status.pending)
            .replace('%%uploading%%', status.uploading)
            .replace('%%completed%%', status.completed)
            .replace('%%failed%%', status.failed)
            .replace('%%folder%%',
                '<a href="' +
                    Jsw.prepareUrl(
                        '/file/index/type/' +
                        encodeURIComponent(this._explorer.getType()) +
                        '/?folder=' +
                        encodeURIComponent(this._explorer.getFolder())) +
                    '">' + this._explorer.getEscapedFolderTitle() + '</a>')
        );

        if (this._sendEmailNotification && sendLinks && !this._skipSelected) {
            this._notificationDialog._execute();
        }
    },

    _reportError: function(message) {
        Smb.Views.FileSharing.clearStatusMessages();
        Jsw.addStatusMessage('error', message);

        this.close();
    },

    updateUploadProgressItemStatus: function(queueItem, status) {
        queueItem.status = status;
        this._uploadProgressDialog.updateItem(queueItem);
    },

    addBackButton: function(handler, isDefault, title) {
        var button = this.addLeftButton(
            (title ? title : this._locale.buttonBack),
            handler,
            isDefault,
            false
        );
        return button;
    },

    addUpdateButton: function(handler, isDefault, title) {
        var button = this.addRightButton(
            (title ? title : this._locale.buttonOk),
            handler,
            isDefault,
            true
        );
        $(this._actionButtonsId).show();
        return button;
    },

    addCancelButton: function(title, isDefault) {
        var button = this.addRightButton(
            (title ? title : this._locale.buttonCancel),
            this._onCancelClick,
            isDefault,
            false
        );
        $(this._actionButtonsId).show();
        return button;
    },

    _onCancelClick: function() {
        this.hide();
    },

    _cleanupDialog: function() {
        this.setTitle('');
        this.setHint('');
        this.setHint1('');
        this.removeRightButtons();
        this.removeLeftButtons();

        $(this._actionButtonsId).hide();
    },

    getQueueStatus: function() {
        var status = {
            completed: 0,
            pending: 0,
            failed: 0,
            uploading: 0,
            total: this._uploadQueue.length
        };

        this._uploadQueue.each(function(queueItem) {
            switch (queueItem.status) {
                case 'PENDING':
                    status.pending++;
                    break;
                case 'UPLOADING':
                    status.uploading++;
                    break;
                case 'ERROR':
                    status.failed++;
                    break;
                case 'SUCCESS':
                    status.completed++;
                    break;
            }
        }, this);

        return status;
    },

    close: function() {
        if (this._confirmDialog) {
            this._confirmDialog.remove();
            this._confirmDialog = null;
        }

        if (this._uploadProgressDialog) {
            this._uploadProgressDialog.remove();
            this._uploadProgressDialog = null;
        }
        this.hide();
    }

});

Smb.FileSharing.uploadProgressDialog = Class.create(Smb.FileSharing.ListPopup, {
    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', 'pp-upload');

        this._items = this._getConfigParam('items', $A());

        this._locale = this._getConfigParam('locale', {});
        this._title = this._getConfigParam('title', this._locale.title);

        this._uploadProgressDialogDescriptionId = this._id + '-description';
        this._uploadProgressDialogItemsAreaId = this._id + '-items';

        this._onCancelClickExternal = this._getConfigParam('onCancelClick', null);
    },

    _renderContentArea: function($super) {
        $super();
        if ( 1 == this._items.length ) {
            return;
        }
        $(this._contentAreaId).insert({
            top:  '<div class="paging">' +
                '<div class="paging-area">' +
                    '<span class="paging-info" id="' + this._uploadProgressDialogDescriptionId + '"> </span>' +
                 '</div>' +
            '</div>'
        });
    },

    render: function($super) {
        $super();

        this.setBoxType('list-box');
        this.setTitleType('pp-upload');

        this.setTitle(this._locale.titleUploadProgress);
        this.setHint(this._locale.hintUploadProgress);

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        if (this._items.length > 1) {
            this._cancelButton = this.addRightButton(
                this._locale.buttonCancelUpload,
                this._onCancelClick,
                true,
                false
            );
        }

        this.resizeList();
    },

    _onCancelClick: function() {
        this._updateButton(this._cancelButton, {
            title: this._locale.buttonCancelUploadClicked,
            disabled: true
        });
        if (this._onCancelClickExternal) {
            this._onCancelClickExternal();
        }
    },

    _addItem: function(queueItem) {
        this._itemClass = ('even' == this._itemClass) ? 'odd' : 'even';

        var itemHtml = '<tr class="' + this._uploadProgressItemClass + '" id="' + queueItem.progressId + '">' +
            '<td class="first">' +
                '<span class="type-' + queueItem.type.escapeHTML() + '">' +
                    queueItem.savedFileName.escapeHTML() +
                '</span>' +
            '</td>' +
            '<td class="last">' +
                '<span class="hint-pending">' +
                    this._locale.statusPending +
                '</span>' +
            '</td>' +
        '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    },

    updateListDescription: function(html) {
        $(this._uploadProgressDialogDescriptionId).update(html);
    },

    updateItem: function(queueItem) {
        var statusIndicator = '';

        switch (queueItem.status) {
            case 'UPLOADING':
                statusIndicator = '<span class="hint-wait">' + this._locale.statusUploading +'</span>';
                break;
            case 'ERROR':
                statusIndicator = '<span class="hint-failed">' + this._locale.statusFailed +'</span>';
                break;
            case 'SUCCESS':
                statusIndicator = '<span class="hint-ok">' + this._locale.statusCompleted +'</span>';
                break;
        }

        $(queueItem.progressId).down('.last').update(statusIndicator);
    }
});

Smb.FileSharing.Delete = Class.create({

    initialize: function(config) {

        this._locale = config.locale;
        var confirmationDialogLocale = config.confirmationDialogLocale;
        this._errorDialogLocale = config.errorDialogLocale;

        this._list = Jsw.getComponent('files-list');

        var selection = $A(this._list.getSelectedItemsIds());

        this._type = this._list.getStorageType();
        this._folder = this._list.getCurrentFolder();

        this._selected = $A();

        this._foldersCount = 0;
        this._filesCount = 0;

        this._list.getData().each(function(dataItem) {
            var itemIndex = selection.indexOf(dataItem.id);
            if (itemIndex < 0) {
                return;
            }
            this._selected.push(dataItem);
            if ('folder' == dataItem.type) {
                this._foldersCount++;
            } else {
                this._filesCount++;
            }
        }, this);

        if (0 == this._foldersCount + this._filesCount) {
            Smb.Views.FileSharing.showWarning(confirmationDialogLocale.itemsNotSelectedOnDelete);
            return;
        }

        var message = '';
        if ( this._foldersCount && this._filesCount ) {
            message = confirmationDialogLocale.textOnDeleteFilesAndFolders;
        } else if (1 == this._foldersCount) {
            message = confirmationDialogLocale.textOnDeleteFolder;
        } else if (1 == this._filesCount) {
            message = confirmationDialogLocale.textOnDeleteFile;
        } else if (this._foldersCount) {
            message = confirmationDialogLocale.textOnDeleteFolders;
        } else if (this._filesCount) {
            message = confirmationDialogLocale.textOnDeleteFiles;
        }

        fileName = 1 == this._selected.length
            ? this._selected.first().name
            : '';

        Jsw.messageBox.show({
            'type': Jsw.messageBox.TYPE_YESNO,
            'text': message
                .replace('%%count%%', this._foldersCount + this._filesCount)
                .replace('%%foldersCount%%', this._foldersCount)
                .replace('%%filesCount%%', this._filesCount)
                .replace('%%fileName%%', '<b>' + fileName.escapeHTML() + '</b>'),
            'subtype': 'delete',
            'description': '',
            'buttonTitles': {
                'yes': confirmationDialogLocale.buttonYes,
                'no': confirmationDialogLocale.buttonNo
            },
            'onYesClick': this._onConfirmationYesClick.bind(this)
        });
    },

    _onConfirmationYesClick: function() {

        var ids = $H();
        var count = 0;
        this._selected.each(function(dataItem) {
            ids.set('ids[' + count + ']', dataItem.id);
            count++;
        });

        new Ajax.Request(
            Jsw.prepareUrl('/file/delete'),
            {
                method: 'post',
                parameters: ids.merge({
                    'type': this._type,
                    'folder': this._folder
                }),
                onCreate: this._onCreate.bind(this),
                onSuccess: this._onSuccess.bind(this),
                onException: this._onException.bind(this)
            }
        );
    },

    _onCreate: function() {
        this._waitDialog = new Smb.FileSharing.Wait({
            cls: 'popup-panel',
            title: this._locale.title,
            message: this._locale.waitMessage,
            titleCls: 'pp-delete'
        });
    },

    _onSuccess: function(transport) {
        try {
            var response = transport.responseText.evalJSON();
            switch (response.status) {
                case 'SUCCESS':
                    this._showSuccessMessage();
                    break;
                case 'ERROR':
                    if (response.data) {
                        this._onExecuteError(response.message, $H(response.data));
                    } else {
                        this._userError(response.message);
                    }
                    break;
                default:
                    this._internalError('Unknown response status: ' + response.status);
            }
            Smb.FileSharing.ListInstance.reload();
        } catch (e) {
            this._internalError(transport.responseText);
        }
    },

    _userError: function (message) {
        this.close();

        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', message);
    },

    _internalError: function (message) {
        this.close();

        Jsw.clearStatusMessages();
        Jsw.addStatusMessage('error', 'Internal error: ' + message);
    },

    _showSuccessMessage: function() {
        var message = '';
        if ( this._foldersCount && this._filesCount ) {
            message = this._locale.textOnDeleteFilesAndFolders;
        } else if (1 == this._foldersCount) {
            message = this._locale.textOnDeleteFolder;
        } else if (1 == this._filesCount) {
            message = this._locale.textOnDeleteFile;
        } else if (this._foldersCount) {
            message = this._locale.textOnDeleteFolders;
        } else if (this._filesCount) {
            message = this._locale.textOnDeleteFiles;
        }

        fileName = 1 == this._selected.length
            ? this._selected.first().name
            : '';

        this.close();

        Jsw.addStatusMessage('info', message
            .replace('%%count%%', this._foldersCount + this._filesCount)
            .replace('%%foldersCount%%', this._foldersCount)
            .replace('%%filesCount%%', this._filesCount)
            .replace('%%fileName%%', '<b>' + fileName.escapeHTML() + '</b>')
        );
    },

    _onExecuteError: function(errorMessage, items) {
        var errorItems = $A();

        this._selected.each((function(listItem) {
            var errorItem = Object.clone(listItem);
            items.each(function(pair) {
                if (listItem.id == pair.key) {
                    errorItem.status = pair.value;
                    throw $break;
                }
            }, this);

            errorItems.push(errorItem);
        }).bind(this));

        var message = null;
        if (1 < errorItems.length) {
            message = this._locale.failed;
        }

        this.close();

        this._errorDialog = new Smb.FileSharing.ErrorDialog({
            errorMessage: errorMessage,
            message: message,
            locale: this._errorDialogLocale,
            items: errorItems
        });
    },

    _onException: function(transport, exception) {
        this._internalError(exception);
    },

    close: function() {
        if (this._waitDialog) {
            this._waitDialog.hide();
            this._waitDialog.remove();
            this._waitDialog = null;
        }

        if (this._errorDialog) {
            this._errorDialog.hide();
            this._errorDialog.remove();
            this._errorDialog = null;
        }
    }
});

Smb.FileSharing.ConfirmationDialog = Class.create(Smb.FileSharing.ListPopup, {

    _initConfiguration: function($super, config) {
        $super(config);

        //:INFO: destination param should be HTML escaped
        this._destination = this._getConfigParam('destination', '');
        this._items = this._getConfigParam('items', []);
        this._locale = this._getConfigParam('locale', {});

        this._onOverwriteHandler = this._getConfigParam('overwriteHandler', function() {});
        this._onSkipHandler = this._getConfigParam('skipHandler', function() {});
        this._onCancelHandler = this._getConfigParam('cancelHandler', function() {});
    },

    render: function($super) {
        $super();

        this.setBoxType('list-box');
        this.setTitleType('pp-alert');
        this.setTitle(this._locale.title);
        this.setHint(this._locale.description);
        this.setHint1(this._locale.hint
            .replace('%%folder%%', '<b>' + this._destination + '</b>'));

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        this.addRightButton(
            this._locale.buttonOverwrite,
            this._onOverwriteHandler,
            false,
            true
        );
        this.addRightButton(
            this._locale.buttonSkip,
            this._onSkipHandler,
            true,
            true
        );
        this.addRightButton(
            this._locale.buttonCancel,
            this._onCancelHandler,
            false,
            false
        );

        this.resizeList();
    },

    _renderContentArea: function($super) {
        $super();
        $(this._itemsAreaId).insert(
            '<tr>' +
                '<th class="first">' + this._locale.columnName + '</th>' +
                '<th>' + this._locale.columnNew + '</th>' +
                '<th class="last">' + this._locale.columnExisting + '</th>' +
            '</tr>'
        );
    },

    _addItem: function(item) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td class="first">' +
                    '<span class="type-' + item.existItem.type.escapeHTML() + '">' +
                        item.existItem.name.escapeHTML() +
                    '</span>' +
                '</td>' +
                '<td class="number">' +
                    '<div>' +
                        item.newItem.formatedSize.escapeHTML() +
                    '</div>' +
                    '<div>' +
                        this._locale.modifiedAt.replace(
                            '%%modificationDate%%',
                            item.newItem.modificationDate.escapeHTML()
                        ) +
                    '</div>' +
                '</td>' +
                '<td class="number last">' +
                    '<div>' +
                        item.existItem.formatedSize.escapeHTML() +
                    '</div>' +
                    '<div>' +
                        this._locale.modifiedAt.replace(
                            '%%modificationDate%%',
                            item.existItem.modificationDate.escapeHTML()
                        ) +
                    '</div>' +
                '</td>' +
            '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    }
});

Smb.FileSharing.UploadConfirmationDialog = Class.create(Smb.FileSharing.ConfirmationDialog, {

    _addItem: function($super, item) {
        this._itemClass = ('odd' == this._itemClass) ? 'even' : 'odd';

        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td class="">' +
                    '<span class="type-' + item.type.escapeHTML() + '">' +
                        item.name.escapeHTML() +
                    '</span>' +
                '</td>' +
                '<td class="number">' +
                    item.formatedSize.escapeHTML() +
                '</td>' +
                '<td class="number">' +
                    this._locale.modifiedAt.replace(
                        '%%modificationDate%%',
                        item.modificationDate.escapeHTML()
                    ) +
                '</td>' +
                '<td class="minor"></td>' +
            '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    }
});

Smb.FileSharing.ErrorDialog = Class.create(Smb.FileSharing.ListPopup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._cls = this._getConfigParam('cls', 'popup-panel');
        this._titleCls = this._getConfigParam('titleCls', 'pp-error');

        this._errorMessage = this._getConfigParam('errorMessage', '');
        this._message = this._getConfigParam('message', null);
        this._items = this._getConfigParam('items', $A());

        this._locale = this._getConfigParam('locale', {});
        this._title = this._getConfigParam('title', this._locale.title);
    },

    render: function($super) {
        $super();

        this.setBoxType('list-box');
        this.setTitle(this._title);

        this._addErrorMessage(this._errorMessage);
        if (null == this._message) {
            var contentElement = $(this._popupContentAreaId);
            contentElement.select('.msg-error').each(function(messageElement) {
                messageElement.addClassName('list-error');
            });
        } else {
            this.setHint(this._message);
        }

        this._items.each(function(item) {
            this._addItem(item);
        }, this);

        this.addRightButton(
            this._locale.buttonClose,
            this._onClose.bind(this),
            true,
            true
        );

        this.resizeList();
    },

    _onClose: function() {
        this.hide();
    },

    _addItem: function(item) {
        this._itemClass = ('even' == this._itemClass) ? 'odd' : 'even';

        var statusClass = (item.status)
            ? ('ERROR' == item.status ? 'failed' : 'ok')
            : 'not-started';

        var statusTitle = (item.status)
            ? ('ERROR' == item.status
                ? this._locale.statusError
                : this._locale.statusSuccess)
            : this._locale.statusNotStarted;

        var statusHml = '' +
            '<span class="hint-' + statusClass + '">' +
                statusTitle +
            '</span>';

        var itemHtml = '' +
            '<tr class="' + this._itemClass + '">' +
                '<td class="first">' +
                    '<span class="type-' + item.type.escapeHTML() + '">' +
                        item.name.escapeHTML() +
                    '</span>' +
                '</td>' +
                '<td class="last">' +
                    statusHml +
                '</td>' +
            '</tr>';

        $(this._itemsAreaId).insert({ bottom : itemHtml });
    }
});

Smb.FileSharing.List = Class.create(Jsw.List, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._storageType = this._getConfigParam('storageType', '');
        this._storageUrl = this._getConfigParam('storageUrl', '');
        this._storageTabs = $H(this._getConfigParam('storageTabs'));
        this._storageTypes = $H(this._getConfigParam('storageTypes'));
        this._users = $H(this._getConfigParam('users'));
        this._userContactName = this._getConfigParam('userContactName');
        this._userLogin = this._getConfigParam('userLogin');
        this._isBuiltIn = this._getConfigParam('isBuiltIn');
        this._loadingTitle = this._getConfigParam('loadingTitle', '');
        this._currentFolder = this._getConfigParam('currentFolder', '');
        this._previousFolder = this._currentFolder;
        this._previousRequestParams = '';
        this._onReloadFolderComplete = null;
        this._locale = this._getConfigParam('locale', {});
        this._footer = this._getConfigParam('footer', null);

        Event.observe(
            window,
            'resize',
            (function() {
                this.resizeList();
            }).bind(this),
            false
        );
    },

    getData: function() {
        return this._data;
    },

    getUsers: function() {
        return this._users;
    },

    getUserContactName: function() {
        return this._userContactName;
    },

    _getListHtml: function($super) {
        var html = $super();

        var footerHtml = (this._footer) ? this._getFooterHtml() : '';

        html = '' +
            '<div class="objects-content">' +
                '<div id="' + this._id + '-objects-content-area" class="objects-content-area">' +
                    '<div class="list-box">' +
                        '<div class="box-area">' +
                            '<div class="content">' +
                                '<div class="content-area">' +
                                    html +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                footerHtml +
            '</div>';

        return html;
    },

    _getFooterHtml: function() {
        return '' +
            '<div id="' + this._id + '-objects-content-footer" class="objects-details-footer">' +
                '<table cellspacing="0" class="footer-container"><tbody><tr>' +
                    '<td>' +
                        '<span class="' + this._footer.cls + '">' +
                            this._footer.description +
                        '</span>' +
                    '</td>' +
                    '<td id="' + this._id + '-footer-actions-area" class="footer-action">' +
                    '</td>' +
                '</tr></tbody></table>' +
            '</div>';
    },

    makeRelativePath: function(item) {
        return ('' == this._currentFolder)
            ? item
            : Smb.FileSharing.trimLastSlash(this._currentFolder) + '/' + item;
    },

    changeFolder: function(folder, onComplete) {
        this._previousFolder = this._currentFolder;
        this._currentFolder = folder;
        this._onReloadFolderComplete = (
            function() {
                Smb.Views.FileSharing.clearStatusMessages();
                if (Object.isFunction(onComplete)) {
                    onComplete();
                }
            }
        ).bind(this);

        this.reload(this._previousRequestParams);
    },

    getCurrentFolder: function() {
        return this._currentFolder;
    },

    getFileUrl: function(file) {
        var storageUrl = (this._isBuiltIn || ('private' != this._storageType))
            ? this._storageUrl
            : (this._storageUrl + this._userLogin + '/');

        return Smb.FileSharing.getFileUrl(storageUrl, this._currentFolder, file);
    },

    getStorageType: function() {
        return this._storageType;
    },

    getStorageTitle: function(storageType) {
        var storageInfo = this._storageTypes.get(storageType);
        return storageInfo.title;
    },

    getCurrentStorageTitle: function() {
        return this.getStorageTitle(
            this.getStorageType()
        );
    },

    getEscapedCurrentPathTitle: function() {
        var folder = this.getCurrentFolder();
        if ('' != folder) {
            folder = '/' + folder;
        }
        return this.getCurrentStorageTitle() + folder;
    },

    isBuiltIn: function() {
        return this._isBuiltIn;
    },

    getStorageTabs: function() {
        return this._storageTabs;
    },

    getStorageTypes: function() {
        return this._storageTypes;
    },

    updateFolderLinkAction: function(link, storageType, folder) {
        if (this.getStorageType() == storageType) {
            $(link).observe(
                'click',
                (function(event) {
                    Event.stop(event);
                    this.changeFolder(folder);
                }).bind(this)
            );
        } else {
            var uri = '';

            $A(folder.split('/')).each(
                function(item) {
                    if('' == item) {
                        return;
                    }
                    uri += item + '/';
                }
            );
            if ('' != uri) {
                uri = '/folder/' + encodeURIComponent(uri);
            }
            $(link).href = Jsw.baseUrl +
                '/file/index/type/' + encodeURIComponent(storageType) +
                uri;
        }
    },

    cleanSelection: function() {
        this._componentElement.select('input[name="listCheckbox[]"]').each(function(checkbox) {
            checkbox.up('tr').removeClassName('selected');
            checkbox.checked = false;
        });

        var globalCheckbox = this._componentElement.select('input[name="listGlobalCheckbox"]').first();
        globalCheckbox.checked = false;
    },

    disable: function($super) {
        var element = $(this._id + '-pathbar');
        if (element) {
            element.hide();
        }
        element = $(this._id + '-objects-content-area');
        if (element) {
            element.hide();
        }
        element = $(this._id + '-objects-content-footer');
        if (element) {
            element.hide();
        }
        element = $(this._id + '-loading-container');
        if (element) {
            element.show();
        } else {
            this._componentElement.insert(
                '<div class="ajax-loading" id="' + this._id + '-loading-container">' +
                    this._loadingTitle +
                '</div>'
            );
        }
        this._disablerOverlay.show();
        this._disablerOverlay.clonePosition($(this._id + '-actions-box'));
    },

    enable: function($super) {
        $(this._id + '-loading-container').hide();

        var element = $(this._id + '-pathbar');
        if (element) {
            element.show();
        }

        element = $(this._id + '-objects-content-area');
        if (element) {
            element.show();
        }
        element = $(this._id + '-objects-content-footer');
        if (element) {
            element.show();
        }
        this._disablerOverlay.hide();
    },

    render: function($super) {
        $super();

        this.reload();

        this._disablerOverlay.style.zIndex = 850;
    },

    _addFooter: function() {
        this._footer.actions.each(function(config) {
            eval('var action = new ' + config.componentType + '(config);');
            action.setRenderTarget(this._id + '-footer-actions-area');
            action.render();
        }, this);
    },

    redraw: function($super) {
        Smb.Views.FileSharing.hideWarning();

        if (this._operations && 'private' == this._storageType && this._isBuiltIn) {
            if ('' == this._currentFolder) {
                this._switchOperations(false);
            } else {
                this._switchOperations(true);
            }
        }

        $super();

        if ((0 == this._pager.totalItemCount) && (this._operations)) {
            this._addOperations();
        }
        if (this._footer) {
            this._addFooter();
        }

        this.resizeList();
    },

    _switchOperations: function(enabled) {
        this._operations.each(function(config) {
            config.disabled = !enabled;
        });
    },

    reload: function($super, params) {
        Smb.FileSharing.collapseExpandedWidgets();
        if (params) {
            this._previousRequestParams = params;
        } else {
            params = '';
        }
        params += '/type/' + encodeURIComponent(this._storageType);
        params += '/folder/' + encodeURIComponent(this._currentFolder);

        return $super(params);
    },

    _onReloadFailure: function($super, transport) {
        $super(transport);

        this.processReloadError(null);
    },

    _onReloadException: function($super, transport, exception) {
        $super(transport, exception);

        this.processReloadError(null);
    },

    _onReloadCreate: function($super, transport) {
        this.disable();
    },

    _onReloadComplete: function($super) {
        $super();

        if (Object.isFunction(this._onReloadFolderComplete)) {
            this._onReloadFolderComplete();
            this._onReloadFolderComplete = null;
        }
    },

    resizeList: function() {
        var height = this._getListHeight();

        height -= $('files-list-actions-box').getHeight();
        if ($('files-list-pathbar')) {
            height -= $('files-list-pathbar').getHeight();
        }
        height -= 2; //border size

        var footer = $(this._id + '-objects-content-footer');
        if (footer) {
            height -= footer.getHeight();
        }

        if (height > 0) {
            $(this._id + '-objects-content-area').style.height = height + 'px';
        }
    },

    _getListHeight: function() {
        var height = $('objects-details').getHeight();

        if (!Prototype.Browser.IE) {
            return height;
        }
        var version = parseInt(
            navigator.userAgent.substring(
              navigator.userAgent.indexOf("MSIE")+5
            )
        );
        if (6 != version) {
            return height;
        }

        var tmpHeight = parseInt($('objects-details').style.height);
        if (!isNaN(tmpHeight)) {
            height = tmpHeight;
        }
        return height;
    },

    _showItemsNotSelectedWarning: function($super) {
        Smb.Views.FileSharing.showWarning(this._locale.itemsNotSelected);
    },

    _hideItemsNotSelectedWarning: function($super) {
        Smb.Views.FileSharing.hideWarning();
    },

    processReloadError: function($super, data) {
        if (data && data.status && data.message && ('ERROR' == data.status)) {
            Jsw.clearStatusMessages();
            Jsw.addStatusMessage(
                'error',
                'Internal error: ' + data.message.escapeHTML()
            );
        }
        this._onReloadFolderComplete = (
            function() {
                var element = $(this._id + '-pathbar');
                if (element) {
                    element.hide();
                }
            }
        ).bind(this);

        $(this._id + '-container').down('.content-area').replace(
            '<div class="content-area"><div id="' + this._id  + '-errorMessage" class="msg-box msg-warning">' +
                '<div><div><div><div><div>' +
                    '<div class="msg-content">' +
                        this._locale.loadingFailed +
                        ' <a onclick="Smb.FileSharing.ListInstance.reload(); return false;" href="#retry">' +
                            this._locale.loadingRetry +
                        '</a>' +
                    '</div>' +
                '</div></div></div></div></div>' +
            '</div></div>'
        );
    },

    _addPathbar: function($super) {
        if (this._pathbarComponent && Object.isFunction(this._pathbarComponent.destroy)) {
            this._pathbarComponent.destroy();
        }
        $super();
    }
});

// TODO: replace with Jsw.list.AdditionalActions
Smb.FileSharing.List.AdditionalActions = Class.create(Jsw.Component, {

    _tag: 'div',

    _initConfiguration: function($super, config) {
        $super(config);

        this._operations = this._getConfigParam('operations', null);
        this._cls = this._getConfigParam('cls', 'actions-menu');
    },

    enable: function() {
        this._titleButton.enable();
    },

    disable: function() {
        this._titleButton.disable();
    },

    toggle: function() {
        Smb.FileSharing.collapseExpandedWidgets(this._id);
        this._componentElement.down('.popup-box').toggleClassName('collapsed');
    },

    collapse: function() {
        this._componentElement.down('.popup-box').addClassName('collapsed');
    },

    render: function($super) {
        $super();

        this._titleButton = new Jsw.SmallButton({
            title: this._getConfigParam('title', ''),
            cls: 's-btn',
            addCls: 'sb-more-actions',
            renderTo: this._id,
            handler: (function(event) {
                Event.stop(event);
                this.toggle();
            }).bind(this)
        });

        this._componentElement.insert(
            {
                bottom:
                    '<div class="popup-box collapsed">' +
                        '<table class="popup-wrapper" cellspacing="0"><tbody><tr>' +
                            '<td class="popup-container">' +
                                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                                    '<div class="popup-heading">' +
                                        '<div class="heading-area">' +
                                            '<span class="close"></span>' +
                                            '<h4>' +
                                                '<span>' +
                                                    this._getConfigParam('title', '') +
                                                '</span>' +
                                            '</h4>' +
                                        '</div>' +
                                    '</div>' +
                                    '<div class="popup-content">'+
                                        '<div class="popup-content-area">' +
                                            '<ul id="' + this._id + '-operations">' +
                                            '</ul>' +
                                        '</div>' +
                                    '</div>' +
                                '</div></div></div></div></div>' +
                            '</td>' +
                        '</tr></tbody></table>' +
                    '</div>'
            }
        );

        if (!this._operations) {
            return;
        }

        var operationsArea = $(this._id + '-operations');
        this._operations.each(function(config) {
            var liElement = new Element('li');
            if (config.liCls) {
                liElement.addClassName(config.liCls);
            }
            operationsArea.insert(liElement);

            eval('var operation = new ' + config.componentType + '(config);');
            operation.setRenderTarget(liElement);
            operation.render();
        }, this);

        if (this._getConfigParam('disabled', false)) {
            this.disable();
        } else {
            this.enable();
        }
    }
});

Smb.FileSharing.List.Pathbar = Class.create(Jsw.Pathbar, {

    _initComponentElement: function($super) {
        $super();

        this._linkDisabled = this._getConfigParam('linkDisabled', false);

        var linkElement = '';
        if (this._linkDisabled) {
            linkElement = '<img src="' + Jsw.skinUrl + '/img/icons/small/file-folder-link-disabled_16.gif" alt="" />';
        } else {
            linkElement =
                '<a class="folder-link" id="' + this._id + '-folder-link" href="#">' +
                    '<img src="' + Jsw.skinUrl + '/img/icons/small/file-folder-link_16.gif" alt="" />' +
                '</a>' +
                '<div class="popup-box collapsed" id="' + this._id + '-popup-box"/>';
        }

        this._updateComponentElement('<div class="folder-link-block">' + linkElement + '</div>');
    },

    _renderItem: function($super, item) {
        $super(item);

        if (item instanceof Jsw.Pathbar.Item) {

            var context = this;

            item.addEventObserver('click', function(event) {
                event.stop();
                Smb.FileSharing.collapseExpandedWidgets();

                Smb.FileSharing.ListInstance.changeFolder(
                    decodeURIComponent(item.getHref())
                );
            });
        }
    },

    _renderItemSuffix: function($super, item) {
        if (!(item instanceof Jsw.Pathbar.Item)) {
            return;
        }
        $super(item);
    },

    render: function($super) {
        $super();
        if (this._linkDisabled) {
            return;
        }
        this._linkToFolderPopup = new Smb.FileSharing.LinkToFolderPopupArea({
            locale: this._getConfigParam('locale', {}),
            applyTo: this._id + '-popup-box',
            title: 'title',
            toggleElementId: this._id + '-folder-link',
            webDavUrl: this._getConfigParam('webDavUrl', ''),
            httpUrl: this._getConfigParam('httpUrl', ''),
            webDavHowToLink: this._getConfigParam('webDavHowToLink', '')
        });
    },

    destroy: function() {
        if (this._linkToFolderPopup) {
            this._linkToFolderPopup.destroy();
        }
    }
});

Smb.FileSharing.TreeBox = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._locale = this._getConfigParam('locale', {});
        this._value = this._getConfigParam('value', '');
        this._treeNodesAreaId = this._id + '-tree-nodes-area';
        this._fileTypesIconsPathTemplate = this._getConfigParam('fileTypesIconsPathTemplate', null);
        this._onSelectFolder = this._getConfigParam('onSelectFolder', {});
        this._onLoad = this._getConfigParam('onLoad', {});
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="tree-box">' +
                '<div class="box-area"><ul id="' + this._treeNodesAreaId + '">' +
                '</ul></div>' +
            '</div>'
        );
    },

    _addEvents: function($super) {
        $super();

        this._componentElement.select('.tree-box').each(function(element) {
            element.observe('click', this.setValue.bind(this, ''));
        }, this);
    },

    _initItems: function($super) {
        var items = this._getConfigParam('items');
        this._items = items ? $A(items) : null;
        var source = this._getConfigParam('itemsSource', null);
        if (source) {
            new Ajax.Request(source, {
                onSuccess: this._onItemsSuccess.bind(this),
                onException: this._onItemsException.bind(this)
            });
        }
    },

    _onItemsSuccess: function(transport) {
        var response = transport.responseText.evalJSON();
        switch (response.status) {
            case 'SUCCESS':
                this._items = $A(response.data);
                this._renderItems();

                if (Object.isFunction(this._onLoad)) {
                    this._onLoad();
                }
                break;
            case 'ERROR':
                this._reportError(response.message);
                break;
            default:
                this._reportError('Bad folders list response status: ' + result.status);
                break;
        }
    },

    _onItemsException: function(transport, exception) {
        this._reportError(exception);
    },

    _reportError: function() {
        var retryElement = new Element('a', {href: '#"'}).update(this._locale.loadingRetry);
        $(this._treeNodesAreaId).update(
            '<span class="hint-error">' + this._locale.loadingFailed +'</span> '
        );
        $(this._treeNodesAreaId).insert(retryElement);
        retryElement.observe('click', (function() {
            this._initItems();
            this._renderLoadingItems();
        }).bind(this));
    },

    _renderItems: function($super) {
        if (null == this._items) {
            this._renderLoadingItems();
        } else
        if (0 == this._items.length) {
            this._renderNoItems();
        } else {
            $(this._treeNodesAreaId).update('');
            var parent = $(this._treeNodesAreaId);
            this._items.each(function(item) {
                this._renderFolder(item, parent);
            }, this);
        }
    },

    _renderLoadingItems: function() {
        $(this._treeNodesAreaId).update(
            '<span class="hint-wait">' + this._locale.loadingFolders +'</span>'
        );
    },

    _renderNoItems: function() {
        $(this._treeNodesAreaId).update(
            '<span class="hint">' + this._locale.noFolders +'</span>'
        );
    },

    _renderItem: function(item) {
        return this._renderFolder(item, $(this._treeNodesAreaId));
    },

    setValue: function(value) {
        var folder = this._getFolderByValue(value, this._items);
        this._changeFolder(folder);
    },

    _getFolderByValue: function(value, folders) {
        if (!folders) {
            return null;
        }
        var ret = null;
        folders.each(function(folder) {
            if (value == folder.value) {
                return folder;
            }
            ret = this._getFolderByValue(value, folder.sub);
            if (ret) {
                throw $break;
            }
        }, this);

        return ret;
    },

    _changeFolder: function(folder) {
        this._componentElement.select('DIV.treeBoxNode').each(function(element) {
            element.removeClassName('selected');
        });
        if (folder) {
            folder.element.addClassName('selected');
            this._value = folder.value;
        } else {
            this._value = '';
        }
        if (Object.isFunction(this._onSelectFolder)) {
            this._onSelectFolder();
        }
    },

    _onClick: function(event, folder) {
        Event.stop(event);
        this._changeFolder(folder);
    },

    _renderFolder: function(folder, parent) {
        var listElement = new Element('li');
        var handlerElement = new Element('span').update(this._renderFileTypeIcon('folder') + '<span>' + folder.name.escapeHTML() + '</span>');
        folder.element = new Element('div', {'class':'treeBoxNode'}).update(handlerElement);

        if (('' == this._value) && folder.isDefault) {
            this._value = folder.value;
        }
        if (folder.value == this._value) {
            folder.element.addClassName('selected');
        }

        listElement.insert(folder.element);
        parent.insert(listElement);

        handlerElement.observe('click', this._onClick.bindAsEventListener(this, folder));

        if (null != folder.sub) {
            var subElements = new Element('ul');
            listElement.insert(subElements);
            folder.sub.each(function(item) {this._renderFolder(item, subElements)}, this);
        }
    },

    _renderFileTypeIcon: function(type) {
        if (!this._fileTypesIconsPathTemplate) {
            return '';
        }
        return '<img alt="" src="' + this._fileTypesIconsPathTemplate.replace('%%type%%', type) + '"/>';
    },

    getValue: function() {
        return this._value;
    }

});

Smb.FileSharing.TypesList = Class.create(Jsw.Container, {

    _tag: 'ul',

    _initConfiguration: function($super, config) {
        $super(config);

        this._value = this._getConfigParam('value', null);
        this._onClickHandler = this._getConfigParam('onClick', null);

        this._treeNodes = $H();
    },

    reset: function() {
        this._treeNodes.each(function(pair) {
            pair.value.removeClassName('active');
        });
        this._value = this._getConfigParam('value', null);
    },

    _initItems: function($super) {
        this._items = $H(this._getConfigParam('items', {}));
    },

    _renderItems: function($super) {
        if (null != this._items) {
            var parent = this._componentElement;
            this._items.each(function(pair) {
                this._renderItem(pair.key, $H(pair.value), parent);
            }, this);
        }
    },

    _renderItem: function(id, item, parent) {
        var listElement = new Element('li', {
            'class': 'typeNode' +
                     ' files-' + id +
                     (this._value == id ? ' active': '')
        });
        var nodeElement = new Element('div').update('<span>' + item.get('title') + '</span>');
        parent.insert(listElement.update(nodeElement));
        nodeElement.observe('click', this._onClick.bindAsEventListener(this, id, item));
        if (null != item.get('sub-items')) {
            var subElements = new Element('ul');
            listElement.insert(subElements);
            $H(item.get('sub-items')).each(function(pair) {
                this._renderSubItem(pair.key, $H(pair.value), subElements)
            }, this);
        }
        this._treeNodes.set(id, listElement);
    },

    _renderSubItem: function(id, item, parent) {
        var listElement = new Element('li', {
            'class': 'typeNode' +
                     (this._value == id ? ' active': '')
        });
        var nodeElement = new Element('div').update('<span>' + item.get('title') + '</span>');
        parent.insert(listElement.update(nodeElement));
        nodeElement.observe('click', this._onClick.bindAsEventListener(this, id, item));
        this._treeNodes.set(id, listElement);
    },

    _onClick: function(event, id, item) {
        var treeNode = Event.findElement(event, 'LI.typeNode');
        if (!treeNode) {
            return;
        }
        Event.stop(event);
        this.setValue(id);
    },

    getValue: function() {
        return this._value;
    },

    setValue: function(value) {
        this.reset();

        this._treeNodes.get(value).addClassName('active');

        this._value = value;
        if (this._onClickHandler) {
            this._onClickHandler(value);
        }
    }
});

Smb.FileSharing.CreateFolder = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._locale = this._getConfigParam('locale', {});
        this._handlerUrl = this._getConfigParam('handlerUrl', '');
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._locale.title);

        $(this._contentAreaId).insert(
            '<form method="post" action="" enctype="application/x-www-form-urlencoded" id="' + this._id + '-form">' +
                '<div>' +
                    '<div class="form-row">&#160;</div>' +
                    '<div class="form-row">' +
                        '<div class="field-name">' +
                            '<label for="' + this._id + '-folder-name">' +
                                this._locale.folderName +
                            '</label>' +
                        '</div>' +
                        '<div class="field-value">' +
                            '<input name="folderName" id="' + this._id + '-folder-name" ' +
                                'value="" class="f-middle-size input-text" type="text">' +
                            '<span class="field-errors" style="display: none;"></span>' +
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</form>'
        );

        this.addOkButton();
        this.addCancelButton();

        var form = $(this._id + '-form');
        form._formSubmit = form.submit;
        form.submit = this._onSubmit.bind(this);
        form.observe('submit', this._onSubmitEvent.bind(this));

        this.reset();
    },

    reset: function() {
        this.setHint(
            this._locale.hint.replace(
                '%%folder%%', '<b>' + this._getCurrentFolder().escapeHTML() + '</b>'
            )
        );

        Smb.Views.FileSharing.clearStatusMessages();
        this._clearMessages();

        $(this._id + '-folder-name').value = '';

        try {
            $(this._id + '-folder-name').focus();
        } catch (e) {}
    },

    _getCurrentFolder: function() {
        var listControl = Jsw.getComponent('files-list');
        return listControl.getEscapedCurrentPathTitle();
    },

    addOkButton: function() {
        this.addRightButton(this._locale.buttonOk, this._onOkClick, true, true);
    },

    addCancelButton: function() {
        this.addRightButton(this._locale.buttonCancel, this._onCancelClick, false, false);
    },

    _onOkClick: function() {
        this._onSubmit();
    },

    _onSubmitEvent: function(event) {
        this._onSubmit();
        Event.stop(event);
        return false;
    },

    _onSubmit: function() {
        var folderName = $(this._id + '-folder-name').value;
        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters:
                {
                    'type': Smb.FileSharing.ListInstance.getStorageType(),
                    'folder': Smb.FileSharing.ListInstance.getCurrentFolder(),
                    'newFolderName': folderName
                },
                onCreate: this._onOkHandlerCreate.bind(this),
                onSuccess: (function(transport) {
                    this._onOkHandlerSuccess(transport, folderName);
                }).bind(this),
                onException: this._onOkHandlerException.bind(this),
                onComplete: this._onOkHandlerComplete.bind(this)
            }
        );
    },

    _onOkHandlerCreate: function(transport) {
        this.disable();
    },

    _onOkHandlerComplete: function(transport) {
        this.enable();
    },

    _onOkHandlerSuccess: function(transport, folderName) {
        this._clearMessages();
        try {
            var response = transport.responseText.evalJSON();
            if ('SUCCESS' == response.status) {
                Smb.FileSharing.ListInstance.reload();
                var message = response.message
                    .replace(
                        '%%folder%%',
                        '<a href="#" id="' + this._id + '-success-message-link">' +
                            folderName.escapeHTML() +
                        '</a>'
                    );
                Jsw.addStatusMessage('info', message);
                $(this._id + '-success-message-link').observe(
                    'click',
                    function(event) {
                        Event.stop(event);
                        Smb.FileSharing.ListInstance.changeFolder(
                            Smb.FileSharing.ListInstance.makeRelativePath(folderName)
                        );
                    }
                );
                this.hide();
            } else {
                this._addErrorMessage(response.message);
            }
        } catch (e) {
            this._addErrorMessage(
                'Internal error: ' + transport.responseText
            );
        }
    },

    _onOkHandlerException: function(transport, exception) {
        this._addErrorMessage('Internal error: ' + exception);
    },

    _onCancelClick: function() {
        this.hide();
    },

    disable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                context._okButtonTitle = sendButton.innerHTML;
                sendButton.update(
                    '<span class="wait">' + context._locale.waitButtonTitle + '</span>'
                );
            }
        );

        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = true;
                actionButtonWrapper.addClassName('disabled');
            }
        );
    },

    enable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                sendButton.update(context._okButtonTitle);
            }
        );
        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = false;
                actionButtonWrapper.removeClassName('disabled');
            }
        );
    }
});

Smb.FileSharing.Notification = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._selectUsersBoxId = this._id + '-select-users';
        this._expiredLinksBlockId = this._id + '-expired-links';
        this._linkTypeSharedId = this._id + '-link-shared';
        this._linkTypePublicId = this._id + '-link-public';
        this._linkExpirationTimeId = this._id + '-link-expiration-time';
        this._additionalAddressesId = this._id + '-additional-email';
        this._emailSubjectId = this._id + '-email-subject';
        this._emailBodyId = this._id + '-email-body';
        this._selectLinkTypeBlockId = this._id + '-link-type-block';

        this._handlerUrl = Jsw.prepareUrl('/file/email');
        this._embedded = this._getConfigParam('embedded', false);

        this._locale = this._getConfigParam('locale', {});
        this._title = this._getConfigParam('title', this._locale.title);

        this._storageType = this._getConfigParam('storageType', '');
        this._folder = this._getConfigParam('folder', '');

        this._onBack = this._getConfigParam('onBack', null);
        this._onOk = this._getConfigParam('onOk', null);
        this._onCancel = this._getConfigParam('onCancel', null);

        this._prependValidation = this._onOk ? true : false;

        this._getFilesHandler = this._getConfigParam('getFilesHandler', null);

        this._selected = $A();
        this._users = $H(Smb.FileSharing.ListInstance.getUsers());
        this._userContactName = Smb.FileSharing.ListInstance.getUserContactName();

        this._displayStatusMessages = this._getConfigParam('displayStatusMessages', false);
    },

    setTargetType: function(type) {
        this._storageType = type;
    },

    setTargetFolder: function(folder) {
        this._folder = folder;
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._title);
        this.setHint1(this._locale.hint);

        this._renderLinkTypeSelector();
        this._renderSelectUsersDialog();
        this._renderEmailFields();

        this.addBackButton();
        this.addOkButton();
        this.addCancelButton();

        this.reset();
    },

    _getSelectedFiles: function() {
        if (this._getFilesHandler) {
            return this._getFilesHandler();
        }

        var selected = $A();
        var ids = $A();
        Smb.FileSharing.ListInstance.getSelectedItemsIds().each(function(id) {
            ids.push(id);
        });

        this._folderSelected = false;
        Smb.FileSharing.ListInstance.getData().each((function(dataItem) {
            var itemIndex = ids.indexOf(dataItem.id);
            if (itemIndex < 0) {
                return;
            }

            if ('folder' == dataItem.type) {
                this._folderSelected = true;
            } else {
                selected.push(dataItem.name);
            }
        }).bind(this));

        if (selected.length < 1) {
            this.hide();
            Smb.Views.FileSharing.showWarning(this._locale.nothingSelected);
            return $A();
        }

        if (!this._embedded && this._folderSelected) {
            this.hide();
            Smb.Views.FileSharing.showWarning(this._locale.folderSelected);
            return $A();
        }

        return selected;
    },

    reset: function() {
        this._selected = this._getSelectedFiles();
        if (0 == this._selected.length) {
            return false;
        }

        $(this._emailSubjectId).value = this._getDefaultEmailSubject();
        $(this._emailBodyId).value = this._getDefaultEmailBody();

        this._updateLinkTypeSelectionBlock();

        return true;
    },

    _getDefaultEmailSubject: function() {
        var filename = this._selected.first();
        if (1 == this._selected.length) {
            return this._locale.emailSubjectOneFile.replace('%%file%%', filename.escapeHTML());
        }
        return this._locale.emailSubjectSeveralFiles.replace('%%file%%', filename.escapeHTML());
    },

    _getDefaultEmailBody: function() {
        var linkPlaceHolder = "<- " + this._locale.emailLinksPlaceHolderText + ' ->';
        return this._locale.emailBodyText.
            replace('%%user%%', this._userContactName).
            replace('%%links_placeholder%%', linkPlaceHolder);
    },

    _renderUsersList: function() {
        $(this._selectUsersBoxId).update('');
        this._users.each((function(pair) {
            var user = pair.value;
            var inputId = this._id + '-user-block-' + user['id']
            $(this._selectUsersBoxId).insert(
                '<label for="' + inputId + '">' +
                    '<span>' +
                        '<input id="' + inputId + '" type="checkbox" class="checkbox" value="' + user['id'] + '" /> ' +
                        user['contactName'] + ' &lt;' + user['email'] + '&gt;' +
                    '</span>' +
                '</label>'
            );
        }).bind(this));

        this._correctUsersListElementWidth();

        $(this._selectUsersBoxId).select('input').each(function(element) {
            element.observe('click', this._onSelectUsersItemClick.bindAsEventListener(this));
        }, this);
    },

    _correctUsersListElementWidth: function() { // CSS trick
        var maxWidth = 0;
        $(this._selectUsersBoxId).select('span').each(function(element) {
            maxWidth = [Element.getWidth(element), maxWidth].max();
        }, this);

        var innerElementPadding = 12;
        if (maxWidth + innerElementPadding < Element.getWidth($(this._selectUsersBoxId))) {
            return;
        }

        $(this._selectUsersBoxId).select('label').each(function(element) {
            element.setStyle({ 'width': maxWidth + 'px' });
        }, this);
    },

    _isLinkTypeSelectionBlockRequired: function() {
        if ('private' == this._storageType) {
            return false;
        }
        return true;
    },

    _isLinkExpirationBlockRequired: function() {
        if ('protected' == this._storageType || 'external' == this._storageType) {
            return false;
        }
        return true;
    },

    _updateLinkTypeSelectionBlock: function() {
        if (this._isLinkTypeSelectionBlockRequired()) {
            $(this._selectLinkTypeBlockId).update(
                '<div class="simple-row">' +
                    '<div class="field-value">' +
                        '<label><input type="radio" class="radio" name="linkType" id="' + this._linkTypeSharedId + '" checked="checked" /> ' + this._locale.linkTypeShared + '</label>' +
                    '</div>' +
                    '<div class="field-value">' +
                        '<div class="indent-box">' +
                            '<input type="radio" class="radio" name="linkType" id="' + this._linkTypePublicId + '" />' +
                            '<div class="indent-box-content">' +
                                '<label for="' + this._linkTypePublicId + '">' + this._locale.linkTypePublic + '</label>' +
                                '<div id="' + this._expiredLinksBlockId + '" class="expired-info" />' +
                            '</div>' +
                        '</div>' +
                    '</div>' +
                '</div>'
            );
            $(this._linkTypeSharedId).observe('click', this._onLinkSharedClick.bindAsEventListener(this));
            $(this._linkTypePublicId).observe('click', this._onLinkPublicClick.bindAsEventListener(this));
        } else {
            $(this._selectLinkTypeBlockId).update(
                '<div class="simple-row">' +
                    '<div class="field-value">' +
                        '<label>' + this._locale.linkTypePublic + '</label>' +
                        '<div id="' + this._expiredLinksBlockId + '" class="expired-info" />' +
                    '</div>' +
                '</div>'
            );
        }

        if (this._isLinkExpirationBlockRequired()) {
            $(this._expiredLinksBlockId).insert(
                this._locale.linkExpiredIn + ' ' +
                '<select id="' + this._linkExpirationTimeId + '" disabled="disabled">' +
                    '<option value="24">' + this._locale.linkExpiredOneDay + '</option>' +
                    '<option value="168" selected="selected">' + this._locale.linkExpiredOneWeek + '</option>' +
                    '<option value="720">' + this._locale.linkExpiredOneMonth + '</option>' +
                    '<option value="-1">' + this._locale.linkExpiredUnlimited + '</option>' +
                '</select>'
            );
        }

        if (!this._isLinkTypeSelectionBlockRequired()) {
            $(this._linkExpirationTimeId).disabled = false;
        }
    },

    _renderLinkTypeSelector: function() {
        $(this._contentAreaId).insert('<div class="form-row" id="' + this._selectLinkTypeBlockId + '" />');
    },

    _onLinkSharedClick: function(event) {
        if ($(this._linkExpirationTimeId))
            $(this._linkExpirationTimeId).disabled = true;
    },

    _onLinkPublicClick: function(event) {
        if ($(this._linkExpirationTimeId))
            $(this._linkExpirationTimeId).disabled = false;
    },

    _renderSelectUsersDialog: function() {
        $(this._contentAreaId).insert(
            '<hr class="hr" />' +
            '<div class="two-cols-block clearfix">' +
                '<div class="first-col">' +
                    '<div class="col-content">' +
                        '<div class="col-heading">' + this._locale.selectUsers + '</div>' +
                        '<div class="select-block" id="' + this._selectUsersBoxId + '" > </div>' +
                    '</div>' +
                '</div>' +
                '<div class="second-col">' +
                    '<div class="col-content">' +
                        '<div class="col-heading">' + this._locale.enterAdditionalAddresses + '</div>' +
                        '<textarea id="' + this._additionalAddressesId + '"></textarea>' +
                        '<div class="hint">' + this._locale.enterAdditionalAddressesHint + '</div>' +
                    '</div>' +
                '</div>' +
            '</div>'
        );

        this._renderUsersList();
    },

    _onSelectUsersItemClick: function(event) {
        var element = Event.element(event);
        if (element.checked)
            element.up('label').addClassName('selected');
        else
            element.up('label').removeClassName('selected');
    },

    _renderEmailFields: function() {
        $(this._contentAreaId).insert(
            '<div class="form-row">' +
                '<div class="simple-row">' +
                    '<div class="field-name">' + this._locale.emailSubject + '</div>' +
                    '<div class="field-value"><input id="' + this._emailSubjectId + '" type="text" class="input-text f-max-size" /></div>' +
                '</div>' +
            '</div>' +
            '<div class="form-row">' +
                '<div class="simple-row">' +
                    '<div class="field-name">' + this._locale.emailBody + '</div>' +
                    '<div class="field-value"><textarea id="' + this._emailBodyId + '" class="f-max-size"></textarea></div>' +
                '</div>' +
            '</div>'
        );
    },

    _onOk: function() {
        this._execute();
    },

    _execute: function() {
        var params = this.getRequestParams();

        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: params,
                onCreate: this._onExecuteCreate.bind(this),
                onSuccess: this._onExecute.bind(this),
                onException: (function(transport, exception) {
                    this._internalError(exception + "\n" + transport.responseText);
                }).bind(this),
                onComplete: this._onExecuteComplete.bind(this)
        });
    },

    getRequestParams: function() {
        var params = $H({
            linkExpirationTime: this._isLinkExpirationBlockRequired() ? $(this._linkExpirationTimeId).value : 0,
            emailSubject: $(this._emailSubjectId).value,
            emailBody: $(this._emailBodyId).value,
            additionalEmailAddresses: $(this._additionalAddressesId).value
        });

        if (this._isLinkTypeSelectionBlockRequired()) {
            params.set('linkType', $(this._linkTypePublicId).checked  ? 'public' : 'shared');
        } else {
            params.set('linkType', 'public');
        }

        params.set('targetType', this._storageType);
        params.set('targetFolder', this._folder);

        if (!this._embedded) {
            var number = 0;
            this._selected.each(function(item) {
                params.set('items[' + number++ + ']', item);
            });
        }

        this._selectedUsers = $A();
        $(this._selectUsersBoxId).select('input').each(function(element) {
            if (element.checked)
                this._selectedUsers.push(element.value);
        }, this);

        var number = 0;
        this._selectedUsers.each(function(item) {
            params.set('users[' + number++ + ']', item);
        });

        if (this._prependValidation) {
            params.set('validationOnly', 1);
        }

        return params;
    },

    _onExecute: function(transport) {
        try {
            var result = transport.responseText.evalJSON();
        } catch(exception) {
            this._internalError(exception + "\n" + transport.responseText);
            return;
        }

        switch (result.status) {
            case 'SUCCESS':
                if (this._prependValidation) {
                    this._prependValidation = false;
                    if (Object.isFunction(this._onOk)) {
                        this._onOk();
                    }
                } else {
                    this._close();
                    this._showSuccessMessage();
                }
                break;
            case 'ERROR':
                switch (result.data) {
                    case 'noRecipients':
                        this._addErrorMessage(this._locale.errorNoEmailAddresses);
                        break;
                    case 'badRecipients':
                        this._addErrorMessage(this._locale.errorWrongEmailAddresses);
                        $(this._additionalAddressesId).insert({
                            after: '<span class="error-hint">' + this._locale.hintWrongEmailAddresses + '</span>'
                        });
                        break;
                    default:
                        this._addErrorMessage(result.message);
                }
                break;
            default:
                this._internalError(
                    'Unknown response status: ' + result.status
                );
        }
    },

    _onExecuteCreate: function(transport) {
        this.disable();
    },

    _onExecuteComplete: function(transport) {
        this.enable();
    },

    disable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                context._okButtonTitle = sendButton.innerHTML;
                sendButton.update(
                    '<span class="wait">' + context._locale.waitButtonTitle + '</span>'
                );
            }
        );

        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = true;
                actionButtonWrapper.addClassName('disabled');
            }
        );
    },

    enable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(actionButtonWrapper) {
                var sendButton = actionButtonWrapper.down('button');
                sendButton.update(context._okButtonTitle);
            }
        );
        $(this._rightActionButtonsAreaId).select('.btn').each(
            function(actionButtonWrapper) {
                actionButtonWrapper.down('button').disabled = false;
                actionButtonWrapper.removeClassName('disabled');
            }
        );
    },

    _getFileUrl: function(file) {
        return Smb.FileSharing.getFileUrl(
            Smb.FileSharing.ListInstance.getStorageTypes().get(this._storageType).url,
            this._folder,
            file
        );
    },

    getSuccessMessage: function() {
        var message = '';

        if (1 == this._selected.length) {
            var fileName = this._selected.first();

            message = this._locale.statusOneFile.replace(
                '%%file%%',
                '<a href="' +
                        this._getFileUrl(fileName) +
                    '" target="_blank">' +
                    fileName.escapeHTML() +
                 '</a>'
            );
        } else {
            message = this._locale.statusFiles.replace(
                '%%number%%',
                this._selected.length
            );
        }
        return message;
    },

    _showSuccessMessage: function() {
        if (this._displayStatusMessages) {
            Smb.Views.FileSharing.clearStatusMessages();
            Jsw.addStatusMessage('info', this.getSuccessMessage());
        }
    },

    _close: function() {
        this.hide();
    },

    addBackButton: function() {
        if (Object.isFunction(this._onBack)) {
            this.addLeftButton(
                this._locale.buttonBack,
                this._onBack,
                false,
                true
            );
        }
    },

    addOkButton: function() {
        this.addRightButton(
            this._locale.buttonOk,
            this._onOkButtonClick.bind(this),
            true,
            true
        );
    },

    _clearMessages: function($super) {
        $super();
        var textElement = $(this._additionalAddressesId);
        if (!textElement) {
            return;
        }
        $(this._additionalAddressesId).up().select('.error-hint').each(function(element) {
            element.remove();
        });
    },

    _onOkButtonClick: function() {
        this._clearMessages();

        var params = this.getRequestParams();

        if (
            !params.get('additionalEmailAddresses') &&
            !(params.get('users[0]') && params.get('users[0]').strip())
        ) {
            this._addErrorMessage(this._locale.errorNoEmailAddresses);
            return;
        }

        if (this._prependValidation) {
            this._execute();
        } else if (Object.isFunction(this._onOk)) {
            this._onOk();
        } else {
            this._execute();
        }
    },

    addCancelButton: function() {
        this.addRightButton(
            this._locale.buttonCancel,
            this._onCancelButtonClick.bind(this),
            false,
            false
        );
    },

    _onCancelButtonClick: function() {
        if (Object.isFunction(this._onCancel)) {
            this._onCancel();
        } else {
            this.hide();
        }
    }
});

Smb.FileSharing.Wait = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._title = this._getConfigParam('title', '');
        this._message = this._getConfigParam('message', '');
        this._showButtons = false;
    },

    _getContentArea: function() {
        return '' +
            '<p class="loading-block">' +
                '<span class="ajax-loading">' +
                    this._message +
                '</span>' +
            '</p>'
    },

    render: function($super) {
        $super();

        this.setTitle(this._title);
    }
});

Smb.FileSharing.SelectDestination = Class.create(Smb.FileSharing.Popup, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._locale = this._getConfigParam('locale', {});
        this._foldersExplorerLocale = this._getConfigParam('foldersExplorerLocale', {});
        this._notificationLocale = this._getConfigParam('notificationLocale', {});
        this._confirmationDialoglocale = this._getConfigParam('confirmationDialogLocale', {});
        this._errorDialoglocale = this._getConfigParam('errorDialogLocale', {});

        this._handlerUrl = this._getConfigParam('handlerUrl', '');

        this._explorerBlockId = this._id + '-explorer-block';
        this._warningBlockId = this._id + '-warning-block';
        this._notificationCheckboxId = this._id + '-notification-checkbox';

        this._selected = $A();
        this._foldersNumber = 0;
        this._filesNumber = 0;
        this._notificationDialog = null;
        this._waitDialog = null;
        this._confirmDialog = null;
        this._errorDialog = null;
        this._explorer = null;
    },

    render: function($super) {
        $super();

        this.setBoxType('form-box');
        this.setTitle(this._locale.title);

        $(this._contentAreaId).insert(
            this._getContentHtml()
        );

        $(this._notificationCheckboxId).observe(
            'click',
            this._onNotificationClick.bindAsEventListener(this)
        );

        this.addOkButton();
        this.addCancelButton();

        this.reset();
        this._explorer = this._initExplorer();
    },

    _getContentHtml: function() {
        return '' +
            '<div>' +
                this._getAdditionalHtml() +
                '<div class="form-row" id="' + this._explorerBlockId + '">' +
                    this._getExplorerTitleHtml() +
                '</div>' +
                '<div class="form-row" style="display: none;" id="' + this._warningBlockId + '">' +
                    '<div class="single-row">' +
                        '<div class="hint-attention">' +
                        '</div>' +
                    '</div>' +
                '</div>' +
                '<div class="form-row">' +
                    '<div class="single-row">' +
                        '<label for="' + this._notificationCheckboxId + '">' +
                            '<input type="checkbox" class="checkbox" value="1" ' +
                                'id="' + this._notificationCheckboxId + '"/>' +
                            ' ' + this._locale.sendNotification +
                        '</label>' +
                    '</div>' +
                '</div>' +
            '</div>';
    },

    _getExplorerTitleHtml: function() {
        return '' +
            '<p>' +
                this._locale.selectDestinationFolder +
            '</p>';
    },

    _getAdditionalHtml: function() {
        return '';
    },

    _hideWarning: function() {
        $(this._warningBlockId).hide();
    },

    _showWarning: function(message) {
        $(this._warningBlockId).down('.hint-attention').update(message);
        $(this._warningBlockId).show();
    },

    _initExplorer: function() {
        return new Smb.FileSharing.Explorer({
            id: this._id + 'Explorer',
            renderTo: this._explorerBlockId,
            storageTabs: this._getStorageTabs(),
            storageTypes: this._getStorageTypes(),
            fileTypesIconsPathTemplate: Jsw.skinUrl + '/img/icons/small/file-%%type%%_16.gif',
            locale: this._foldersExplorerLocale,
            onSelectFolder: this._onSelectFolder.bind(this),
            onDeselectFolder: this._onDeselectFolder.bind(this)
        });
    },

    _onSelectFolder: function() {
        this.enable();
    },

    _onDeselectFolder: function() {
        this.disable();
    },

    _getStorageTabs: function() {
        var listControl = Jsw.getComponent('files-list');
        return listControl.getStorageTabs();
    },

    _getStorageTypes: function() {
        var listControl = Jsw.getComponent('files-list');
        return listControl.getStorageTypes();
    },

    _resetExplorer: function() {
        if (this._explorer) {
            this._explorer.reset();
        }
    },

    reset: function() {
        this._selected = $A();
        this._foldersNumber = 0;
        this._filesNumber = 0;
        $(this._notificationCheckboxId).checked = false;

        Smb.Views.FileSharing.hideWarning();

        this._resetExplorer();

        if (this._notificationDialog) {
            this._notificationDialog.remove();
            this._notificationDialog = null;
        }
        if (this._confirmDialog) {
            this._confirmDialog.remove();
            this._confirmDialog = null;
        }
        if (this._errorDialog) {
            this._errorDialog.remove();
            this._errorDialog = null;
        }
        $(this._rightActionButtonsAreaId).select('.btn.action').each((
            function(buttonWrapper) {
                var button = buttonWrapper.down('button');
                button.update(
                    this._locale.buttonOk
                );
            }
        ).bind(this));

        var listControl = Jsw.getComponent('files-list');

        var ids = $A();
        listControl.getSelectedItemsIds().each(function(id) {
            ids.push(id);
        });
        listControl.getData().each((function(dataItem) {
            var itemIndex = ids.indexOf(dataItem.id);
            if (itemIndex < 0) {
                return;
            }

            this._selected.push(dataItem);

            if ('folder' == dataItem.type) {
                this._foldersNumber++;
            } else {
                this._filesNumber++;
            }
        }).bind(this));

        if (1 > this._selected.length) {
            this._close();
            Smb.Views.FileSharing.showWarning(this._locale.nothingSelected);

            return false;
        }

        var hint = '';
        if (1 == this._selected.length) {
            hint = (0 == this._foldersNumber)
                ? this._locale.hintOneFile
                : this._locale.hintOneFolder;

            var fileName = this._selected.first().name;
            hint = hint.replace('%%file%%', '<b>' + fileName.escapeHTML() + '</b>');
        } else if (0 == this._foldersNumber) {
            hint = this._locale.hintFiles.replace('%%number%%', this._filesNumber);
        } else {
            if (0 == this._filesNumber) {
                hint = this._locale.hintFolders.replace('%%number%%', this._foldersNumber);
            } else {
                hint = this._locale.hintFilesFolders
                    .replace('%%foldersNumber%%', this._foldersNumber)
                    .replace('%%filesNumber%%', this._filesNumber);
            }
        }
        this.setHint1(hint);

        if (0 == this._filesNumber) {
            $(this._notificationCheckboxId).disable();
        } else {
            $(this._notificationCheckboxId).enable();
        }

        return true;
    },

    addCancelButton: function() {
        this.addRightButton(
            this._locale.buttonCancel,
            this._close.bind(this),
            false,
            false
        );
    },

    _close: function() {
        if (this._notificationDialog) {
            this._notificationDialog.remove();
            this._notificationDialog = null;
        }
        if (this._confirmDialog) {
            this._confirmDialog.remove();
            this._confirmDialog = null;
        }
        if (this._errorDialog) {
            this._errorDialog.remove();
            this._errorDialog = null;
        }
        if (this._waitDialog) {
            this._waitDialog.hide();
        }
        this.hide();
    },

    addOkButton: function() {
        this.addRightButton(
            this._locale.buttonOk,
            this._onOkClick.bind(this),
            true,
            true
        );
    },

    _onOkClick: function() {
        if ($(this._notificationCheckboxId).checked) {
            this.hide();
            if (this._notificationDialog) {
                this._notificationDialog.show();
            } else {
                this._notificationDialog = new Smb.FileSharing.Notification({
                    id: this._id + '-notification',
                    renderTo: this.getRenderTarget() + '-notification',
                    cls: 'popup-panel',
                    locale: this._notificationLocale,
                    titleCls: this._titleCls,
                    title: this._notificationLocale.specifyEmailParameters,
                    onBack: this._onNotificationBackClick.bind(this),
                    onOk: this._onNotificationOkClick.bind(this),
                    onCancel: this._onNotificationCancelClick.bind(this),
                    embedded: true
                });
            }
            this._notificationDialog.setTargetType(this._explorer.getType());
            this._notificationDialog.setTargetFolder(this._explorer.getFolder());
            this._notificationDialog.reset();
        } else {
            this._execute(false);
        }
    },

    _onNotificationClick: function(event) {
        var element = Event.element(event);

        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(buttonWrapper) {
                var button = buttonWrapper.down('button');
                button.update(
                    (element.checked)
                        ? context._locale.buttonNext
                        : context._locale.buttonOk
                );
            }
        );
    },

    _onNotificationBackClick: function() {
        this._notificationDialog.hide();
        this.show();
    },

    _onNotificationOkClick: function() {
        this._execute(false);
    },

    _onNotificationCancelClick: function() {
        this._close();
    },

    _execute: function(force) {
        var listControl = Jsw.getComponent('files-list');
        var params = $H({
            type: listControl.getStorageType(),
            folder: listControl.getCurrentFolder(),
            targetType: this._explorer.getType(),
            targetFolder: this._explorer.getFolder(),
            force: (force) ? 'true' : 'false'
        });

        var number = 0;
        this._selected.each(function(item) {
            params.set(
                'items[' + number++ + ']',
                item.id
            );
        });

        if ($(this._notificationCheckboxId).checked && this._notificationDialog) {
            params.set('sendNotification', 'true');
            params = params.merge(this._notificationDialog.getRequestParams());
        }

        new Ajax.Request(
            this._handlerUrl,
            {
                method: 'post',
                parameters: params,
                onCreate: this._showWaitDialog.bind(this),
                onSuccess: this._onExecute.bind(this),
                onException: (function(transport, exception) {
                    this._internalError(exception);
                }).bind(this)
            }
        );
    },

    _onExecute: function(transport) {
        try {
            var result = transport.responseText.evalJSON();
        } catch(exception) {
            this._internalError(
                exception + "\n" +
                transport.responseText
            );
            return;
        }

        switch (result.status) {
            case 'SUCCESS':
                this._showSuccessMessage();
                this._close();
                break;
            case 'CONFLICT':
                this._onExecuteConflict($A(result.data));
                break;
            case 'ERROR':
                if (result.data) {
                    this._onExecuteError(result.message, $H(result.data));
                } else {
                    this._userError(
                        result.message
                    );
                }
                break;
            default:
                this._internalError(
                    'Unknown response status: ' + result.status
                );
        }
    },

    _onExecuteError: function(errorMessage, items) {
        var errorItems = $A();

        this._selected.each((function(listItem) {
            var errorItem = Object.clone(listItem);
            items.each((function(pair) {
                if (listItem.id == pair.key) {
                    errorItem.status = pair.value;
                    throw $break;
                }
            }).bind(this));

            errorItems.push(errorItem);
        }).bind(this));

        var message = null;
        if (1 < errorItems.length) {
            message = this._locale.failed;
        }
        this._showErrorDialog(errorMessage, message, errorItems);
    },

    _showErrorDialog: function(errorMessage, message, items) {
        this.hide();
        if (this._notificationDialog) {
            this._notificationDialog.hide();
        }
        if (this._confirmDialog) {
            this._confirmDialog.hide();
        }
        if (this._waitDialog) {
            this._waitDialog.hide();
        }

        this._errorDialog = new Smb.FileSharing.ErrorDialog({
            id: this._id + '-error-dialog',
            renderTo: this.getRenderTarget() + '-error-dialog',
            errorMessage: errorMessage,
            message: message,
            locale: this._errorDialoglocale,
            items: items
        });
    },

    _showSuccessMessage: function() {
        var message = '';
        var listControl = Jsw.getComponent('files-list');

        if (1 == this._selected.length) {
            var fileName = this._selected.first().name;
            if (0 == this._foldersNumber) {
                var href = this._explorer.getFileUrl(fileName);
                message = this._locale.statusOneFile.replace(
                    '%%file%%',
                    '<a href="' + href + '" target="_blank">' +
                        fileName.escapeHTML() +
                    '</a>'
                );
            } else {
                message = this._locale.statusOneFolder.replace(
                    '%%file%%',
                    '<a href="#" id="' + this._id + '-folder-source-link">' +
                        fileName.escapeHTML() +
                    '</a>'
                );
            }
        } else if (0 == this._foldersNumber) {
            message = this._locale.statusFiles.replace(
                '%%number%%',
                this._filesNumber
            );
        } else {
            if (0 == this._filesNumber) {
                message = this._locale.statusFolders.replace(
                    '%%number%%',
                    this._foldersNumber
                );
            } else {
                message = this._locale.statusFilesFolders
                    .replace('%%foldersNumber%%', this._foldersNumber)
                    .replace('%%filesNumber%%', this._filesNumber);
            }
        }

        message = message.replace(
            '%%destination%%',
            '<a href="#" id="' + this._id + '-success-destination-link">' +
                this._explorer.getEscapedFolderTitle() +
            '</a>'
        );

        Smb.Views.FileSharing.clearStatusMessages();
        Jsw.addStatusMessage(
            'info',
            message
        );

        listControl.updateFolderLinkAction(
            $(this._id + '-success-destination-link'),
            this._explorer.getType(),
            this._explorer.getFolder()
        );

        if ($(this._id + '-folder-source-link')) {
            listControl.updateFolderLinkAction(
                $(this._id + '-folder-source-link'),
                this._explorer.getType(),
                this._explorer.makeRelativePath(this._selected.first().name)
            );
        }
    },

    _onExecuteConflict: function(items) {
        var conflictedItems = $A();
        var listControl = Jsw.getComponent('files-list');

        items.each((function(conflictedItem) {
            var listItem = null;
            listControl.getData().each((function(dataItem) {
                if (dataItem.id == conflictedItem.id) {
                    listItem = dataItem;
                    throw $break;
                }
            }).bind(this));
            if (!listItem) {
                return;
            }
            conflictedItems.push({
                newItem: listItem,
                existItem: conflictedItem
            });
        }).bind(this));

        this.hide();
        if (this._notificationDialog) {
            this._notificationDialog.hide();
        }
        if (this._errorDialog) {
            this._errorDialog.hide();
        }
        if (this._waitDialog) {
            this._waitDialog.hide();
        }

        this._confirmDialog = new Smb.FileSharing.ConfirmationDialog({
            id: this._id + '-confirmation',
            renderTo: this._id + '-confirmation-dialog-box',
            cls: 'popup-panel',
            locale: this._confirmationDialoglocale,
            destination: this._explorer.getEscapedPathTitle(),
            items: conflictedItems,
            overwriteHandler: this._onConflictOverwrite.bind(this),
            skipHandler: this._onConflictSkip.bind(this, conflictedItems),
            cancelHandler: this._close.bind(this)
        });
    },

    _onConflictOverwrite: function() {
        this._execute(true);
    },

    _onConflictSkip: function(items) {
        var selected = $A();
        this._foldersNumber = 0;
        this._filesNumber = 0;

        var listControl = Jsw.getComponent('files-list');
        this._selected.each((function(dataItem) {
            var listItem = null;
            items.each((function(conflictedItem) {
                if (dataItem.id == conflictedItem.existItem.id) {
                    listItem = dataItem;
                    throw $break;
                }
            }).bind(this));
            if (listItem) {
                return;
            }

            selected.push(dataItem);
            if ('folder' == dataItem.type) {
                this._foldersNumber++;
            } else {
                this._filesNumber++;
            }
        }).bind(this));

        this._selected = selected;

        if (1 > this._selected.length) {
            this._close();
            Smb.Views.FileSharing.showWarning(this._locale.nothingChanged);
            return;
        }

        this._execute(false);
    },

    _showWaitDialog: function() {
        this.hide();
        if (this._notificationDialog) {
            this._notificationDialog.hide();
        }
        if (this._confirmDialog) {
            this._confirmDialog.hide();
        }
        if (this._errorDialog) {
            this._errorDialog.hide();
        }

        if (this._waitDialog) {
            this._waitDialog.show();
            return;
        }
        this._waitDialog = new Smb.FileSharing.Wait({
            id: this._id + '-wait',
            renderTo: this.getRenderTarget() + '-wait',
            cls: 'popup-panel',
            title: this._locale.title,
            message: this._locale.waitMessage,
            titleCls: this._titleCls
        });
    },

    disable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(buttonWrapper) {
                buttonWrapper.down('button').disabled = true;
                buttonWrapper.addClassName('disabled');
            }
        );
    },

    enable: function() {
        var context = this;
        $(this._rightActionButtonsAreaId).select('.btn.action').each(
            function(buttonWrapper) {
                buttonWrapper.down('button').disabled = false;
                buttonWrapper.removeClassName('disabled');
            }
        );
    }
});

Smb.FileSharing.CheckDestination = Class.create(Smb.FileSharing.SelectDestination, {

    _onSelectFolder: function($super) {
        $super();

        this._hideWarning();

        var listControl = Jsw.getComponent('files-list');
        var storageType = this._explorer.getType();
        var folder = this._explorer.getFolder();

        if (('' == folder) && ('private' == storageType) && listControl.isBuiltIn()) {
            this.disable();
            this._showWarning(this._locale.forbidToPrivateRoot);
            return;
        }

        var destinationPath = storageType + '/' + folder;
        var sourcePath = listControl.getStorageType() + '/' + listControl.getCurrentFolder();

        if (sourcePath == destinationPath) {
            this.disable();
            this._showWarning(this._locale.forbidToSameFolder);
            return;
        }

        this._selected.each((function(listItem) {
            if ('folder' != listItem.type) {
                return;

            }
            var itemPath = sourcePath + '/' + listItem.name;
            if (destinationPath.startsWith(itemPath)) {
                this.disable();
                this._showWarning(
                    this._locale.forbidToChild
                        .replace('%%folder%%', listItem.name.escapeHTML())
                );
                return;
            }
        }).bind(this));
    },

    _onDeselectFolder: function($super) {
        $super();

        this._hideWarning();
    }
});

Smb.FileSharing.Copy = Class.create(Smb.FileSharing.CheckDestination, {

    _showSuccessMessage: function($super) {
        $super();

        var listControl = Jsw.getComponent('files-list');
        listControl.cleanSelection();
    }
});

Smb.FileSharing.Move = Class.create(Smb.FileSharing.CheckDestination, {

    _showSuccessMessage: function($super) {
        $super();

        var listControl = Jsw.getComponent('files-list');
        listControl.reload();
    }
});

Smb.FileSharing.Share = Class.create(Smb.FileSharing.SelectDestination, {

    _getExplorerTitleHtml: function() {
        return '';
    },

    _initExplorer: function() {
        return new Smb.FileSharing.PredefinedExplorer({
            id: this._id + 'Explorer',
            renderTo: this._explorerBlockId,
            storageTabs: this._getStorageTabs(),
            storageTypes: this._getStorageTypes(),
            type: this._getDefaultStorageType(),
            title: this._locale.selectDestinationFolder,
            fileTypesIconsPathTemplate: Jsw.skinUrl + '/img/icons/small/file-%%type%%_16.gif',
            onSelectFolder: this._onSelectFolder.bind(this),
            onDeselectFolder: this._onDeselectFolder.bind(this),
            locale: this._foldersExplorerLocale
        });
    },

    _getDefaultStorageType: function() {
        return 'shared';
    },

    _showSuccessMessage: function($super) {
        $super();

        this._reloadList();
    },

    _reloadList: function() {
        var listControl = Jsw.getComponent('files-list');
        listControl.reload();
    }
});

Smb.FileSharing.Publish = Class.create(Smb.FileSharing.Share, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._protectFilesCheckboxId = this._id + '-protect-files-checkbox';
    },

    reset: function($super) {
        var result = $super();

        if ($(this._protectFilesCheckboxId)) {
            $(this._protectFilesCheckboxId).checked = false;
        }
        if (this._explorer) {
            this._explorer.setType(this._getDefaultStorageType());
        }
        return result;
    },

    _getAdditionalHtml: function($super) {
        var result = $super();

        var listControl = Jsw.getComponent('files-list');
        var storageTypes = listControl.getStorageTypes();
        if (!storageTypes.get('protected')) {
            return result;
        }

        return result +
        '<div class="form-row">' +
            '<div class="single-row">' +
                '<label for="' + this._protectFilesCheckboxId + '">' +
                    '<input type="checkbox" class="checkbox" value="1" ' +
                        'id="' + this._protectFilesCheckboxId + '"/>' +
                    ' ' + this._locale.protectFiles +
                '</label>' +
            '</div>' +
        '</div>';
    },

    render: function($super) {
        $super();

        if ($(this._protectFilesCheckboxId)) {
            $(this._protectFilesCheckboxId).observe(
                'click',
                this._onProtectFilesCheckboxClick.bindAsEventListener(this)
            );
        }
    },

    _getDefaultStorageType: function() {
        return 'external';
    },

    _onProtectFilesCheckboxClick: function(event) {
        this._explorer.setType(
            ($(this._protectFilesCheckboxId).checked)
                ? 'protected'
                : 'external'
        );
    },

    _showSuccessMessage: function($super) {
        $super();

        var listControl = Jsw.getComponent('files-list');
        listControl.cleanSelection();
    },

    _reloadList: function($super) {
        //:INFO: reload is not required here
    }
});

Smb.FileSharing.PopupArea = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._toggleElementId = this._getConfigParam('toggleElementId', '');
        this._locale = this._getConfigParam('locale', {});
    },

    _initComponentElement: function($super) {
        $super();
        this._updateComponentElement(
            '<table class="popup-wrapper" cellspacing="0"><tbody><tr><td class="popup-container">' +
                '<div class="c1"><div class="c2"><div class="c3"><div class="c4"><div class="c5">' +
                    '<div class="popup-heading">' +
                        '<div class="heading-area">' +
                            '<span class="close" id="' + this._id + '-close"></span>' +
                            '<h4><span>' + this._getTitle() + '</span></h4>' +
                        '</div>' +
                    '</div>' +
                    '<div class="popup-content">' +
                        '<div class="popup-content-area">' + this._getContent() + '</div>' +
                    '</div>' +
                  '</div></div></div></div></div>' +
            '</td></tr></tbody></table>'
        );
    },

    _addEvents: function() {
        $(this._toggleElementId).observe(
            'click',
            this.toggle.bindAsEventListener(this)
        );
        $(this._id + '-close').observe(
            'click',
            this.hide.bindAsEventListener(this)
        );
    },

    hide: function(event) {
        if (event) {
            Event.stop(event);
        }
        this._componentElement.addClassName('collapsed');
    },

    toggle: function(event) {
        Event.stop(event);

        Smb.FileSharing.collapseExpandedWidgets(this._id);
        this._componentElement.toggleClassName('collapsed')
    }
});

Smb.FileSharing.LinkToFolderPopupArea = Class.create(Smb.FileSharing.PopupArea, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._webDavUrl = this._getConfigParam('webDavUrl', '');
        this._httpUrl = this._getConfigParam('httpUrl', '');
        this._webDavHowToUrl = Jsw.prepareUrl(this._getConfigParam('webDavHowToLink', ''));
        this._flashInstalled = DetectFlashVer('9', '0', '0');
        if (this._flashInstalled) {
            ZeroClipboard.setMoviePath(Jsw.baseUrl + '/externals/zeroclipboard/ZeroClipboard.swf');
        }
    },

    _getTitle: function() {
        return this._locale.linkToFolderAreaTitle;
    },

    _getContent: function() {
        var copyDavButton = '';
        if (this._flashInstalled) {
            copyDavButton = '<div class="popup-action">' +
                '<span class="btn" id="' + this._id +'-copyWebDavUrl"><button type="submit" value="">' +
                    this._locale.copyToClipboardButton +
                '</button></span>' +
            '</div>';
        }

        var content = '<p>' + this._locale.linkToWebDavFolderDescription +
            '<br/>' +
            '<a href="' + this._webDavHowToUrl + '" target="_blank">' +
                this._locale.webDavHowToLink +
            '</a>' +
        '</p>' +
        '<p><input class="input-text" value="' + this._webDavUrl + '" readonly="readonly" type="text"></p>' +
        copyDavButton;

        if (this._httpUrl) {
            copyHttpButton = '';
            if (this._flashInstalled) {
                copyHttpButton = '<div class="popup-action">' +
                    '<span class="btn" id="' + this._id +'-copyHttpUrl"><button type="submit" value="">' +
                        this._locale.copyToClipboardButton +
                    '</button></span>' +
                '</div>';
            }

            content += '<hr>' +
            '<p>' + this._locale.linkToHttpFolderDescription + '</p>' +
            '<p><input class="input-text" value="' + this._httpUrl + '" readonly="readonly" type="text"></p>' +
            copyHttpButton;
        }
        return content;
    },

    toggle: function($super, event) {
        $super(event);

        if (!this._flashInstalled) {
            return;
        }

        var hide = this._componentElement.hasClassName('collapsed')

        if (!this._davClip) {
            this._davClip = new ZeroClipboard.Client(this._id +'-copyWebDavUrl');
            if (Prototype.Browser.WebKit) {
                this._davClip.div.style.overflow = 'hidden';
            }
            this._davClip.setText(this._webDavUrl);

            Event.observe(
                window,
                'resize',
                function() {
                    this._davClip.reposition();
                }.bindAsEventListener(this)
            );

            this._davClip.addEventListener('onComplete', function() {
                if (!$(this._id +'-copyWebDavUrlHint')) {
                    var hint = '<div class="hint" id="' + this._id +'-copyWebDavUrlHint">' +
                        this._locale.linkCopiedHint +
                    '</div>';
                    $(this._id +'-copyWebDavUrl').insert({after: hint});
                    if (this._httpClip) {
                        this._httpClip.reposition();
                    }
                }
            }.bindAsEventListener(this));

        } else if (hide) {
            this._davClip.hide();
        } else {
            this._davClip.show();
        }

        if (this._httpUrl) {
            if (!this._httpClip) {
                this._httpClip = new ZeroClipboard.Client(this._id +'-copyHttpUrl');
                if (Prototype.Browser.WebKit) {
                    this._httpClip.div.style.overflow = 'hidden';
                }
                this._httpClip.setText(this._httpUrl);

                Event.observe(
                    window,
                    'resize',
                    function() {
                        this._httpClip.reposition();
                    }.bindAsEventListener(this)
                );

                this._httpClip.addEventListener('onComplete', function() {
                    if (!$(this._id +'-copyHttpUrlHint')) {
                        var hint = '<div class="hint" id="' + this._id +'-copyHttpUrlHint">' +
                            this._locale.linkCopiedHint +
                        '</div>';
                        $(this._id +'-copyHttpUrl').insert({after: hint});
                        this._davClip.reposition();
                    }
                }.bindAsEventListener(this));

            } else if (hide) {
                this._httpClip.hide();
            } else {
                this._httpClip.show();
            }
        }
    },

    destroy: function() {
        if (this._davClip) {
            this._davClip.destroy();
        }
        if (this._httpClip) {
            this._httpClip.destroy();
        }
    },

    render: function($super) {
        $super();
        this._tooltip = Jsw.Tooltip.init(this._toggleElementId, {text: this._locale.linkToFolderHint});
    }
});

Smb.FileSharing.AccessInfoPopup = Class.create(Smb.FileSharing.PopupArea, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._login = this._getConfigParam('login', '');
        this._password = this._getConfigParam('password', '');
        this._changePasswordLink = this._getConfigParam('changePasswordLink', null);
    },

    _getTitle: function() {
        return this._locale.popupHeader;
    },

    _getContent: function() {
        var content = '' +
            '<table cellspacing="0" class="access-info">' +
                '<tbody>' +
                    '<tr>' +
                        '<th>' +
                            this._locale.login +
                        '</th>' +
                        '<td>' +
                            this._login.escapeHTML() +
                        '</td>' +
                    '</tr>' +
                    '<tr>' +
                        '<th>' +
                            this._locale.password +
                        '</th>' +
                        '<td>' +
                            this._password.escapeHTML() +
                        '</td>' +
                    '</tr>' +
                '</tbody>' +
            '</table>';

        if (this._changePasswordLink) {
            content += '' +
                '<div class="popup-action">' +
                    '<a href="' + this._changePasswordLink.escapeHTML() + '">' +
                        this._locale.changePasswordLink +
                    '</a>' +
                '</div>';
        }

        return content;
    }
});

Smb.FileSharing.AccessInfo = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);

        this._locale = this._getConfigParam('locale', {});
        this._login = this._getConfigParam('login', '');
        this._password = this._getConfigParam('password', '');
        this._changePasswordLink = this._getConfigParam('changePasswordLink', null);
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<a id="' + this._id + '-show" href="#" class="show-access-info">' +
                this._locale.title +
            '</a>'
        );
    },

    render: function($super) {
        $super();

        var popup = new Smb.FileSharing.AccessInfoPopup({
            id: this._id + '-popup',
            toggleElementId: this._id + '-show',
            cls: 'popup-box collapsed',
            locale: this._locale,
            login: this._login,
            password: this._password,
            changePasswordLink: this._changePasswordLink
        });

        popup.setRenderTarget(this._id);
        popup.render();
    }
});