Jsw.namespace('Smb.Migration');
Smb.Migration.Progress = Class.create(Jsw.Component, {
    STATUS_WORKING: 'working',
    STATUS_STARTING: 'starting',    
    STATUS_FINISHED: 'finished',
    STATUS_STOPPED: 'stopped',
    STATUS_NONE: 'none',    

    updateInterval: 60,

    _initConfiguration: function($super, config) {
        $super(config);
        this._locale = this._getConfigParam('locale', {});
        this._downloadLogUrl = this._getConfigParam('downloadLogUrl', null);
        this._taskStatusUrl = this._getConfigParam('taskStatusUrl', null);
        this._taskStopUrl = this._getConfigParam('taskStopUrl', null);
    },

    _addEvents: function() {
        this._processStatus(this._getConfigParam('status', {}));
    },

    _sheduleUpdateStatus: function() {
        this._updateStatus.bind(this).delay(this.updateInterval);
    },

    _updateStatus: function() {
        new Ajax.Request(Jsw.prepareUrl(this._taskStatusUrl), {
          method: 'get',
          onSuccess: this._onSuccess.bind(this),
          onFailure: this._onFailure.bind(this)
        });
    },

    _onSuccess: function(transport) {
        var status = transport.responseText.evalJSON();
        this._processStatus(status);
    },

    _onFailure: function(transport) {
        var status = transport.responseText.evalJSON();
        this._processStatusFailure(status);
    },

    _processStatus: function(status) {
        if (!this._lastTaskStatus) {
            this._lastTaskStatus = status.taskStatus;
        } 
        if (status.taskStatus != this._lastTaskStatus) {
            Jsw.redirect('/migration');
        } else if (this.STATUS_FINISHED == status.taskStatus || this.STATUS_STOPPED == status.taskStatus) {
            this._showFinished(status);
        } else if (this.STATUS_WORKING == status.taskStatus || this.STATUS_STARTING == status.taskStatus) {
            this._showWorking(status);
            this._sheduleUpdateStatus();
        } else {
            this._showNone(status);
        }
    },

    _processStatusFailure: function(transport) {
        this._sheduleUpdateStatus();
    },

    _showNone: function(status) {
        this.removeElements();
    },
    
    _removeElements: function() {
        if (!this._componentElement) {
            return;
        }
        this._componentElement.childElements().each(Element.remove);
    },

    _enableMigrationButton: function(value){
        var createMigrationButton = Jsw.getComponent('create-migration');
        if (value) {
            createMigrationButton.enable();
            createMigrationButton.setTooltip(this._locale.hintButtonMigration);
        } else {
            createMigrationButton.disable();
            createMigrationButton.setTooltip(this._locale.hintButtonMigration);            
        }
    },
    
    _showFinished: function(status) {    
        this._enableMigrationButton(true);
        
        var content =           
            this._showCommonInfo(status) + 
            (status.migratedDataDescription == ''? '': this._showInfoRow(this._locale.migratedData, status.migratedDataDescription.escapeHTML()));
            
        this._updateFormElement(
            this._getSection(this._locale.lastMigrationDetails, content)
        );
    },
    
    _showWorking: function(status) {
        this._enableMigrationButton(false);
        
        var content = 
            this._showCommonInfo(status) +
            this._showProgress(status) +
            this._showButtons();
        
        this._updateFormElement(
                this._getSection(this._locale.currentMigrationDetails, content)
        );
        $("btn-send").observe('click',this._onStopButton.bind(this)); 
    },
    
    _onStopButton: function() {
        Jsw.messageBox.show({
            text: this._locale.stopConfirmation,
            description: '<p>' + this._locale.stopConfirmationHint + '</p><p>' + this._locale.stopQuestion + '</p>',
            buttonTitles: {yes: this._locale.stopYesButton, no: this._locale.stopNoButton},
            onYesClick: this._onStop.bind(this),
            subtype: 'confirm'
        });
        return false;
    },
    
    _getSection: function(name, content) {
        return '' + 
            '<div class="box-area">' +
                '<div class="title"><div class="title-area"><h3><span>' + name + '</span></h3></div></div>' +
                '<div class="content">' +
                    '<div class="content-area">' +
                        content +
                    '</div>' +    
                '</div>' +
            '</div>';
    },
    
    _updateFormElement: function(content) {
        this._componentElement.update(
            '<div class="form-box">' +
            content +
            '</div>');
    },
    
    _showProgress: function(status) {        
        var content = 
            this._showInfoRow(this._locale.elapsedTime, status.elapsedTime.escapeHTML()) +
            this._showInfoRow(this._locale.currentOperation, 
                   '<span class="ajax-loading">' + status.currentOperationDescription.escapeHTML() + '</span>') +
            this._showInfoRow(this._locale.overallProgress, this._showGuage(status));        
        return '' +
            this._getSection(this._locale.progress, content);
    },
    
    _showGuage: function(status) {
        progress = Math.floor((status.operationProgress/status.operationMaxProgress) * 100);
        return '' + 
            '<div class="migration-progress-indicator">' +
            '<span class="gauge-wrapper f6 not_empty" title="">' +
              '<span class="gauge-container">' +
                '<span class="gauge-filled" style="width: ' + progress + '%;"></span>' +
              '</span>' +
            '</span>' +
            status.currentOperationProgressDescription.escapeHTML() +
            '</div>';
    },
    
    _showButtons: function() {
        return '' +
            '<div class="btns-box">' +
                '<div class="box-area">' +
                    '<div class="form-row">' + 
                        '<div class="field-name"></div>' +
                        '<div class="field-value">' + 
                        '<span class="btn action" id="btn-send"><button type="button" value="" name="send">' + this._locale.stop + '</button></span>' + 
                        '</div>' +
                    '</div>' +
                '</div>' +
            '</div>';        
    },
    
    _showCommonInfo: function(status) {
        return '' +
            this._showInfoRow(this._locale.status, this._getStatusView(status)) +
            this._showLogDownloadLink(status) +
            this._showInfoRow(this._locale.sourceHost, status.sourceHost) +
            this._showInfoRow(this._locale.startDate, status.startDate);
    },
    
    _showLogDownloadLink: function(status) {
        if (status.taskResult == 'working' || !status.taskLogExists) {
            return '';
        } else {
            return '<div class="form-row">' +
                    '<div class="field-name"></div>' +
                    '<div class="field-value">' +
                        '<a id="downloadLink" href="' + Jsw.prepareUrl(this._downloadLogUrl) + '">' + 
                            this._locale.downloadTitle.escapeHTML() + 
                        '</a>' +
                    '</div>' +
                '</div>';
        }
    },
       
    _getStatusView: function(status) {
        var text;
        var cls;
        switch(status.taskResult) {
        case "error":
            text = this._locale.statusFailed;
            cls = "migration-status-failed";
            break;
        case "working":
            text = this._locale.statusWorking;
            cls = "migration-status-working";
            break;
        case "stopped":
            text = this._locale.statusStopped;
            cls = "migration-status-stopped";
            break;        
        case "warnings":
            text = this._locale.statusWarnings;
            cls = "migration-status-stopped";
            break;
        case "success":
            text = this._locale.statusSuccess;
            cls = "migration-status-success";
            break;           
        };
        return '<span class="' + cls + '">' + text + '</span>'; 
    },
    
    _showInfoRow: function(name, value) {
        return '' +
            '<div class="form-row">' +
                '<div class="field-name">' + name + '</div>' +
                '<div class="field-value"><div class="text-value">' + value + '</div></div>' +
            '</div>';
    },

    _onStop: function() {
        if (!this._componentElement) {
            return;
        }
        this._removeElements();
        new Ajax.Request(Jsw.prepareUrl(this._taskStopUrl), {
            method: 'post',
            onSuccess: this._onSuccess.bind(this),
            onFailure: this._onFailure.bind(this)
        });
    }
});