/* SMB specific JavaScript classes */

Jsw.namespace('Smb');

Smb.DynamicSubForm = Class.create(Jsw.DynamicSubForm, {
    _initConfiguration: function($super, config) {
        $super(config);
        this._addButtonId = this._getConfigParam('addButtonId', '');
        this._addButtonTitle = this._getConfigParam('addButtonTitle', '');
        this._removeButtonId = this._getConfigParam('removeButtonId', '');
        this._removeButtonTitle = this._getConfigParam('removeButtonTitle', '');
    },

    _onInitDynamic: function() {
        $(this._addButtonId).replace('<div id="' + this._addButtonId + '" class="text-value"></div>');
        new Jsw.SmallButton({
            id: 'dynamicSubFormContentAddButton-' + this._id,
            title: this._addButtonTitle,
            addCls: 'sb-item-add dynamicSubFormContentAddButton',
            handler: this.addContent.bind(this),
            disabled: this._isAddButtonDisabled(),
            renderTo: this._addButtonId
        });
    },

    _isAddButtonDisabled: function() {
        return false;
    },

    _bindDynamicContent: function(content, isFirstElement) {
        if (!isFirstElement) {
            var wrapperId = this._initRemoveButtonWrapper(content);
            new Jsw.SmallButton({
                id: wrapperId,
                title: this._removeButtonTitle,
                addCls: 'sb-item-remove dynamicSubFormContentRemoveButton',
                handler: this.deleteContent.bind(this, content),
                renderTo: wrapperId
            });
        }
        return content;
    },

    _initRemoveButtonWrapper: function(content) {
        var wrapperId = this._removeButtonId + '-' + this._getRandomNumber();
        content.select('.field-value input').last().insert({
            after: '<span id="' + wrapperId + '" class="' + this._removeButtonId + '"></span>'
        });
        return wrapperId;
    },

    _prepareDynamicContent : function(content) {
        var newId = Math.floor(1000000*Math.random());

        var nameRegExp = new RegExp("(name=.*\\[" + this._id + "\\]\\[)dynamicSubFormTemplate(\\])", "g");
        var idRegExp = new RegExp("(id=.*\\-" + this._id + "-)dynamicSubFormTemplate(-)", "g");

        content = content.update(content.innerHTML
            .replace(nameRegExp, '\$1c' + newId +'\$2')
            .replace(idRegExp, '\$1c' + newId +'\$2')
        );

        return content;
    },

    _onUpdateDynamic: function() {
        var childs = this._componentElement.select('.dynamicSubFormContent');
        if (0 < childs.size()) {
            childs.each(function(child, i) {
                if (0 == i) {
                    child.select('label').last().show();
                } else {
                    child.select('label').last().hide();
                }
            });
            $$('label[for="' + this._addButtonId + '"]').first().hide();
        } else {
            $$('label[for="' + this._addButtonId + '"]').first().show();
        }
    },

    addContent: function($super, event) {
        $super();
        event.stop();
    },

    deleteContent: function($super, content, event) {
        $super(content);
        event.stop();
    }
});

Smb.Webmail = {

    urlWrapper: function(webmailUrl) {
        var wrapper = 'window.open("' + webmailUrl + '", "webmail", ' +
            '"width=700,height=600,innerWidth=700,innerHeight=600,toolbar=yes,personalbar=no,' +
            'locationbar=no,statusbar=no,scrollbars=yes,resizable=yes");';

        return wrapper;
    }

}

Smb.ApsLicense = {

    buy: function(action, body) {
        var form = new Element('form');
        var input = new Element('input');

        form.hide();
        form.action = action;
        form.target = '_blank';
        form.method = 'post';
        form.enctype = 'application/x-www-form-urlencoded';

        input.type = 'hidden';
        input.name = 'data';
        input.value = body;

        form.insert(input);
        $(document.body).insert(form);
        form.submit();
    },

    initForm: function() {
        Event.observe($('apsLicense-uploadType-text'), 'click', this.onChangeUploadType);
        Event.observe($('apsLicense-uploadType-file'), 'click', this.onChangeUploadType);

        this.onChangeUploadType();
    },

    onChangeUploadType: function() {
        if ($('apsLicense-uploadType-text').checked) {
            $('apsLicense-uploadText').enable();
            $('apsLicense-uploadFile').disable();
        } else {
            $('apsLicense-uploadText').disable();
            $('apsLicense-uploadFile').enable();
        }
    }

};

Smb.GettingStartedPopup = Class.create({

    initialize: function(containerId) {
        this._container = $(containerId);

        if (!this._container) {
            return;
        }

        this._container.select('.getting-started-link').each((
            function(element) {
                element.observe('click', this._onOkClick.bindAsEventListener(this));
            }
        ).bind(this));
        this._container.select('.s-btn.action').each((
            function(element) {
                element.observe('click', this._onOkClick.bindAsEventListener(this));
            }
        ).bind(this));
        this._container.select('.s-btn').each((
            function(element) {
                if (element.hasClassName('action')) {
                    return;
                }
                element.observe('click', this._onCancelClick.bindAsEventListener(this));
            }
        ).bind(this));

        if ('true' != Jsw.Cookie.get('show-getting-started-popup')) {
            return;
        }
        if ('true' == Jsw.Cookie.get('do-not-show-getting-started-popup')) {
            return;
        }

        this._container.addClassName('active');
        $(document.body).insert({
            top: '<div class="main-disabled-block"></div>'
        });
    },

    _onOkClick: function(event) {
        Event.stop(event);

        this._close();
        this._openWizard();
    },

    _onCancelClick: function(event) {
        Event.stop(event);

        this._close();
    },

    _openWizard: function() {
        Jsw.redirect('/wizard');
    },

    _close: function() {
        this._container.removeClassName('active');

        $(document.body).select('.main-disabled-block').each(
            function(element) {
                element.remove();
            }
        );

        // use two cookies for backward compatibility
        Jsw.Cookie.set('show-getting-started-popup', 'false', null, Jsw.baseUrl + '/');
        Jsw.Cookie.setPermanent('do-not-show-getting-started-popup', 'true', Jsw.baseUrl + '/');
    }

});

Smb.NavigationTabs = Class.create(Jsw.bar.Tabs, {
    _renderItems: function($super) {
        $super();
        var contentAreaWidth = $(this._contentAreaId).getWidth();
        var items = $(this._contentAreaId).childElements();
        var itemsWidth = 0;

        items.each(function(item) {
            itemsWidth += item.getWidth() + 1;
        });

        if (itemsWidth > contentAreaWidth) {
            var delta = Math.ceil((itemsWidth - contentAreaWidth) / items.size());
            var paddingDelta = Math.ceil(delta / 2);

            var leftPadding = 10 - paddingDelta;
            if (leftPadding < 0) {
                leftPadding = 0;
            }

            var rightPadding = 15 - paddingDelta;
            if (rightPadding < 5) {
                rightPadding = 5;
            }

            items.each(function(item) {
                item.down('span').setStyle({
                    padding: '0 ' + rightPadding + 'px 0 ' + leftPadding + 'px'
                })
            });
        }
    }
});

//prevent AJAX request canceling on Escape button press in Firefox
$(document).observe('keydown', function(event) {
    if (Event.KEY_ESC == event.keyCode) {
        Event.stop(event);
    }
});

Jsw.namespace('Smb.UserPack');

Smb.UserPack.Promotion = Class.create(Jsw.Component, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._locale = this._getConfigParam('locale', {});
        this._buyUserPackLinkId = this._id + '-buy-userpack-link';
    },

    _initComponentElement: function($super) {
        $super();
        var gaugeClass = '';
        var value = this._getConfigParam('value', 0);
        if (100 <= value) {
            gaugeClass = 'over';
        } else if (0 < value) {
            gaugeClass = 'not_empty';
        }
        this._updateComponentElement(
            '<div class="info-box">' +
                '<table cellspacing="0"><tbody><tr><td>' +
                    '<div class="info-box-data">' +
                        this._locale.header +
                    '</div>' +
                    '<span title="" class="gauge-wrapper f5 ' + gaugeClass + '">' +
                        '<span class="gauge-container">' +
                            '<span style="width: ' +
                                (100 >= value ? value : 100) +
                                '%;" class="gauge-filled">' +
                            '</span>' +
                        '</span>' +
                    '</span>' +
                    this._getUserPackPromotionLink() +
                '</td></tr></tbody></table>' +
            '</div>'
        );
    },

    _getUserPackPromotionLink: function() {
        if (this._getConfigParam('buyUserPackUrl')) {
            return '' +
                '<div class="info-box-link">' +
                    '<a href="#" id="' + this._buyUserPackLinkId + '">' +
                        this._locale.link +
                    '</a>' +
                '</div>';
        } else {
            return '' + this._locale.contactToProvider;
        }
    },

    _addEvents: function($super) {
        $super();
        var userPackLink = $(this._buyUserPackLinkId);
        if (userPackLink) {
            userPackLink.observe('click', this._onBuyUserPackClick.bindAsEventListener(this));
        }
    },

    _onBuyUserPackClick: function(event) {
            Event.stop(event);
            Jsw.redirectPost(this._getConfigParam('buyUserPackUrl'), {
                    data: this._getConfigParam('buyUserPackData')
            }, '_blank');
    }

});

Smb.ToolButton = Class.create(Jsw.BigButton, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._cls = this._getConfigParam('cls', 'tool-block');
        this._disabledClass = this._getConfigParam('disabledClass', 'tool-block-disabled');
    },

    _initComponentElement: function() {
        this._componentElement = new Element('a', { 'class': this._cls }).insert(
            '<span class="tool-icon">' + '<img src="' + this._icon + '"></span>' +
            '<span class="tool-name">' + this._config['title'].escapeHTML() + '</span>' +
            '<span class="tool-info">' + this._config['comment'].escapeHTML() + '</span>'
        );

        this._initVisibility();
    }

});

Smb.SubscriptionDropdown = Class.create(Jsw.Container, {

    _initConfiguration: function($super, config) {
        $super(config);
        this._contentAreaId = this._id + '-content-area';
        this._cls = this._getConfigParam('cls', 'dropdown-box');
        this._onChange = this._getConfigParam('onChange', false);
    },

    _initComponentElement: function($super) {
        $super();

        this._updateComponentElement(
            '<div class="dropdown-heading">' +
                '<div class="heading-area">' +
                    '<em></em>' +
                    '<span>' + this._getConfigParam('title') + '</span>' +
                '</div>' +
            '</div>' +
            '<div class="dropdown-content">' +
                '<div class="dropdown-content-area">' +
                    '<table class="dropdown-wrapper" cellspacing="0">' +
                        '<tr>' +
                            '<td class="dropdown-container"><ul id="' + this._contentAreaId + '"></ul></td>' +
                        '</tr>' +
                    '</table>' +
                '</div>' +
            '</div>'
        );

        if (this._items.size() > 1) {
            this._componentElement.addClassName(this._cls);
        }
    },

    _addEvents: function($super) {
        $super();

        var boxElement = this._componentElement;

        Event.observe(boxElement, 'mouseover', function() {
            boxElement.addClassName('expanded');
        });

        Event.observe(boxElement, 'mouseout', function() {
            boxElement.removeClassName('expanded');
        });
    },

    _renderItem: function(item) {
        var dropdownItem = new Jsw.DropdownItem(item);

        var valueItem = new Element('a', { 'href': '#' }).update('<span>' + dropdownItem.title.escapeHTML() + '</span>');

        $(this._contentAreaId).insert(
            new Element('li').update(valueItem)
        );

        valueItem.observe('click', this._onChange.bindAsEventListener(this, dropdownItem));
    }

});
