<?php

require_once(dirname(__FILE__) . '/helpers/base.php');

/**
 * Locale export utility
 *
 */
class Lkit_Application_Export extends Lkit_Application_Abstract
{

    private $_outputDir;

    /**
     * Run application
     *
     */
    public function run()
    {
        $this->_log("Locale exporter.");

        $this->_initEnvironment();
        $this->_parseOptions();
        $this->_exportLocale();
    }

    /**
     * Parse command line options
     *
     */
    private function _parseOptions()
    {
        $options = new Zend_Console_Getopt(
            array(
                'code|c=s' => 'Locale code (for example, en-US).',
            )
        );

        $arguments = $_SERVER["argv"];

        if ((count($arguments) < 2) || !$options->code) {
            echo $options->getUsageMessage();
            exit(1);
        }

        // retrive and normalize locale code
        $this->_localeCode = $options->code;
        $this->_localeCode = str_replace('_', '-', $this->_localeCode);
        $this->_log("Locale code: $this->_localeCode");

        $this->_outputDir = "locale-$this->_localeCode";
        $this->_log("Output directory: " . getcwd() . DIRECTORY_SEPARATOR . $this->_outputDir);
    }

    /**
     * Export locale from products
     *
     */
    private function _exportLocale()
    {
        if (!is_dir($this->_outputDir)) {
            mkdir($this->_outputDir);
        }

        $this->_exportSmbLocale();
        $this->_log("Step 1 out of 3 is done.");
        $this->_exportPleskLocale();
        $this->_log("Step 2 out of 3 is done.");
        $this->_exportSiteEditorLocale();
        $this->_log("Step 3 out of 3 is done.");

        $this->_createLocalePackDescription();

        $this->_log("Locale data was exported to $this->_outputDir directory.");
    }

    /**
     * Export SMB locale
     *
     */
    private function _exportSmbLocale()
    {
        $outputDir = "$this->_outputDir/smb/";

        if (!is_dir($outputDir)) {
            mkdir($outputDir);
        }

        $converter = "$this->_productSmbDir/utils/locale/smb2plesk.php";
        $inputDir = "$this->_productSmbDir/resources/languages/$this->_localeCode";
        $this->_exec("$this->_phpBinary \"$converter\" -d \"$inputDir\" -o \"$outputDir\"");

        if (file_exists("$outputDir/locale.php")) {
            unlink("$outputDir/locale.php");
        }

        rename("$outputDir/$this->_localeCode.php", "$outputDir/locale.php");
    }

    /**
     * Export Plesk locale
     *
     */
    private function _exportPleskLocale()
    {
        $outputDir = "$this->_outputDir/panel/";

        if (!is_dir($outputDir)) {
            mkdir($outputDir);
        }

        $inputDir = "$this->_productBaseDir/admin/plib/locales/$this->_localeCode/";

        copy("{$inputDir}common_messages_{$this->_localeCode}.php", "{$outputDir}common_messages.php");
        copy("{$inputDir}conhelp_{$this->_localeCode}.php", "{$outputDir}conhelp.php");
        copy("{$inputDir}messages_{$this->_localeCode}.php", "{$outputDir}messages.php");

        if ($this->_isOsWindow()) {
            copy("{$inputDir}cli_messages_{$this->_localeCode}.php", "{$outputDir}cli_messages.php");
        }

        // remove locale codes from included files names in main locale file
        $mainLocaleFile = file_get_contents("{$outputDir}messages.php");
        $mainLocaleFile = preg_replace('/(@include_once\(\'[a-z_]+)(_[a-z]{2}-[A-Z]{2})(\.php)/m', '\1\3', $mainLocaleFile);
        file_put_contents("{$outputDir}messages.php", $mainLocaleFile);
    }

    /**
     * Export SiteEditor locale
     *
     */
    private function _exportSiteEditorLocale()
    {
        $outputDir = "$this->_outputDir/siteeditor/";

        if (!is_dir($outputDir)) {
            mkdir($outputDir);
        }

        $seLocaleCode = str_replace('-', '_', $this->_localeCode);

        $converter = dirname(__FILE__) . "/helpers/se_convert_locales.php";
        $inputDir = "$this->_productSiteEditorDir/resources/locale/$seLocaleCode/";
        $this->_exec("$this->_phpBinary \"$converter\" --mode=lion --inputdir=\"$inputDir\" > \"$outputDir/locale.xml\"");
    }

    /**
     * Create locale pack description and save it to info.xml file
     *
     */
    private function _createLocalePackDescription()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $root = $dom->createElement("locale");
        $dom->appendChild($root);

        $root->appendChild($dom->createElement('code', $this->_localeCode));

        $platform = $this->_isOsUnix() ? 'unix' : 'windows';
        $root->appendChild($dom->createElement('platform', $platform));

        $root->appendChild($dom->createElement('version', trim(file_get_contents("$this->_productBaseDir/version"))));

        $dom->save("$this->_outputDir/info.xml");
    }

}

$application = new Lkit_Application_Export();
$application->run();