<?php

/**
 * Base class for localization kit utilities
 *
 */
abstract class Lkit_Application_Abstract
{

    protected $_productBaseDir;
    protected $_productSmbDir;
    protected $_productSiteEditorDir;
    protected $_mysqlDir;
    protected $_localeCode;
    protected $_phpBinary;

    /**
     * Run application
     *
     */
    abstract public function run();

    /**
     * Init environment
     *
     */
    protected function _initEnvironment()
    {
        if ($this->_isOsUnix()) {
            $this->_phpBinary = $_ENV['_'];
            $this->_parseProductConfig();
        } else {
            $this->_phpBinary = "\"{$_ENV['plesk_bin']}\\php.exe\" -dauto_prepend_file= ";
            $this->_productBaseDir = $_ENV['plesk_dir'];
        }

        $this->_productSmbDir = $this->_productBaseDir . '/admin/smb/application';
        $this->_productSiteEditorDir = $this->_getSiteEditorDir();

        if (!is_dir($this->_productSmbDir)) {
            $this->_fatalError("The target product is not installed (directory $this->_productSmbDir is not found).");
        }

        require_once("$this->_productSmbDir/library/init.php");
    }

    /**
     * Log message
     *
     * @param string $message
     */
    protected function _log($message)
    {
        echo "$message\n";
    }

    /**
     * Report fatal error
     *
     * @param string $message
     */
    protected function _fatalError($message)
    {
        echo "Fatal error: $message\n";
        exit(1);
    }

    /**
     * Execute utility
     *
     * @param string $command
     */
    protected function _exec($command)
    {
        if ($this->_isOsWindow()) {
            // see http://bugs.php.net/bug.php?id=34671
            $command = '"' . $command . '"';
        }

        exec($command, $output, $returnCode);

        if (0 !== $returnCode) {
            $this->_fatalError(
                "Utility execution failed.\n"
                . "Command: $command\n"
                . "Return code: $returnCode\n"
                . "Output: \n" . join("\n", $output)
            );
        }
    }

    /**
     * Parse product's config and fill info about it
     *
     */
    private function _parseProductConfig()
    {
        $config = '/etc/psa/psa.conf';

        if (!file_exists($config)) {
            $this->_fatalError("The target product is not installed (configuration file $config is not found).");
        }

        $configData = file_get_contents($config);

        preg_match('/^PRODUCT_ROOT_D\s+(.+)/m', $configData, $matches);
        $this->_productBaseDir = $matches[1];

        preg_match('/^MYSQL_BIN_D\s+(.+)/m', $configData, $matches);
        $this->_mysqlDir = $matches[1];
    }

    /**
     * Get SiteEditor installation directory
     *
     * @return string
     */
    private function _getSiteEditorDir()
    {
        if ($this->_isOsUnix()) {
            return (is_dir('/opt/siteeditor'))
                ? '/opt/siteeditor'
                : '/usr/local/siteeditor';
        } else {
            return $this->_productBaseDir . '\\Siteeditor';
        }
    }

    /**
     * Check if OS is MS Windows
     *
     * @return bool
     */
    protected function _isOsWindow()
    {
        return 'WINNT' == PHP_OS;
    }

    /**
     * Check if OS is Unix
     *
     * @return bool
     */
    protected function _isOsUnix()
    {
        return !$this->_isOsWindow();
    }

}