<?php

$categoriesList = array();

$productInfo = array(
	'name' 	=>	 "Sitebuilder for Linux/Unix",
	'version' 	=>	"4.0.0"
);

$options = array(
	'mode'	=> null,
	'inputdir'	=> null,
	'inputfile' 	=> null,
	'outdir'	=> null
);

if(!isset($argc) || !isset($argv)) {
	die('No argc/argv support');
}

if($argc < 2) {
	echo "Usage:
	$argv[0] --mode=lion --inputdir=<dirname> #convert from locale to LION
	$argv[0] --mode=locale --inputfile=<lion_file> --outdir==<dirname> #convert form LION to locale\n";
	exit;
}

$cmdArgs = $argv;
array_shift($cmdArgs);
foreach($cmdArgs as $arg) {
	list($key, $value) = split('=', $arg );
	$key = str_replace('--','', $key);
	$options[trim($key)] = trim($value);
}

$mode = $options['mode'];
if($mode == null) {
	exit_error("Unknown mode: <empty>");
}

if( $mode == "lion") {
	if($options['inputdir'] == null) {
		exit_error("'--inputdir' parameter is empty.");
	}
	convert_locale_to_lion($options['inputdir']);
} elseif( $mode == "locale") {
	if($options['outdir'] == null) {
		exit_error("'--outdir' parameter is empty.");
	}
	if($options['inputfile'] == null) {
		exit_error("'--inputfile' is empty.");
	}
	convert_lion_to_locale($options['inputfile'], $options['outdir']);
} else {
	exit_error("Unknown mode: $mode");
}


exit(0);

function exit_error($error) {
	print "Error: $error\n";
	exit(1);
}

function process_dir_to_lion($lionDoc, $path) {
	$dir = opendir($path);
	 while (false !== ($file = readdir($dir))) {
		if ($file == "." || $file == ".." || $file == "CVS" || $file == ".svn") {
			continue;
		}
		$path_parts = pathinfo($file);
		$filepath = $path.'/'.$file;
		if(is_dir($filepath)) {
			process_dir_to_lion($lionDoc, $filepath);
		}
		if(is_file($filepath) && isset($path_parts['extension']) && $path_parts['extension'] == 'lng') {
			process_file_to_lion($lionDoc, $filepath);
		}
	}
}

function process_file_to_lion($lionDoc, $name) {
	global $options, $categoriesList;
	$count = 0;
	$list = parse_ini_file($name);
	$section = $lionDoc->createElement("section");
	$name =str_replace($options['inputdir'].'/', '', $name);
	$node = $lionDoc->createElement("name");
	$node->nodeValue = $name;
	$section->appendChild($node);
	$data = $lionDoc->createElement("data");
	$section->appendChild($data);
	foreach($list as $key=>$value) {
		$entry = $lionDoc->createElement("entry");
		$data->appendChild($entry);

		$node = $lionDoc->createElement("keyword");
		$node->nodeValue = $key;
		$entry->appendChild($node);

		$node = $lionDoc->createElement("phrase");
		$node->nodeValue = htmlspecialchars($value);
		$entry->appendChild($node);

		$node = $lionDoc->createElement("comment");
		$node->nodeValue = '';
		$entry->appendChild($node);
		$count++;
	}
	$lionDoc->documentElement->appendChild($section);
	$categoriesList[$name] = $count;
}

function create_infoXML_document($code, $valuesList) {
  $infoXML = new DOMDocument('1.0', 'utf-8');
  $root = $infoXML->createElement("locale");
  $root->setAttribute('code', $code);
  $root->setAttribute('version', '1.0');
  $infoXML->appendChild($root);

  foreach($valuesList as $key=>$value) {
	$node = $infoXML->createElement($key);
	$node->nodeValue = $value;
	$root->appendChild($node);
  }

  return $infoXML;
}

function convert_locale_to_lion($inputdir) {
	global $productInfo, $categoriesList;

	if(file_exists($inputdir.'/info.xml')) {
	  $infoXML = DOMDocument::load( $inputdir.'/info.xml' );
	} else {

	  $valuesList = array(
		  'native_name' => 'unknown',
		  'english_name' => 'unknown',
		  'decimal_point' => '.',
		  'monetary_unit' => 'unknown',
		  'currency_symbol_left' => 'unknown',
		  'currency_symbol_right' => '',
		  'date_format' => 'Y/m/d',
		  'time_format' => 'H:i:s',
		  'week_begin' => '1'
	  );
	  $infoXML = create_infoXML_document('unknown', $valuesList);
	}

	$lionDoc = new DOMDocument('1.0', 'utf-8');
	$root = $lionDoc->createElement("locale");
	$code = $infoXML->documentElement->getAttribute('code');
	$codeNode = $lionDoc->createElement("code");
	$codeNode->nodeValue = $code;
	$root->appendChild($codeNode);
	$info = $lionDoc->createElement("info");
	$infoList = $infoXML->documentElement->childNodes;
	foreach($infoList as $infoItem) {
	  if($infoItem instanceof DOMElement) {
  		$node = $lionDoc->createElement($infoItem->nodeName);
		$node->nodeValue = $infoItem->nodeValue;
		$info->appendChild($node);
	  }

	}
	$root->appendChild($info);
	$product=$lionDoc->createElement("product");
	$name = $lionDoc->createElement("name");
	$name->nodeValue = $productInfo['name'];
	$version = $lionDoc->createElement("version");
	$version->nodeValue = $productInfo['version'];
	$elem = $lionDoc->createElement("name");
	$product->appendChild($name);
	$product->appendChild($version);

	$root->appendChild($product);
	$lionDoc->appendChild($root);

	process_dir_to_lion($lionDoc, $inputdir);

	$lionDoc->formatOutput = 1;
	print $lionDoc->saveXML();

/*	print "<!--\n";
	print print_r($categoriesList,1);
	print "\nTotal categories = ".sizeof($categoriesList)."\n";
	$keysCount = 0;
	foreach ($categoriesList as $category=>$count) {
	  $keysCount += $count;
	}
	print "\nTotal keys = ".$keysCount."\n";
	print serialize($categoriesList);
	print "-->\n";
*/
}

function convert_lion_to_locale($inputfile, $outdir) {
	global $productInfo, $categoriesList;
	$lionDoc = DOMDocument::load( $inputfile );
	$product = $lionDoc->getElementsByTagName('product')->item(0);
	$name = $product->getElementsByTagName('name')->item(0);
	$version = $product->getElementsByTagName('version')->item(0);
	$code = $lionDoc->getElementsByTagName('code')->item(0);
	if($name->nodeValue != $productInfo['name']) {
		exit_error("Incorrect product: ".$name->nodeValue.", ".$version->textContent);
	}
	$info = $lionDoc->getElementsByTagName('info');
	if($info->length > 0) {
	  $infoList = $info->item(0)->childNodes;
	  $valuesList = array();
	  foreach($infoList as $infoItem) {
		if($infoItem instanceof DOMElement) {
		  $valuesList[(string)$infoItem->nodeName] = $infoItem->nodeValue;
		}

	  }
	  $infoXML = create_infoXML_document($code->nodeValue, $valuesList);
	  $infoXML->formatOutput = 1;
	  $infoXML->save($outdir.'/info.xml');
	}

	$sectionList = $lionDoc->getElementsByTagName('section');
	for($i=0; $i<$sectionList->length; $i++) {
		$section = $sectionList->item($i);
		$name = $section->getElementsByTagName('name')->item(0);
		$path_parts = pathinfo($name->nodeValue);
		$filepath = $path_parts['dirname']; //dirname($name->nodeValue);
		@mkdir($outdir."/".$filepath, 0777, true);
		$filepath = $outdir."/".$name->nodeValue;
//print $filepath. "\n";
		$data = $section->getElementsByTagName('data')->item(0);
		$entryList = $data->getElementsByTagName('entry');
		$fp =  fopen($filepath, "w+");
		$categoriesList[$name->nodeValue] = $entryList->length;
		for($j=0; $j<$entryList->length; $j++) {
			$entry = $entryList->item($j);
			$keyword = $entry->getElementsByTagName('keyword')->item(0);
			$phrase = $entry->getElementsByTagName('phrase')->item(0);
			$comment = $entry->getElementsByTagName('comment')->item(0);
			@fwrite($fp,  $keyword->nodeValue.' = "'.htmlspecialchars_decode($phrase->nodeValue)."\"\n");
		}
		@fclose($fp);

		$content = print_r($categoriesList,1);
		$content .= "\nTotal categories = ".sizeof($categoriesList)."\n";
		$keysCount = 0;
		foreach ($categoriesList as $category=>$count) {
		  $keysCount += $count;
		}
		$content .= "\nTotal keys = ".$keysCount."\n";
		$content .= serialize($categoriesList);

		file_put_contents($outdir.'/stat.txt', $content);
	}
}


?>

